@@ -5,9 +5,7 @@ from nautobot.core.exceptions import (
     AbortTransaction,
     CeleryWorkerNotRunningException,
     FilterSetFieldNotFound,
-    ViewConfigException,
 )
-from nautobot.core.runner.importer import ConfigurationError
 from nautobot.extras.secrets.exceptions import (
     SecretError,
     SecretParametersError,
@@ -16,6 +14,10 @@ from nautobot.extras.secrets.exceptions import (
 )
 
 
+class ConfigurationError(Exception):
+    """Deprecated - no longer used in Nautobot core."""
+
+
 __all__ = (
     "AbortTransaction",
     "CeleryWorkerNotRunningException",
@@ -27,5 +29,4 @@ __all__ = (
     "SecretValueNotFoundError",
     "SerializerNotFound",
     "ServiceUnavailable",
-    "ViewConfigException",
 )
