@@ -12,6 +12,7 @@ from nautobot.extras.jobs import (
     enqueue_job_hooks,
     FileVar,
     get_job,
+    get_jobs,
     IntegerVar,
     IPAddressVar,
     IPAddressWithMaskVar,
@@ -21,6 +22,7 @@ from nautobot.extras.jobs import (
     Job,
     JobButtonReceiver,
     JobHookReceiver,
+    JSONVar,
     MultiChoiceVar,
     MultiObjectVar,
     ObjectVar,
@@ -30,24 +32,20 @@ from nautobot.extras.jobs import (
     TextVar,
 )
 
-
 __all__ = (
     "BaseJob",
     "BooleanVar",
     "ChoiceVar",
     "DatabaseFileField",
     "DryRunVar",
-    "enqueue_job_hooks",
     "FileVar",
-    "get_job",
     "GitRepositoryDryRun",
     "GitRepositorySync",
-    "IntegerVar",
     "IPAddressVar",
     "IPAddressWithMaskVar",
     "IPNetworkVar",
-    "is_job",
-    "is_variable",
+    "IntegerVar",
+    "JSONVar",
     "Job",
     "JobButtonReceiver",
     "JobHookReceiver",
@@ -55,9 +53,14 @@ __all__ = (
     "MultiObjectVar",
     "NautobotKombuJSONEncoder",
     "ObjectVar",
-    "register_jobs",
     "RunJobTaskFailed",
     "ScriptVariable",
     "StringVar",
     "TextVar",
+    "enqueue_job_hooks",
+    "get_job",
+    "get_jobs",
+    "is_job",
+    "is_variable",
+    "register_jobs",
 )
