@@ -1,5 +1,7 @@
 from django import forms
 
+from nautobot.cloud.models import CloudNetwork
+from nautobot.core.constants import CHARFIELD_MAX_LENGTH
 from nautobot.core.forms import (
     CommentField,
     DatePicker,
@@ -9,12 +11,13 @@ from nautobot.core.forms import (
     TagFilterField,
 )
 from nautobot.dcim.form_mixins import (
+    LocatableModelBulkEditFormMixin,
     LocatableModelFilterFormMixin,
     LocatableModelFormMixin,
 )
 from nautobot.extras.forms import (
-    NautobotFilterForm,
     NautobotBulkEditForm,
+    NautobotFilterForm,
     NautobotModelForm,
     StatusModelBulkEditFormMixin,
     StatusModelFilterFormMixin,
@@ -22,8 +25,8 @@ from nautobot.extras.forms import (
 )
 from nautobot.tenancy.forms import TenancyFilterForm, TenancyForm
 from nautobot.tenancy.models import Tenant
-from .models import Circuit, CircuitTermination, CircuitType, Provider, ProviderNetwork
 
+from .models import Circuit, CircuitTermination, CircuitType, Provider, ProviderNetwork
 
 #
 # Providers
@@ -61,7 +64,7 @@ class ProviderForm(NautobotModelForm):
 class ProviderBulkEditForm(TagsBulkEditFormMixin, NautobotBulkEditForm):
     pk = forms.ModelMultipleChoiceField(queryset=Provider.objects.all(), widget=forms.MultipleHiddenInput)
     asn = forms.IntegerField(required=False, label="ASN")
-    account = forms.CharField(max_length=100, required=False, label="Account number")
+    account = forms.CharField(max_length=CHARFIELD_MAX_LENGTH, required=False, label="Account number")
     portal_url = forms.URLField(required=False, label="Portal")
     noc_contact = forms.CharField(required=False, widget=SmallTextarea, label="NOC contact")
     admin_contact = forms.CharField(required=False, widget=SmallTextarea, label="Admin contact")
@@ -110,7 +113,7 @@ class ProviderNetworkForm(NautobotModelForm):
 class ProviderNetworkBulkEditForm(TagsBulkEditFormMixin, NautobotBulkEditForm):
     pk = forms.ModelMultipleChoiceField(queryset=ProviderNetwork.objects.all(), widget=forms.MultipleHiddenInput)
     provider = DynamicModelChoiceField(queryset=Provider.objects.all(), required=False)
-    description = forms.CharField(max_length=100, required=False)
+    description = forms.CharField(max_length=CHARFIELD_MAX_LENGTH, required=False)
     comments = CommentField(widget=SmallTextarea, label="Comments")
 
     class Meta:
@@ -133,6 +136,12 @@ class ProviderNetworkFilterForm(NautobotFilterForm):
 #
 # Circuit types
 #
+class CircuitTypeBulkEditForm(NautobotBulkEditForm):
+    pk = forms.ModelMultipleChoiceField(queryset=CircuitType.objects.all(), widget=forms.MultipleHiddenInput)
+    description = forms.CharField(max_length=CHARFIELD_MAX_LENGTH, required=False)
+
+    class Meta:
+        nullable_fields = ["description"]
 
 
 class CircuitTypeForm(NautobotModelForm):
@@ -144,6 +153,12 @@ class CircuitTypeForm(NautobotModelForm):
         ]
 
 
+class CircuitTypeFilterForm(NautobotFilterForm):
+    model = CircuitType
+    q = forms.CharField(required=False, label="Search")
+    name = forms.CharField(required=False)
+
+
 #
 # Circuits
 #
@@ -184,7 +199,7 @@ class CircuitBulkEditForm(TagsBulkEditFormMixin, StatusModelBulkEditFormMixin, N
     provider = DynamicModelChoiceField(queryset=Provider.objects.all(), required=False)
     tenant = DynamicModelChoiceField(queryset=Tenant.objects.all(), required=False)
     commit_rate = forms.IntegerField(required=False, label="Commit rate (Kbps)")
-    description = forms.CharField(max_length=100, required=False)
+    description = forms.CharField(max_length=CHARFIELD_MAX_LENGTH, required=False)
     comments = CommentField(widget=SmallTextarea, label="Comments")
 
     class Meta:
@@ -208,6 +223,7 @@ class CircuitFilterForm(
         "circuit_type",
         "provider",
         "provider_network",
+        "cloud_network",
         "status",
         "location",
         "tenant_group",
@@ -226,6 +242,12 @@ class CircuitFilterForm(
         to_field_name="name",
         label="Provider Network",
     )
+    cloud_network = DynamicModelMultipleChoiceField(
+        queryset=CloudNetwork.objects.all(),
+        required=False,
+        to_field_name="name",
+        label="Cloud Network",
+    )
     commit_rate = forms.IntegerField(required=False, min_value=0, label="Commit rate (Kbps)")
     tags = TagFilterField(model)
 
@@ -235,10 +257,38 @@ class CircuitFilterForm(
 #
 
 
+class CircuitTerminationBulkEditForm(TagsBulkEditFormMixin, LocatableModelBulkEditFormMixin, NautobotBulkEditForm):
+    pk = forms.ModelMultipleChoiceField(
+        queryset=CircuitTermination.objects.all(),
+        widget=forms.MultipleHiddenInput,
+    )
+    provider_network = DynamicModelChoiceField(queryset=ProviderNetwork.objects.all(), required=False)
+    cloud_network = DynamicModelChoiceField(queryset=CloudNetwork.objects.all(), required=False)
+    port_speed = forms.IntegerField(required=False, min_value=0, label="Port Speed (Kbps)")
+    upstream_speed = forms.IntegerField(required=False, min_value=0, label="Upstream Speed (Kbps)")
+    xconnect_id = forms.CharField(max_length=CHARFIELD_MAX_LENGTH, required=False, label="Cross-connect ID")
+    pp_info = forms.CharField(max_length=CHARFIELD_MAX_LENGTH, required=False, label="Patch Panel/Port(s)")
+    description = forms.CharField(max_length=CHARFIELD_MAX_LENGTH, required=False)
+
+    class Meta:
+        model = CircuitTermination
+        nullable_fields = [
+            "location",
+            "provider_network",
+            "cloud_network",
+            "port_speed",
+            "upstream_speed",
+            "xconnect_id",
+            "pp_info",
+            "description",
+        ]
+
+
 class CircuitTerminationForm(LocatableModelFormMixin, NautobotModelForm):
     provider_network = DynamicModelChoiceField(
         queryset=ProviderNetwork.objects.all(), required=False, label="Provider Network"
     )
+    cloud_network = DynamicModelChoiceField(queryset=CloudNetwork.objects.all(), required=False, label="Cloud Network")
 
     class Meta:
         model = CircuitTermination
@@ -246,6 +296,7 @@ class CircuitTerminationForm(LocatableModelFormMixin, NautobotModelForm):
             "term_side",
             "location",
             "provider_network",
+            "cloud_network",
             "port_speed",
             "upstream_speed",
             "xconnect_id",
@@ -261,3 +312,15 @@ class CircuitTerminationForm(LocatableModelFormMixin, NautobotModelForm):
         widgets = {
             "term_side": forms.HiddenInput(),
         }
+
+
+class CircuitTerminationFilterForm(LocatableModelFilterFormMixin, NautobotFilterForm):
+    model = CircuitTermination
+    q = forms.CharField(required=False, label="Search")
+    circuit = DynamicModelMultipleChoiceField(queryset=Circuit.objects.all(), to_field_name="cid", required=False)
+    provider_network = DynamicModelMultipleChoiceField(
+        queryset=ProviderNetwork.objects.all(), to_field_name="name", required=False
+    )
+    cloud_network = DynamicModelMultipleChoiceField(
+        queryset=CloudNetwork.objects.all(), to_field_name="name", required=False
+    )
