@@ -1,48 +1,56 @@
-import itertools
-import logging
-import platform
 from collections import OrderedDict
-import re
+import logging
+import os
+import platform
 
 from django import __version__ as DJANGO_VERSION, forms
 from django.apps import apps
 from django.conf import settings
-from django.core.exceptions import ObjectDoesNotExist, ValidationError
-from django.http.response import HttpResponseBadRequest
+from django.contrib.contenttypes.models import ContentType
+from django.core.cache import cache
+from django.core.exceptions import FieldDoesNotExist, ObjectDoesNotExist, ValidationError
 from django.db import transaction
 from django.db.models import ProtectedError
+from django.db.models.fields.related import ForeignKey, ManyToManyField, RelatedField
+from django.db.models.fields.reverse_related import ManyToManyRel, ManyToOneRel
+from django.http.response import HttpResponseBadRequest
 from django.shortcuts import get_object_or_404, redirect
-from django.urls import NoReverseMatch, reverse as django_reverse
-from rest_framework import status
-from rest_framework.response import Response
-from rest_framework.reverse import reverse
-from rest_framework.views import APIView
-from rest_framework.viewsets import ModelViewSet as ModelViewSet_
-from rest_framework.viewsets import ReadOnlyModelViewSet as ReadOnlyModelViewSet_
-from rest_framework.permissions import AllowAny, IsAuthenticated
-from rest_framework.exceptions import PermissionDenied, ParseError
+from django.utils.decorators import method_decorator
+from django.views.decorators.gzip import gzip_page
 from drf_spectacular.plumbing import get_relative_url, set_query_parameters
 from drf_spectacular.renderers import OpenApiJsonRenderer
 from drf_spectacular.utils import extend_schema
-from drf_spectacular.views import SpectacularSwaggerView, SpectacularRedocView
-
+from drf_spectacular.views import SpectacularAPIView, SpectacularRedocView, SpectacularSwaggerView
+from graphene_django.settings import graphene_settings
+from graphene_django.views import GraphQLView, HttpError, instantiate_middleware
 from graphql import get_default_backend
 from graphql.execution import ExecutionResult
-from graphql.type.schema import GraphQLSchema
 from graphql.execution.middleware import MiddlewareManager
-from graphene_django.settings import graphene_settings
-from graphene_django.views import GraphQLView, instantiate_middleware, HttpError
+from graphql.type.schema import GraphQLSchema
+import redis.exceptions
+from rest_framework import routers, serializers as drf_serializers, status
+from rest_framework.exceptions import APIException, ParseError, PermissionDenied
+from rest_framework.generics import GenericAPIView
+from rest_framework.permissions import IsAuthenticated
+from rest_framework.response import Response
+from rest_framework.reverse import reverse
+from rest_framework.views import APIView
+from rest_framework.viewsets import ModelViewSet as ModelViewSet_, ReadOnlyModelViewSet as ReadOnlyModelViewSet_
+import yaml
 
 from nautobot.core.api import BulkOperationSerializer
+from nautobot.core.api.exceptions import SerializerNotFound
+from nautobot.core.api.utils import get_serializer_for_model
 from nautobot.core.celery import app as celery_app
 from nautobot.core.exceptions import FilterSetFieldNotFound
-from nautobot.core.utils.config import get_settings_or_config
-from nautobot.core.utils.data import is_uuid
+from nautobot.core.models.fields import TagsField
+from nautobot.core.utils.data import is_uuid, render_jinja2
 from nautobot.core.utils.filtering import get_all_lookup_expr_for_field, get_filterset_parameter_form_field
-from nautobot.core.utils.lookup import get_form_for_model, get_route_for_model
-from nautobot.core.utils.permissions import get_permission_for_model
+from nautobot.core.utils.lookup import get_form_for_model
+from nautobot.core.utils.querysets import maybe_prefetch_related, maybe_select_related
 from nautobot.core.utils.requests import ensure_content_type_and_field_name_in_query_params
-from nautobot.extras.registry import registry
+from nautobot.core.views.utils import get_csv_form_fields_from_serializer_class
+
 from . import serializers
 
 HTTP_ACTIONS = {
@@ -55,6 +63,10 @@ HTTP_ACTIONS = {
     "DELETE": "delete",
 }
 
+
+logger = logging.getLogger(__name__)
+
+
 #
 # Mixins
 #
@@ -179,10 +191,11 @@ class ModelViewSetMixin:
         queryset = self.filter_queryset(self.get_queryset())
         lookup_url_kwarg = self.lookup_url_kwarg or self.lookup_field
 
-        assert lookup_url_kwarg in self.kwargs, (
-            f"Expected view {self.__class__.__name__} to be called with a URL keyword argument named "
-            f'"{lookup_url_kwarg}". Fix your URL conf, or set the `.lookup_field` attribute on the view correctly.'
-        )
+        if lookup_url_kwarg not in self.kwargs:
+            raise RuntimeError(
+                f"Expected view {self.__class__.__name__} to be called with a URL keyword argument named "
+                f'"{lookup_url_kwarg}". Fix your URL conf, or set the `.lookup_field` attribute on the view correctly.'
+            )
 
         if lookup_url_kwarg == "pk" and hasattr(queryset.model, "composite_key"):
             # Support lookup by either PK (UUID) or composite_key
@@ -226,6 +239,53 @@ class ModelViewSetMixin:
 
         return context
 
+    def get_queryset(self):
+        """
+        Attempt to optimize the queryset based on the fields present in the associated serializer.
+
+        See similar logic in nautobot.core.tables.BaseTable.
+        """
+        queryset = super().get_queryset()
+        model = queryset.model
+        serializer = self.get_serializer()
+
+        select_fields = []
+        prefetch_fields = []
+
+        for field_instance in serializer.fields.values():
+            if field_instance.write_only:
+                continue
+            if field_instance.source == "*":
+                continue
+            if "." in field_instance.source:
+                # DRF uses `field.nested_field` instead of `field__nested_field`
+                # TODO: We don't currently attempt to optimize nested lookups.
+                continue
+            if isinstance(field_instance, (drf_serializers.ManyRelatedField, drf_serializers.ListSerializer)):
+                # ListSerializer with depth > 0, ManyRelatedField with depth 0
+                try:
+                    model_field = model._meta.get_field(field_instance.source)
+                except FieldDoesNotExist:
+                    continue
+                if isinstance(model_field, (ManyToManyField, ManyToManyRel, RelatedField, ManyToOneRel, TagsField)):
+                    prefetch_fields.append(field_instance.source)
+            elif isinstance(field_instance, (drf_serializers.RelatedField, drf_serializers.Serializer)):
+                # Serializer with depth > 0, RelatedField with depth 0
+                try:
+                    model_field = model._meta.get_field(field_instance.source)
+                except FieldDoesNotExist:
+                    continue
+                if isinstance(model_field, ForeignKey):
+                    select_fields.append(field_instance.source)
+
+        if select_fields:
+            queryset = maybe_select_related(queryset, select_fields)
+
+        if prefetch_fields:
+            queryset = maybe_prefetch_related(queryset, prefetch_fields)
+
+        return queryset
+
     def restrict_queryset(self, request, *args, **kwargs):
         """
         Restrict the view's queryset to allow only the permitted objects for the given request.
@@ -343,25 +403,43 @@ class ReadOnlyModelViewSet(NautobotAPIVersionMixin, ModelViewSetMixin, ReadOnlyM
 #
 
 
-class APIRootView(NautobotAPIVersionMixin, APIView):
+class AuthenticatedAPIRootView(NautobotAPIVersionMixin, routers.APIRootView):
     """
-    This is the root of the REST API. API endpoints are arranged by app and model name; e.g. `/api/dcim/locations/`.
+    Extends DRF's base APIRootView class to enforce user authentication.
     """
 
-    _ignore_model_permissions = True
+    permission_classes = [IsAuthenticated]
 
-    def get_view_name(self):
-        return "API Root"
+    name = None
+    description = None
+
+
+class APIRootView(AuthenticatedAPIRootView):
+    """
+    This is the root of the REST API.
+
+    API endpoints are arranged by app and model name; e.g. `/api/dcim/locations/`.
+    """
+
+    name = "API Root"
 
     @extend_schema(exclude=True)
-    def get(self, request, format=None):  # pylint: disable=redefined-builtin
+    def get(self, request, *args, format=None, **kwargs):  # pylint: disable=redefined-builtin
         return Response(
             OrderedDict(
                 (
+                    (
+                        "apps",
+                        reverse("apps-api:api-root", request=request, format=format),
+                    ),
                     (
                         "circuits",
                         reverse("circuits-api:api-root", request=request, format=format),
                     ),
+                    (
+                        "cloud",
+                        reverse("cloud-api:api-root", request=request, format=format),
+                    ),
                     (
                         "dcim",
                         reverse("dcim-api:api-root", request=request, format=format),
@@ -396,6 +474,14 @@ class APIRootView(NautobotAPIVersionMixin, APIView):
                             format=format,
                         ),
                     ),
+                    (
+                        "wireless",
+                        reverse(
+                            "wireless-api:api-root",
+                            request=request,
+                            format=format,
+                        ),
+                    ),
                 )
             )
         )
@@ -416,7 +502,8 @@ class StatusView(NautobotAPIVersionMixin, APIView):
                     "django-version": {"type": "string"},
                     "installed-apps": {"type": "object"},
                     "nautobot-version": {"type": "string"},
-                    "plugins": {"type": "object"},
+                    "nautobot-apps": {"type": "object"},
+                    "plugins": {"type": "object"},  # 3.0 TODO: remove this
                     "python-version": {"type": "string"},
                     "celery-workers-running": {"type": "integer"},
                 },
@@ -432,19 +519,30 @@ class StatusView(NautobotAPIVersionMixin, APIView):
             if version:
                 if isinstance(version, tuple):
                     version = ".".join(str(n) for n in version)
+                else:
+                    version = str(version)
             installed_apps[app_config.name] = version
         installed_apps = dict(sorted(installed_apps.items()))
 
-        # Gather installed plugins
-        plugins = {}
-        for plugin_name in settings.PLUGINS:
-            plugin_name = plugin_name.rsplit(".", 1)[-1]
-            plugin_config = apps.get_app_config(plugin_name)
-            plugins[plugin_name] = getattr(plugin_config, "version", None)
-        plugins = dict(sorted(plugins.items()))
+        # Gather installed Apps
+        nautobot_apps = {}
+        for app_name in settings.PLUGINS:
+            app_name = app_name.rsplit(".", 1)[-1]
+            app_config = apps.get_app_config(app_name)
+            nautobot_apps[app_name] = getattr(app_config, "version", None)
+        nautobot_apps = dict(sorted(nautobot_apps.items()))
 
         # Gather Celery workers
-        workers = celery_app.control.inspect().active()  # list or None
+        try:
+            workers = celery_app.control.inspect().active()  # list or None
+        except redis.exceptions.ConnectionError:
+            # Celery seems to be not smart enough to auto-retry on intermittent failures, so let's do it ourselves:
+            try:
+                workers = celery_app.control.inspect().active()  # list or None
+            except redis.exceptions.ConnectionError as err:
+                logger.error("Repeated ConnectionError from Celery/Redis: %s", err)
+                workers = None
+
         worker_count = len(workers) if workers is not None else 0
 
         return Response(
@@ -452,7 +550,8 @@ class StatusView(NautobotAPIVersionMixin, APIView):
                 "django-version": DJANGO_VERSION,
                 "installed-apps": installed_apps,
                 "nautobot-version": settings.VERSION,
-                "plugins": plugins,
+                "nautobot-apps": nautobot_apps,
+                "plugins": nautobot_apps,  # 3.0 TODO: remove this
                 "python-version": platform.python_version(),
                 "celery-workers-running": worker_count,
             }
@@ -487,18 +586,13 @@ class NautobotSpectacularSwaggerView(APIVersioningGetSchemaURLMixin, Spectacular
 
         format = "openapi"
 
-    renderer_classes = SpectacularSwaggerView.renderer_classes + [FakeOpenAPIRenderer]
+    renderer_classes = [*SpectacularSwaggerView.renderer_classes, FakeOpenAPIRenderer]
 
     template_name = "swagger_ui.html"
 
     @extend_schema(exclude=True)
     def get(self, request, *args, **kwargs):
         """Fix up the rendering of the Swagger UI to work with Nautobot's UI."""
-        if not request.user.is_authenticated and get_settings_or_config("HIDE_RESTRICTED_UI"):
-            doc_url = reverse("api_docs")
-            login_url = reverse(settings.LOGIN_URL)
-            return redirect(f"{login_url}?next={doc_url}")
-
         # For backward compatibility wtih drf-yasg, `/api/docs/?format=openapi` is a redirect to the JSON schema.
         if request.GET.get("format") == "openapi":
             return redirect("schema_json", permanent=True)
@@ -519,6 +613,41 @@ class NautobotSpectacularRedocView(APIVersioningGetSchemaURLMixin, SpectacularRe
     """Extend SpectacularRedocView to support Nautobot's ?api_version=<version> query parameter."""
 
 
+@method_decorator(gzip_page, name="dispatch")
+class NautobotSpectacularAPIView(SpectacularAPIView):
+    def _get_schema_response(self, request):
+        # version specified as parameter to the view always takes precedence. after
+        # that we try to source version through the schema view's own versioning_class.
+        version = self.api_version or request.version or self._get_version_parameter(request)
+        cache_key = f"openapi_schema_cache_{version}_{settings.VERSION}"  # Invalidate cache on Nautobot release
+        etag = f'W/"{hash(cache_key)}"'
+
+        # With combined browser cache and backend cache, we have three options:
+        # - cache expired on browser, but Etag is the same (no changes in nautobot) -> 70-100ms response
+        # - cache expired on browser, Etag is different, cache present on backend -> 400-600ms response
+        # - cache expired on browser, Etag is different, no cache on backend -> 3-4s response
+
+        if_none_match = request.META.get("HTTP_IF_NONE_MATCH", "")
+        if if_none_match == etag:
+            return Response(status=304)
+
+        schema = cache.get(cache_key)
+        if not schema:
+            generator = self.generator_class(urlconf=self.urlconf, api_version=version, patterns=self.patterns)
+            schema = generator.get_schema(request=request, public=self.serve_public)
+            cache.set(cache_key, schema, 60 * 60 * 24 * 7)
+
+        return Response(
+            data=schema,
+            headers={
+                "Content-Disposition": f'inline; filename="{self._get_filename(request, version)}"',
+                "Cache-Control": f"max-age={3 * 24 * 60 * 60}, public",
+                "ETag": etag,
+                "Vary": "Accept, Accept-Encoding",
+            },
+        )
+
+
 #
 # GraphQL
 #
@@ -527,23 +656,25 @@ class NautobotSpectacularRedocView(APIVersioningGetSchemaURLMixin, SpectacularRe
 class GraphQLDRFAPIView(NautobotAPIVersionMixin, APIView):
     """
     API View for GraphQL to integrate properly with DRF authentication mechanism.
-    The code is a stripped down version of graphene-django default View
-    https://github.com/graphql-python/graphene-django/blob/main/graphene_django/views.py#L57
     """
 
-    permission_classes = [AllowAny]
+    # The code is a stripped down version of graphene-django default View
+    # https://github.com/graphql-python/graphene-django/blob/main/graphene_django/views.py#L57
+
+    permission_classes = [IsAuthenticated]
     graphql_schema = None
     executor = None
     backend = None
     middleware = None
     root_value = None
 
-    def __init__(self, schema=None, executor=None, middleware=None, root_value=None, backend=None):
+    def __init__(self, schema=None, executor=None, middleware=None, root_value=None, backend=None, **kwargs):
         self.schema = schema
         self.executor = executor
         self.middleware = middleware
         self.root_value = root_value
         self.backend = backend
+        super().__init__(**kwargs)
 
     def get_root_value(self, request):
         return self.root_value
@@ -602,7 +733,8 @@ class GraphQLDRFAPIView(NautobotAPIVersionMixin, APIView):
         self.executor = self.executor
         self.root_value = self.root_value
 
-        assert isinstance(self.graphql_schema, GraphQLSchema), "A Schema is required to be provided to GraphQLAPIView."
+        if not isinstance(self.graphql_schema, GraphQLSchema):
+            raise ValueError("A Schema is required to be provided to GraphQLAPIView.")
 
     def get_response(self, request, data):
         """Extract the information from the request, execute the GraphQL query and form the response.
@@ -716,188 +848,38 @@ class GraphQLDRFAPIView(NautobotAPIVersionMixin, APIView):
 #
 
 
-class GetMenuAPIView(NautobotAPIVersionMixin, APIView):
-    """API View that returns the nav-menu content applicable to the requesting user."""
-
-    permission_classes = [IsAuthenticated]
-
-    def format_and_remove_hidden_menu(self, request, data, hide_restricted_ui):
-        """
-        Formats the menu data and removes hidden menu items based on user permissions.
-
-        Args:
-            request (HttpRequest): The request object.
-            data (Union[dict, str]): The menu data to format and filter. Can be either a dictionary or a string.
-            hide_restricted_ui (bool): Flag indicating whether to hide restricted menu items.
-
-        Returns:
-            (Union[dict, str]): The formatted menu data without hidden items. Returns a dict if `data` is a
-                `dict`, otherwise returns a string.
-
-        Example:
-            Input:
-            {
-                "Devices": {
-                    "permission": [...],
-                    "weight": "",
-                    "data": "data value"
-                },
-            }
-
-            Output:
-            {"Devices": "data value"}
-        """
-        if isinstance(data, dict):
-            return_value = {}
-            for name, value in data.items():
-                if not hide_restricted_ui or any(
-                    request.user.has_perm(permission) for permission in value["permissions"]
-                ):
-                    return_value[name] = self.format_and_remove_hidden_menu(request, value["data"], hide_restricted_ui)
-            return return_value
-        return data
-
-    @extend_schema(exclude=True)
-    def get(self, request):
-        """Get the menu data for the requesting user.
-
-        Returns the following data-structure (as not all context in registry["nav_menu"] is relevant to the UI):
-
-        {
-            "Inventory": {
-                "Devices": {
-                    "Devices": "/dcim/devices/",
-                    "Device Types": "/dcim/device-types/",
-                    ...
-                    "Connections": {
-                        "Cables": "/dcim/cables/",
-                        "Console Connections": "/dcim/console-connections/",
-                        ...
-                    },
-                    ...
-                },
-                "Organization": {
-                    ...
-                },
-                ...
-            },
-            "Networks": {
-                ...
-            },
-            "Security": {
-                ...
-            },
-            "Automation": {
-                ...
-            },
-            "Platform": {
-                ...
-            },
-        }
-        """
-        base_menu = registry["new_ui_nav_menu"]
-        HIDE_RESTRICTED_UI = get_settings_or_config("HIDE_RESTRICTED_UI")
-        formatted_data = self.format_and_remove_hidden_menu(request, base_menu, HIDE_RESTRICTED_UI)
-        return Response(formatted_data)
-
-
-class GetObjectCountsView(NautobotAPIVersionMixin, APIView):
-    """
-    Enumerate the models listed on the Nautobot home page and return data structure
-    containing verbose_name_plural, url and count.
-    """
+class CSVImportFieldsForContentTypeAPIView(NautobotAPIVersionMixin, APIView):
+    """Get information about CSV import fields for a given ContentType."""
 
     permission_classes = [IsAuthenticated]
 
     @extend_schema(exclude=True)
     def get(self, request):
-        object_counts = {
-            "Inventory": [
-                {"model": "dcim.rack"},
-                {"model": "dcim.devicetype"},
-                {"model": "dcim.device"},
-                {"model": "dcim.virtualchassis"},
-                {"model": "dcim.deviceredundancygroup"},
-                {"model": "dcim.cable"},
-            ],
-            "Networks": [
-                {"model": "ipam.vrf"},
-                {"model": "ipam.prefix"},
-                {"model": "ipam.ipaddress"},
-                {"model": "ipam.vlan"},
-            ],
-            "Security": [{"model": "extras.secret"}],
-            "Platform": [
-                {"model": "extras.gitrepository"},
-                {"model": "extras.relationship"},
-                {"model": "extras.computedfield"},
-                {"model": "extras.customfield"},
-                {"model": "extras.customlink"},
-                {"model": "extras.tag"},
-                {"model": "extras.status"},
-                {"model": "extras.role"},
-            ],
-        }
-        HIDE_RESTRICTED_UI = get_settings_or_config("HIDE_RESTRICTED_UI")
-
-        for entry in itertools.chain(*object_counts.values()):
-            app_label, model_name = entry["model"].split(".")
-            model = apps.get_model(app_label, model_name)
-            permission = get_permission_for_model(model, "view")
-            if HIDE_RESTRICTED_UI and not request.user.has_perm(permission):
-                continue
-            data = {"name": model._meta.verbose_name_plural}
-            try:
-                data["url"] = django_reverse(get_route_for_model(model, "list"))
-            except NoReverseMatch:
-                logger = logging.getLogger(__name__)
-                route = get_route_for_model(model, "list")
-                logger.warning(f"Handled expected exception when generating filter field: {route}")
-            manager = model.objects
-            if request.user.has_perm(permission):
-                if hasattr(manager, "restrict"):
-                    data["count"] = model.objects.restrict(request.user).count()
-                else:
-                    data["count"] = model.objects.count()
-            entry.update(data)
-
-        return Response(object_counts)
-
-
-class GetSettingsView(NautobotAPIVersionMixin, APIView):
-    """
-    This view exposes Nautobot settings.
-
-    Get settings by providing one or more `name` in the query parameters.
-
-    Example:
-
-    - /api/settings/?name=FOO            # Returns the setting with name 'FOO'
-    - /api/settings/?name=FOO&name=BAR   # Returns the setting with these names 'FOO' and 'BAR
-    """
-
-    permission_classes = [IsAuthenticated]
-
-    @extend_schema(exclude=True)
-    def get(self, request):
-        # As of now, we just have `allowed_settings` settings. Because the purpose of this API is limited for the time being,
-        # we may need more access to serializable and non-serializable settings data as the new UI expands. As a result,
-        # exposing all settings data would be desirable in the future.
-        # NOTE: When exposing all settings, include a way to limit settings which can be exposed for security concerns
-        # e.g `SECRET_KEY`, `NAPALM_PASSWORD` e.t.c. also `allowed_settings` can be moved into settings.py
-        allowed_settings = ["FEEDBACK_BUTTON_ENABLED"]
-        # Filter out settings_names not allowed to be exposed to the API
-        valid_settings = [name for name in request.GET.getlist("name") if name in allowed_settings]
-
-        invalid_settings = [name for name in request.GET.getlist("name") if name not in allowed_settings]
-        if invalid_settings:
+        content_type_id = request.GET.get("content-type-id")
+        try:
+            content_type = ContentType.objects.get(pk=content_type_id)
+        except ContentType.DoesNotExist:
+            return Response({"detail": "Invalid content-type-id."}, status=404)
+        model = content_type.model_class()
+        if model is None:
             return Response(
-                {"error": f"Invalid settings names specified: {', '.join(invalid_settings)}."},
-                status=status.HTTP_400_BAD_REQUEST,
+                {
+                    "detail": (
+                        f"Model not found for {content_type.app_label}.{content_type.model}. Perhaps an app is missing?"
+                    ),
+                },
+                status=404,
             )
-
-        res = {settings_name: get_settings_or_config(settings_name) for settings_name in valid_settings}
-        return Response(res)
+        try:
+            serializer_class = get_serializer_for_model(model)
+        except SerializerNotFound:
+            return Response(
+                {"detail": f"Serializer not found for {content_type.app_label}.{content_type.model}."},
+                status=404,
+            )
+        fields = get_csv_form_fields_from_serializer_class(serializer_class)
+        fields.sort(key=lambda field: (not field["required"], field["name"]))
+        return Response({"fields": fields})
 
 
 #
@@ -955,7 +937,6 @@ class GetFilterSetFieldDOMElementAPIView(NautobotAPIVersionMixin, APIView):
             # Cant determine the exceptions to handle because any exception could be raised,
             # e.g InterfaceForm would raise a ObjectDoesNotExist Error since no device was provided
             # While other forms might raise other errors, also if model_form is None a TypeError would be raised.
-            logger = logging.getLogger(__name__)
             logger.debug(f"Handled expected exception when generating filter field: {err}")
 
             # Create a temporary form and get a BoundField for the specified field
@@ -980,33 +961,48 @@ class GetFilterSetFieldDOMElementAPIView(NautobotAPIVersionMixin, APIView):
         return Response(data)
 
 
-class NewUIReadyRoutesAPIView(NautobotAPIVersionMixin, APIView):
-    """API View that returns a list of new UI-ready routes."""
+class SettingsJSONSchemaView(NautobotAPIVersionMixin, APIView):
+    """View that exposes the JSON Schema of the settings.yaml file in the REST API"""
 
     permission_classes = [IsAuthenticated]
 
-    def to_javascript_regex(self, regex_pattern):
-        """
-        Convert a Python regex pattern into JavaScript regex format.
-
-        Args:
-            regex_pattern (str): The Python regex pattern to convert.
-
-        Returns:
-            str: The JavaScript-compatible regex pattern.
-        """
-        # Remove named groups (?P<...>)
-        pattern = re.sub(r"\?P<[a-z]+>", "", regex_pattern)
-
-        # Escape `/` with `\`
-        pattern = pattern.replace("/", "\\/")
-
-        # Replace `/Z` with `$`
-        pattern = pattern.replace(r"\Z", "$")
-
-        return pattern
-
     @extend_schema(exclude=True)
     def get(self, request):
-        url_regex_patterns = registry["new_ui_ready_routes"]
-        return Response([self.to_javascript_regex(pattern) for pattern in url_regex_patterns])
+        file_path = os.path.dirname(os.path.dirname(os.path.abspath(__file__))) + "/settings.yaml"
+        with open(file_path, "r") as yamlfile:
+            schema_data = yaml.safe_load(yamlfile)
+        return Response(schema_data)
+
+
+class RenderJinjaError(APIException):
+    status_code = status.HTTP_400_BAD_REQUEST
+    default_detail = "Failed to render Jinja template."
+    default_code = "render_jinja_error"
+
+
+class RenderJinjaView(NautobotAPIVersionMixin, GenericAPIView):
+    """
+    View to render a Jinja template.
+    """
+
+    name = "Render Jinja2 Template"
+    permission_classes = [IsAuthenticated]
+    serializer_class = serializers.RenderJinjaSerializer
+
+    def post(self, request, *args, **kwargs):
+        data = serializers.RenderJinjaSerializer(data=request.data)
+        data.is_valid(raise_exception=True)
+        template_code = data.validated_data["template_code"]
+        context = data.validated_data["context"]
+        try:
+            rendered_template = render_jinja2(template_code, context)
+        except Exception as exc:
+            raise RenderJinjaError(f"Failed to render Jinja template: {exc}") from exc
+        return Response(
+            {
+                "rendered_template": rendered_template,
+                "rendered_template_lines": rendered_template.split("\n"),
+                "template_code": template_code,
+                "context": context,
+            }
+        )
