@@ -1,128 +1,144 @@
-from importlib.util import find_spec
 import json
 import logging
 import os
 from pathlib import Path
-import pkgutil
 import shutil
 import sys
 
 from celery import Celery, shared_task, signals
 from celery.app.log import TaskFormatter
-from celery.fixups.django import DjangoFixup
 from celery.utils.log import get_logger
+from django.apps import apps
 from django.conf import settings
+from django.db.utils import OperationalError, ProgrammingError
 from django.utils.functional import SimpleLazyObject
 from django.utils.module_loading import import_string
 from kombu.serialization import register
 from prometheus_client import CollectorRegistry, multiprocess, start_http_server
 
-from nautobot.core.celery.control import discard_git_repository, refresh_git_repository  # noqa: F401
+from nautobot import add_failure_logger, add_success_logger
+from nautobot.core.celery.control import discard_git_repository, refresh_git_repository  # noqa: F401  # unused-import
 from nautobot.core.celery.encoders import NautobotKombuJSONEncoder
 from nautobot.core.celery.log import NautobotDatabaseHandler
-
+from nautobot.core.utils.module_loading import import_modules_privately
+from nautobot.extras.plugins.utils import import_object
+from nautobot.extras.registry import registry
 
 logger = logging.getLogger(__name__)
-# The Celery documentation tells us to call setup on the app to initialize
-# settings, but we will NOT be doing that because of a chicken-and-egg problem
-# when bootstrapping the Django settings with `nautobot-server`.
-#
-# Note this would normally set the `DJANGO_SETTINGS_MODULE` environment variable
-# which Celery and its workers need under the hood.The Celery docs and examples
-# normally have you set it here, but because of our custom settings bootstrapping
-# it is handled in the `nautobot.setup() call, and we have implemented a
-# `nautobot-server celery` command to provide the correct context so this does
-# NOT need to be called here.
-# nautobot.setup()
+
+
+os.environ.setdefault("DJANGO_SETTINGS_MODULE", "nautobot_config")
 
 
 class NautobotCelery(Celery):
     task_cls = "nautobot.core.celery.task:NautobotTask"
 
-    def register_task(self, task, **options):
-        """Override the default task name for job classes to allow app provided jobs to use the full module path."""
-        from nautobot.extras.jobs import Job
-
-        if issubclass(task, Job):
-            task = task()
-            task.name = task.registered_name
-
-        return super().register_task(task, **options)
-
 
 app = NautobotCelery("nautobot")
 
 # Using a string here means the worker doesn't have to serialize
-# the configuration object to child processes. Again, this is possible
-# only after calling `nautobot.setup()` which sets `DJANGO_SETTINGS_MODULE`.
+# the configuration object to child processes.
 # - namespace='CELERY' means all celery-related configuration keys
 #   should have a `CELERY_` prefix.
 app.config_from_object("django.conf:settings", namespace="CELERY")
 
-# Because of the chicken-and-egg Django settings bootstrapping issue,
-# Celery doesn't automatically install its Django-specific patches.
-# So we need to explicitly do so ourselves:
-DjangoFixup(app).install()
-
 # Load task modules from all registered Django apps.
 app.autodiscover_tasks()
 
 
 @signals.import_modules.connect
-def import_jobs_as_celery_tasks(sender, database_ready=True, **kwargs):
+def import_jobs(sender=None, **kwargs):
     """
-    Import system Jobs into Celery as well as Jobs from JOBS_ROOT and GIT_ROOT.
+    Import system Jobs into Nautobot as well as Jobs from JOBS_ROOT and GIT_ROOT.
+    Import app-provided jobs if the app provides dynamic jobs.
 
-    Note that app-provided Jobs are automatically imported at startup time via NautobotAppConfig.ready()
+    Note that app-provided jobs are automatically imported at startup time via NautobotAppConfig.ready()
     """
-    logger.debug("Importing system Jobs")
-    sender.loader.import_task_module("nautobot.core.jobs")
+    import nautobot.core.jobs  # noqa: F401
 
-    jobs_root = settings.JOBS_ROOT
-    if jobs_root and os.path.exists(jobs_root):
-        if jobs_root not in sys.path:
-            sys.path.append(jobs_root)
-        for _, module_name, _ in pkgutil.iter_modules([jobs_root]):
-            try:
-                logger.debug("Importing Jobs from %s in JOBS_ROOT", module_name)
-                existing_module = find_spec(module_name)
-                if existing_module is not None:
-                    existing_module_path = os.path.realpath(existing_module.origin)
-                    jobs_root_path = os.path.realpath(jobs_root)
-                    if not existing_module_path.startswith(jobs_root_path):
-                        raise ImportError(
-                            f"JOBS_ROOT Jobs module {module_name} conflicts with existing module {existing_module_path}"
-                        )
-                sender.loader.import_task_module(module_name)
-            except Exception as exc:
-                logger.exception(exc)
+    _import_jobs_from_jobs_root()
+    _import_dynamic_jobs_from_apps()
 
-    git_root = settings.GIT_ROOT
-    if git_root and os.path.exists(git_root):
-        if git_root not in sys.path:
-            sys.path.append(git_root)
+    try:
+        _import_jobs_from_git_repositories()
+    except (
+        OperationalError,  # Database not present, as may be the case when running pylint-nautobot
+        ProgrammingError,  # Database not ready yet, as may be the case on initial startup and migration
+    ):
+        pass
 
-        # We can't detect which Git directories we're *supposed* to auto-load Jobs from if we can't read GitRepository
-        # records from the DB, unfortunately.
-        # We work around this in JobModel.job_task to try later loading Git jobs on-the-fly if needed.
-        if database_ready:
+
+def _import_jobs_from_jobs_root():
+    """
+    (Re)import all modules in settings.JOBS_ROOT.
+    """
+    if not (settings.JOBS_ROOT and os.path.isdir(settings.JOBS_ROOT)):
+        return
+
+    # Flush any previously loaded non-system, non-App Jobs
+    for job_class_path in list(registry["jobs"]):
+        if job_class_path.startswith("nautobot."):
+            # System job
+            continue
+        if any(job_class_path.startswith(f"{app_name}.") for app_name in settings.PLUGINS):
+            # App provided job
+            continue
+        try:
             from nautobot.extras.models import GitRepository
 
-            # Make sure there are no git clones in GIT_ROOT that *aren't* tracked by a GitRepository;
-            # for example, maybe a GitRepository was deleted while this worker process wasn't running?
-            for filename in os.listdir(git_root):
-                filepath = os.path.join(git_root, filename)
-                if (
-                    os.path.isdir(filepath)
-                    and os.path.isdir(os.path.join(filepath, ".git"))
-                    and not GitRepository.objects.filter(slug=filename).exists()
-                ):
-                    logger.warning("Deleting unmanaged (leftover?) Git repository clone at %s", filepath)
-                    shutil.rmtree(filepath)
+            if any(
+                job_class_path.startswith(f"{repo.slug}.")
+                for repo in GitRepository.objects.filter(provided_contents__contains="extras.job")
+            ):
+                # Git provided job
+                continue
+        except ProgrammingError:  # Database not ready yet, as may be the case on initial startup and migration
+            pass
+        # Else, it's presumably a JOBS_ROOT job
+        del registry["jobs"][job_class_path]
 
-            # Make sure all GitRepository records that include Jobs have up-to-date git clones, and load their jobs
-            for repo in GitRepository.objects.all():
-                refresh_git_repository(state=None, repository_pk=repo.pk, head=repo.current_head)
+    # Load all modules in JOBS_ROOT
+    import_modules_privately(path=os.path.realpath(settings.JOBS_ROOT))
+
+
+def _import_jobs_from_git_repositories():
+    git_root = os.path.realpath(settings.GIT_ROOT)
+    if not (git_root and os.path.exists(git_root)):
+        return
+
+    from nautobot.extras.models import GitRepository
+
+    # Make sure there are no git clones in GIT_ROOT that *aren't* tracked by a GitRepository;
+    # for example, maybe a GitRepository was deleted while this worker process wasn't running?
+    for filename in os.listdir(git_root):
+        filepath = os.path.join(git_root, filename)
+        if (
+            os.path.isdir(filepath)
+            and os.path.isdir(os.path.join(filepath, ".git"))
+            and not GitRepository.objects.filter(slug=filename).exists()
+        ):
+            logger.warning("Deleting unmanaged (leftover?) Git repository clone at %s", filepath)
+            shutil.rmtree(filepath, ignore_errors=True)
+
+    # Make sure all GitRepository records that include Jobs have up-to-date git clones, and load their jobs
+    for repo in GitRepository.objects.filter(provided_contents__contains="extras.job"):
+        refresh_git_repository(state=None, repository_pk=repo.pk, head=repo.current_head)
+
+
+def _import_dynamic_jobs_from_apps():
+    for app_name in settings.PLUGINS:
+        app_config = apps.get_app_config(app_name)
+        if not getattr(app_config, "provides_dynamic_jobs", False):
+            continue
+
+        # Unload job modules from sys.modules if they were previously loaded
+        app_jobs = getattr(app_config, "features", {}).get("jobs", [])
+        for job in app_jobs:
+            if job.__module__ in sys.modules:
+                del sys.modules[job.__module__]
+
+        # Load app jobs
+        app_config.features["jobs"] = import_object(f"{app_config.__module__}.{app_config.jobs}")
 
 
 def add_nautobot_log_handler(logger_instance, log_format=None):
@@ -141,6 +157,20 @@ def add_nautobot_log_handler(logger_instance, log_format=None):
     logger_instance.addHandler(handler)
 
 
+@signals.after_setup_logger.connect
+def setup_nautobot_global_logging(logger, **kwargs):  # pylint: disable=redefined-outer-name
+    """Add SUCCESS and FAILURE logs to celery global logger."""
+    logger.success = add_success_logger()
+    logger.failure = add_failure_logger()
+
+
+@signals.after_setup_task_logger.connect
+def setup_nautobot_task_logging(logger, **kwargs):  # pylint: disable=redefined-outer-name
+    """Add SUCCESS and FAILURE logs to celery task logger."""
+    logger.success = add_success_logger()
+    logger.failure = add_failure_logger()
+
+
 @signals.celeryd_after_setup.connect
 def setup_nautobot_job_logging(sender, instance, conf, **kwargs):
     """Add nautobot database logging handler to celery stdout/stderr redirect logger and celery task logger."""
@@ -167,11 +197,11 @@ def setup_prometheus(**kwargs):
     multiprocess_coordination_directory.mkdir(parents=True, exist_ok=True)
 
     # Set up the collector registry
-    registry = CollectorRegistry()
-    multiprocess.MultiProcessCollector(registry, path=multiprocess_coordination_directory)
+    collector_registry = CollectorRegistry()
+    multiprocess.MultiProcessCollector(collector_registry, path=multiprocess_coordination_directory)
     for port in settings.CELERY_WORKER_PROMETHEUS_PORTS:
         try:
-            start_http_server(port, registry=registry)
+            start_http_server(port, registry=collector_registry)
             break
         except OSError:
             continue
@@ -222,9 +252,13 @@ register("nautobot_json", _dumps, _loads, content_type="application/x-nautobot-j
 nautobot_task = shared_task
 
 
+registry["jobs"] = {}
+
+
 def register_jobs(*jobs):
-    """Helper method to register jobs with Celery"""
+    """
+    Method to register jobs - with Celery in Nautobot 2.0 through 2.2.2, with Nautobot itself in 2.2.3 and later.
+    """
     for job in jobs:
-        # TODO: should we only register a job if it corresponds to a Job database record?
-        logger.debug("Registering job %s.%s", job.__module__, job.__name__)
-        app.register_task(job)
+        if job.class_path not in registry["jobs"]:
+            registry["jobs"][job.class_path] = job
