@@ -1,7 +1,7 @@
 from django.apps import apps
 from django.contrib.auth import get_user_model
-from django.core.management.base import BaseCommand, CommandError
 from django.core.exceptions import ValidationError
+from django.core.management.base import BaseCommand, CommandError
 
 
 class Command(BaseCommand):
@@ -63,7 +63,7 @@ class Command(BaseCommand):
         for model in models:
             model_name = f"{model._meta.app_label}.{model.__name__}"
             # Most swap out for user_model
-            if model_name == "auth.User":
+            if model_name == "auth.User":  # pylint: disable=hard-coded-auth-user
                 model = get_user_model()
             # Skip models that aren't actually in the database
             if not model._meta.managed:
