@@ -4,18 +4,19 @@ from django.conf import settings
 from django.contrib.auth import get_user_model
 from django.contrib.auth.models import Group
 from django.contrib.contenttypes.models import ContentType
+from django.db.models import Q
 from django.test.utils import override_settings
 from django.urls import reverse
 from netaddr import IPNetwork
 
 from nautobot.core.settings_funcs import sso_auth_enabled
 from nautobot.core.testing import NautobotTestClient, TestCase
+from nautobot.core.utils import lookup
 from nautobot.dcim.models import Location, LocationType
-from nautobot.extras.models import Status
-from nautobot.ipam.models import Prefix, Namespace
+from nautobot.extras.models import ObjectChange, Status
+from nautobot.ipam.models import Namespace, Prefix
 from nautobot.users.models import ObjectPermission, Token
 
-
 # Use the proper swappable User model
 User = get_user_model()
 
@@ -51,7 +52,7 @@ class ExternalAuthenticationTestCase(TestCase):
         self.assertEqual(settings.REMOTE_AUTH_HEADER, "HTTP_REMOTE_USER")
 
         # Client should not be authenticated
-        self.client.get(reverse("home"), follow=True, **headers)  # noqa
+        self.client.get(reverse("home"), follow=True, **headers)
         self.assertNotIn("_auth_user_id", self.client.session)
 
     @override_settings(AUTHENTICATION_BACKENDS=TEST_AUTHENTICATION_BACKENDS)
@@ -321,7 +322,7 @@ class ObjectPermissionAPIViewTestCase(TestCase):
         # Assign object permission
         obj_perm = ObjectPermission.objects.create(
             name="Test permission",
-            constraints={"location__name": self.locations[0].name},
+            constraints={"locations__name__in": [self.locations[0].name]},
             actions=["view"],
         )
         obj_perm.users.add(self.user)
@@ -348,7 +349,7 @@ class ObjectPermissionAPIViewTestCase(TestCase):
         # Assign object permission
         obj_perm = ObjectPermission.objects.create(
             name="Test permission",
-            constraints={"location__name": self.locations[0].name},
+            constraints={"locations__name__in": [self.locations[0].name]},
             actions=["view"],
         )
         obj_perm.users.add(self.user)
@@ -357,7 +358,7 @@ class ObjectPermissionAPIViewTestCase(TestCase):
         # Retrieve all objects. Only permitted objects should be returned.
         response = self.client.get(url, **self.header)
         self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.data["count"], Prefix.objects.filter(location=self.locations[0]).count())
+        self.assertEqual(response.data["count"], Prefix.objects.filter(locations__in=[self.locations[0]]).count())
 
     @override_settings(EXEMPT_VIEW_PERMISSIONS=[])
     def test_create_object(self):
@@ -377,19 +378,36 @@ class ObjectPermissionAPIViewTestCase(TestCase):
         # Assign object permission
         obj_perm = ObjectPermission.objects.create(
             name="Test permission",
-            constraints={"location__name": self.locations[0].name},
+            constraints={"locations__name__in": [self.locations[0].name]},
             actions=["add"],
         )
         obj_perm.users.add(self.user)
         obj_perm.object_types.add(ContentType.objects.get_for_model(Prefix))
-
+        related_obj_perm = ObjectPermission.objects.create(
+            name="Related object permission",
+            actions=["view"],
+        )
+        related_obj_perm.users.add(self.user)
+        related_obj_perm.object_types.add(
+            ContentType.objects.get_for_model(Namespace),
+            ContentType.objects.get_for_model(Status),
+            ContentType.objects.get_for_model(Location),
+        )
         # Attempt to create a non-permitted object
         response = self.client.post(url, data, format="json", **self.header)
         self.assertEqual(response.status_code, 403)
         self.assertEqual(Prefix.objects.count(), initial_count)
 
-        # Create a permitted object
+        # Attempt to create a permitted object without related object permissions
         data["location"] = self.locations[0].pk
+        related_obj_perm.users.remove(self.user)
+        response = self.client.post(url, data, format="json", **self.header)
+        self.assertEqual(response.status_code, 400)
+        self.assertIn(b"Related object not found using the provided attribute", response.content)
+        self.assertEqual(Prefix.objects.count(), initial_count)
+
+        # Create a permitted object with related object permissions
+        related_obj_perm.users.add(self.user)
         response = self.client.post(url, data, format="json", **self.header)
         self.assertEqual(response.status_code, 201)
         self.assertEqual(Prefix.objects.count(), initial_count + 1)
@@ -405,22 +423,38 @@ class ObjectPermissionAPIViewTestCase(TestCase):
         # Assign object permission
         obj_perm = ObjectPermission.objects.create(
             name="Test permission",
-            constraints={"location__name": f"{self.locations[0].name}"},
+            constraints={"locations__name__in": [self.locations[0].name]},
             actions=["change"],
         )
         obj_perm.users.add(self.user)
         obj_perm.object_types.add(ContentType.objects.get_for_model(Prefix))
-
+        related_obj_perm = ObjectPermission.objects.create(
+            name="Related object permission",
+            actions=["view"],
+        )
+        related_obj_perm.users.add(self.user)
+        related_obj_perm.object_types.add(
+            ContentType.objects.get_for_model(Namespace),
+            ContentType.objects.get_for_model(Status),
+            ContentType.objects.get_for_model(Location),
+        )
         # Attempt to edit a non-permitted object
         data = {"location": self.locations[0].pk}
         url = reverse("ipam-api:prefix-detail", kwargs={"pk": self.prefixes[3].pk})
         response = self.client.patch(url, data, format="json", **self.header)
         self.assertEqual(response.status_code, 404)
 
-        # Edit a permitted object
+        # Attempt to edit a permitted object without related object permissions
+        related_obj_perm.users.remove(self.user)
         data["status"] = self.statuses[1].pk
         url = reverse("ipam-api:prefix-detail", kwargs={"pk": self.prefixes[0].pk})
         response = self.client.patch(url, data, format="json", **self.header)
+        self.assertEqual(response.status_code, 400)
+        self.assertIn(b"Related object not found using the provided attribute", response.content)
+
+        # Edit a permitted object with related object permissions
+        related_obj_perm.users.add(self.user)
+        response = self.client.patch(url, data, format="json", **self.header)
         self.assertEqual(response.status_code, 200)
 
         # Attempt to modify a permitted object to a non-permitted object
@@ -439,7 +473,7 @@ class ObjectPermissionAPIViewTestCase(TestCase):
         # Assign object permission
         obj_perm = ObjectPermission.objects.create(
             name="Test permission",
-            constraints={"location__name": self.locations[0].name},
+            constraints={"locations__name__in": [self.locations[0].name]},
             actions=["delete"],
         )
         obj_perm.users.add(self.user)
@@ -454,3 +488,179 @@ class ObjectPermissionAPIViewTestCase(TestCase):
         url = reverse("ipam-api:prefix-detail", kwargs={"pk": self.prefixes[0].pk})
         response = self.client.delete(url, format="json", **self.header)
         self.assertEqual(response.status_code, 204)
+
+    @override_settings(EXEMPT_VIEW_PERMISSIONS=[])
+    def test_related_object_permission_constraints_on_get_requests(self):
+        """
+        Users who have permission to view Location objects, but not LocationType and Status objects
+        should still be able to view Location objects from the API.
+        """
+        self.add_permissions("dcim.view_location")
+        response = self.client.get(reverse("dcim-api:location-list"), **self.header)
+        self.assertEqual(response.status_code, 200)
+        # we should be able to get all the locations
+        self.assertEqual(len(response.data["results"]), Location.objects.count())
+
+    @override_settings(EXEMPT_VIEW_PERMISSIONS=[])
+    def test_related_object_permission_constraints_on_patch_requests(self):
+        """
+        Users who have permission to view and change Location objects, but not LocationType and Status objects
+        should still be able to change a Location object's name from the API.
+        """
+        self.add_permissions("dcim.view_location", "dcim.change_location")
+        location = Location.objects.first()
+        data = {"name": "New Location Name"}
+        url = reverse("dcim-api:location-detail", kwargs={"pk": location.pk})
+        response = self.client.patch(url, data, format="json", **self.header)
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(response.data["name"], "New Location Name")
+
+    @override_settings(EXEMPT_VIEW_PERMISSIONS=[])
+    def test_user_token_constraints(self):
+        """
+        Test user token as permission constraints.
+        """
+        url = reverse("ipam-api:prefix-list")
+        data = [
+            {
+                "prefix": "10.0.9.0/24",
+                "namespace": self.namespace.pk,
+                "location": self.locations[1].pk,
+                "status": self.statuses[1].pk,
+            },
+            {
+                "prefix": "10.0.10.0/24",
+                "namespace": self.namespace.pk,
+                "location": self.locations[1].pk,
+                "status": self.statuses[1].pk,
+            },
+        ]
+
+        obj_user2 = User.objects.create(username="new-user")
+        token_user2 = Token.objects.create(user=obj_user2)
+        header_user2 = {"HTTP_AUTHORIZATION": f"Token {token_user2.key}"}
+        # Assign object permission to both users to create Prefixes
+        obj_perm = ObjectPermission.objects.create(
+            name="Test ipam permission",
+            actions=["add"],
+        )
+        obj_perm.users.add(self.user, obj_user2)
+        obj_perm.object_types.add(ContentType.objects.get_for_model(Prefix))
+        related_obj_perm = ObjectPermission.objects.create(
+            name="Related object permission",
+            actions=["view"],
+        )
+        related_obj_perm.users.add(self.user, obj_user2)
+        related_obj_perm.object_types.add(
+            ContentType.objects.get_for_model(Namespace),
+            ContentType.objects.get_for_model(Status),
+            ContentType.objects.get_for_model(Location),
+        )
+        # Create one Prefix object per user
+        self.client.post(url, data[0], format="json", **self.header)
+        self.client.post(url, data[1], format="json", **header_user2)
+
+        # Assign object permission to both users to view Change Logs, based on user token constraint
+        obj_perm = ObjectPermission.objects.create(
+            name="Test change log permission",
+            constraints={"user": "$user"},
+            actions=["view", "list"],
+        )
+        obj_perm.users.add(self.user, obj_user2)
+        obj_perm.object_types.add(ContentType.objects.get_for_model(ObjectChange))
+
+        # Retrieve all ObjectChange Log entries for every user
+        url = reverse(lookup.get_route_for_model(ObjectChange, "list", api=True))
+        response_user1 = self.client.get(url, **self.header)
+        response_user2 = self.client.get(url, **header_user2)
+
+        # Assert every user has permissions to view Change Logs
+        self.assertTrue(self.user.has_perms(["extras.view_objectchange", "extras.list_objectchange"]))
+        self.assertTrue(obj_user2.has_perms(["extras.view_objectchange", "extras.list_objectchange"]))
+
+        # Check against 1st user's response
+        self.assertEqual(response_user1.status_code, 200)
+        self.assertEqual(response_user1.data["count"], 1)
+        self.assertEqual(response_user1.data["results"][0]["user"]["id"], self.user.pk)
+
+        # Check against 2nd user's response
+        self.assertEqual(response_user2.status_code, 200)
+        self.assertEqual(response_user2.data["count"], 1)
+        self.assertEqual(response_user2.data["results"][0]["user"]["id"], obj_user2.pk)
+
+    @override_settings(EXEMPT_VIEW_PERMISSIONS=[])
+    def test_user_token_list_constraints(self):
+        """
+        Test user token as permission of a list of constraints.
+        """
+        url = reverse("ipam-api:prefix-list")
+        data = [
+            {
+                "prefix": "10.0.9.0/24",
+                "namespace": self.namespace.pk,
+                "location": self.locations[1].pk,
+                "status": self.statuses[1].pk,
+            },
+            {
+                "prefix": "10.0.10.0/24",
+                "namespace": self.namespace.pk,
+                "location": self.locations[1].pk,
+                "status": self.statuses[1].pk,
+            },
+        ]
+
+        obj_user2 = User.objects.create(username="new-user")
+        token_user2 = Token.objects.create(user=obj_user2)
+        header_user2 = {"HTTP_AUTHORIZATION": f"Token {token_user2.key}"}
+        # Assign object permission to both users to create Prefixes
+        obj_perm = ObjectPermission.objects.create(
+            name="Test ipam permission",
+            actions=["add"],
+        )
+        obj_perm.users.add(self.user, obj_user2)
+        obj_perm.object_types.add(ContentType.objects.get_for_model(Prefix))
+        related_obj_perm = ObjectPermission.objects.create(
+            name="Related object permission",
+            actions=["view"],
+        )
+        related_obj_perm.users.add(self.user, obj_user2)
+        related_obj_perm.object_types.add(
+            ContentType.objects.get_for_model(Namespace),
+            ContentType.objects.get_for_model(Status),
+            ContentType.objects.get_for_model(Location),
+        )
+        # Create one Prefix object per user
+        self.client.post(url, data[0], format="json", **self.header)
+        self.client.post(url, data[1], format="json", **header_user2)
+
+        # Assign object permission to both users to view Change Logs, based on user token constraint
+        obj_perm = ObjectPermission.objects.create(
+            name="Test change log permission",
+            constraints=[{"user": "$user"}, {"action": "delete"}],
+            actions=["view", "list"],
+        )
+        obj_perm.users.add(self.user, obj_user2)
+        obj_perm.object_types.add(ContentType.objects.get_for_model(ObjectChange))
+
+        # Retrieve all ObjectChange Log entries for every user
+        url = reverse(lookup.get_route_for_model(ObjectChange, "list", api=True))
+        response_user1 = self.client.get(url, **self.header)
+        response_user2 = self.client.get(url, **header_user2)
+
+        # Assert every user has permissions to view Change Logs
+        self.assertTrue(self.user.has_perms(["extras.view_objectchange", "extras.list_objectchange"]))
+        self.assertTrue(obj_user2.has_perms(["extras.view_objectchange", "extras.list_objectchange"]))
+
+        # Check against 1st user's response
+        self.assertEqual(response_user1.status_code, 200)
+        self.assertEqual(
+            response_user1.data["count"], ObjectChange.objects.filter(Q(user=self.user) | Q(action="delete")).count()
+        )
+        self.assertEqual(response_user1.data["results"][0]["user"]["id"], self.user.pk)
+
+        # Check against 2nd user's response
+        self.assertEqual(response_user2.status_code, 200)
+        self.assertEqual(
+            response_user2.data["count"], ObjectChange.objects.filter(Q(user=obj_user2) | Q(action="delete")).count()
+        )
+        self.assertEqual(response_user2.data["results"][0]["user"]["id"], obj_user2.pk)
