@@ -6,12 +6,14 @@ from django.db import connections
 
 import nautobot
 
+logger = logging.getLogger(__name__)
+
 # This is the Django default left here for visibility on how the Nautobot pattern
 # differs.
 # os.environ.setdefault("DJANGO_SETTINGS_MODULE", "nautobot.core.settings")
 
-# Instead of importing `DJANGO_SETTINGS_MODULE` we're using the custom loader
-# pattern from `nautobot.core.runner` to read environment or config path for us.
+# Instead of just pointing to `DJANGO_SETTINGS_MODULE` and letting Django run with it,
+# we're using the custom Nautobot loader code to read environment or config path for us.
 nautobot.setup()
 
 # Use try/except because we might not be running uWSGI. If `settings.WEBSERVER_WARMUP` is `True`,
@@ -24,9 +26,7 @@ try:
     def fix_uwsgi():
         import uwsgi
 
-        logging.getLogger(__name__).info(
-            f"Closing existing DB and cache connections on worker {uwsgi.worker_id()} after uWSGI forked ..."
-        )
+        logger.info("Closing existing DB and cache connections on worker %s after uWSGI forked ...", uwsgi.worker_id())
         connections.close_all()
         cache.close_caches()
 
