@@ -1,9 +1,10 @@
 import contextlib
 
 from django.contrib.contenttypes.models import ContentType
+from django.core.exceptions import ValidationError
 from drf_spectacular.utils import extend_schema_field
 from rest_framework import serializers
-from rest_framework.validators import UniqueTogetherValidator
+from rest_framework.validators import UniqueTogetherValidator, UniqueValidator
 
 from nautobot.core.api import (
     ChoiceField,
@@ -27,17 +28,21 @@ from nautobot.dcim.choices import (
     CableLengthUnitChoices,
     CableTypeChoices,
     ConsolePortTypeChoices,
+    ControllerCapabilitiesChoices,
     DeviceFaceChoices,
     DeviceRedundancyGroupFailoverStrategyChoices,
     InterfaceModeChoices,
     InterfaceRedundancyGroupProtocolChoices,
     InterfaceTypeChoices,
     PortTypeChoices,
+    PowerFeedBreakerPoleChoices,
     PowerFeedPhaseChoices,
     PowerFeedSupplyChoices,
     PowerFeedTypeChoices,
     PowerOutletFeedLegChoices,
     PowerOutletTypeChoices,
+    PowerPanelTypeChoices,
+    PowerPathChoices,
     PowerPortTypeChoices,
     RackDimensionUnitChoices,
     RackElevationDetailRenderChoices,
@@ -54,21 +59,31 @@ from nautobot.dcim.models import (
     ConsolePortTemplate,
     ConsoleServerPort,
     ConsoleServerPortTemplate,
+    Controller,
+    ControllerManagedDeviceGroup,
     Device,
     DeviceBay,
     DeviceBayTemplate,
+    DeviceFamily,
     DeviceRedundancyGroup,
     DeviceType,
+    DeviceTypeToSoftwareImageFile,
     FrontPort,
     FrontPortTemplate,
     Interface,
     InterfaceRedundancyGroup,
     InterfaceRedundancyGroupAssociation,
     InterfaceTemplate,
+    InterfaceVDCAssignment,
     InventoryItem,
     Location,
     LocationType,
     Manufacturer,
+    Module,
+    ModuleBay,
+    ModuleBayTemplate,
+    ModuleFamily,
+    ModuleType,
     PathEndpoint,
     Platform,
     PowerFeed,
@@ -82,7 +97,10 @@ from nautobot.dcim.models import (
     RackReservation,
     RearPort,
     RearPortTemplate,
+    SoftwareImageFile,
+    SoftwareVersion,
     VirtualChassis,
+    VirtualDeviceContext,
 )
 from nautobot.extras.api.mixins import (
     TaggedModelSerializerMixin,
@@ -170,12 +188,40 @@ class ConnectedEndpointSerializer(PathEndpointModelSerializerMixin):
     pass
 
 
+class ModularDeviceComponentTemplateSerializerMixin:
+    def validate(self, data):
+        """Validate device_type and module_type field constraints for modular device component templates."""
+        if data.get("device_type") and data.get("module_type"):
+            raise serializers.ValidationError("Only one of device_type or module_type must be set")
+        if data.get("device_type"):
+            validator = UniqueTogetherValidator(queryset=self.Meta.model.objects.all(), fields=("device_type", "name"))
+            validator(data, self)
+        if data.get("module_type"):
+            validator = UniqueTogetherValidator(queryset=self.Meta.model.objects.all(), fields=("module_type", "name"))
+            validator(data, self)
+        return super().validate(data)
+
+
+class ModularDeviceComponentSerializerMixin:
+    def validate(self, data):
+        """Validate device and module field constraints for modular device components."""
+        if data.get("device") and data.get("module"):
+            raise serializers.ValidationError("Only one of device or module must be set")
+        if data.get("device"):
+            validator = UniqueTogetherValidator(queryset=self.Meta.model.objects.all(), fields=("device", "name"))
+            validator(data, self)
+        if data.get("module"):
+            validator = UniqueTogetherValidator(queryset=self.Meta.model.objects.all(), fields=("module", "name"))
+            validator(data, self)
+        return super().validate(data)
+
+
 #
 # Locations
 #
 
 
-class LocationTypeSerializer(NautobotModelSerializer, TreeModelSerializerMixin):
+class LocationTypeSerializer(TreeModelSerializerMixin, NautobotModelSerializer):
     content_types = ContentTypeField(
         queryset=ContentType.objects.filter(FeatureQuery("locations").get_query()),
         required=False,
@@ -185,13 +231,12 @@ class LocationTypeSerializer(NautobotModelSerializer, TreeModelSerializerMixin):
     class Meta:
         model = LocationType
         fields = "__all__"
-        list_display_fields = ["name", "nestable", "content_types", "description"]
 
 
 class LocationSerializer(
-    NautobotModelSerializer,
     TaggedModelSerializerMixin,
     TreeModelSerializerMixin,
+    NautobotModelSerializer,
 ):
     time_zone = TimeZoneSerializerField(required=False, allow_null=True)
     circuit_count = serializers.IntegerField(read_only=True)
@@ -204,47 +249,18 @@ class LocationSerializer(
     class Meta:
         model = Location
         fields = "__all__"
-        list_display_fields = ["name", "status", "parent", "tenant", "description", "tags"]
         # https://www.django-rest-framework.org/api-guide/validators/#optional-fields
         validators = []
 
-        detail_view_config = {
-            "layout": [
-                {
-                    "Location": {
-                        "fields": [
-                            "location_type",
-                            "tenant",
-                            "facility",
-                            "asn",
-                            "time_zone",
-                            "description",
-                        ]
-                    },
-                    "Contact Info": {
-                        "fields": [
-                            "physical_address",
-                            "shipping_address",
-                            "latitude",
-                            "longitude",
-                            "contact_name",
-                            "contact_phone",
-                            "contact_email",
-                        ]
-                    },
-                },
-            ],
-        }
-
-    def validate(self, data):
+    def validate(self, attrs):
         # Validate uniqueness of (parent, name) since we omitted the automatically created validator from Meta.
-        if data.get("parent") and data.get("name"):
+        if attrs.get("parent") and attrs.get("name"):
             validator = UniqueTogetherValidator(queryset=Location.objects.all(), fields=("parent", "name"))
-            validator(data, self)
+            validator(attrs, self)
 
-        super().validate(data)
+        super().validate(attrs)
 
-        return data
+        return attrs
 
 
 #
@@ -252,19 +268,15 @@ class LocationSerializer(
 #
 
 
-class RackGroupSerializer(NautobotModelSerializer, TreeModelSerializerMixin):
+class RackGroupSerializer(TreeModelSerializerMixin, NautobotModelSerializer):
     rack_count = serializers.IntegerField(read_only=True)
 
     class Meta:
         model = RackGroup
         fields = "__all__"
-        list_display_fields = ["name", "location", "rack_count", "description"]
 
 
-class RackSerializer(
-    NautobotModelSerializer,
-    TaggedModelSerializerMixin,
-):
+class RackSerializer(TaggedModelSerializerMixin, NautobotModelSerializer):
     type = ChoiceField(choices=RackTypeChoices, allow_blank=True, required=False)
     width = ChoiceField(choices=RackWidthChoices, required=False, help_text="Rail-to-rail width (in inches)")
     outer_unit = ChoiceField(choices=RackDimensionUnitChoices, allow_blank=True, required=False)
@@ -274,39 +286,24 @@ class RackSerializer(
     class Meta:
         model = Rack
         fields = "__all__"
-        list_display_fields = ["name", "location", "rack_group", "status", "facility_id", "tenant", "role", "u_height"]
         # Omit the UniqueTogetherValidators that would be automatically added to validate (rack_group, facility_id) and (rack_group, name).
         # This prevents facility_id and rack_group from being interpreted as required fields.
         validators = []
-        detail_view_config = {
-            "layout": [
-                {
-                    "Rack": {
-                        "fields": [
-                            "name",
-                            "location",
-                            "rack_group",
-                        ]
-                    },
-                },
-            ],
-            "include_others": True,
-        }
 
-    def validate(self, data):
+    def validate(self, attrs):
         # Validate uniqueness of (rack_group, name) since we omitted the automatically-created validator above.
-        if data.get("rack_group", None):
+        if attrs.get("rack_group", None):
             validator = UniqueTogetherValidator(queryset=Rack.objects.all(), fields=("rack_group", "name"))
-            validator(data, self)
+            validator(attrs, self)
         # Validate uniqueness of (rack_group, facility_id) since we omitted the automatically-created validator above.
-        if data.get("facility_id", None) and data.get("rack_group", None):
+        if attrs.get("facility_id", None) and attrs.get("rack_group", None):
             validator = UniqueTogetherValidator(queryset=Rack.objects.all(), fields=("rack_group", "facility_id"))
-            validator(data, self)
+            validator(attrs, self)
 
         # Enforce model validation
-        super().validate(data)
+        super().validate(attrs)
 
-        return data
+        return attrs
 
 
 class RackUnitSerializer(serializers.Serializer):
@@ -320,11 +317,10 @@ class RackUnitSerializer(serializers.Serializer):
     occupied = serializers.BooleanField(read_only=True)
 
 
-class RackReservationSerializer(NautobotModelSerializer, TaggedModelSerializerMixin):
+class RackReservationSerializer(TaggedModelSerializerMixin, NautobotModelSerializer):
     class Meta:
         model = RackReservation
         fields = "__all__"
-        list_display_fields = ["pk", "rack", "units", "user", "description"]
         extra_kwargs = {
             "units": {"help_text": "List of rack unit numbers to reserve"},
             "user": {
@@ -354,10 +350,11 @@ class RackElevationDetailFilterSerializer(serializers.Serializer):
     expand_devices = serializers.BooleanField(required=False, default=True)
     include_images = serializers.BooleanField(required=False, default=True)
     display_fullname = serializers.BooleanField(required=False, default=True)
+    is_occupied = serializers.BooleanField(required=False, allow_null=True, default=None)
 
     def validate(self, attrs):
-        attrs.setdefault("unit_width", get_settings_or_config("RACK_ELEVATION_DEFAULT_UNIT_WIDTH"))
-        attrs.setdefault("unit_height", get_settings_or_config("RACK_ELEVATION_DEFAULT_UNIT_HEIGHT"))
+        attrs.setdefault("unit_width", get_settings_or_config("RACK_ELEVATION_DEFAULT_UNIT_WIDTH", fallback=230))
+        attrs.setdefault("unit_height", get_settings_or_config("RACK_ELEVATION_DEFAULT_UNIT_HEIGHT", fallback=22))
         return attrs
 
 
@@ -367,6 +364,7 @@ class RackElevationDetailFilterSerializer(serializers.Serializer):
 
 
 class ManufacturerSerializer(NautobotModelSerializer):
+    cloud_account_count = serializers.IntegerField(read_only=True)
     device_type_count = serializers.IntegerField(read_only=True)
     inventory_item_count = serializers.IntegerField(read_only=True)
     platform_count = serializers.IntegerField(read_only=True)
@@ -374,10 +372,17 @@ class ManufacturerSerializer(NautobotModelSerializer):
     class Meta:
         model = Manufacturer
         fields = "__all__"
-        list_display_fields = ["name", "device_type_count", "platform_count", "description"]
 
 
-class DeviceTypeSerializer(NautobotModelSerializer, TaggedModelSerializerMixin):
+class DeviceFamilySerializer(TaggedModelSerializerMixin, NautobotModelSerializer):
+    device_type_count = serializers.IntegerField(read_only=True)
+
+    class Meta:
+        model = DeviceFamily
+        fields = "__all__"
+
+
+class DeviceTypeSerializer(TaggedModelSerializerMixin, NautobotModelSerializer):
     subdevice_role = ChoiceField(choices=SubdeviceRoleChoices, allow_blank=True, required=False)
     front_image = serializers.ImageField(allow_null=True, required=False)
     rear_image = serializers.ImageField(allow_null=True, required=False)
@@ -386,44 +391,30 @@ class DeviceTypeSerializer(NautobotModelSerializer, TaggedModelSerializerMixin):
     class Meta:
         model = DeviceType
         fields = "__all__"
-        list_display_fields = ["model", "manufacturer", "part_number", "u_height", "is_full_depth", "device_count"]
 
-        detail_view_config = {
-            "layout": [
-                {
-                    "Chassis": {
-                        "fields": [
-                            "manufacturer",
-                            "model",
-                            "part_number",
-                            "u_height",
-                            "is_full_depth",
-                            "subdevice_role",
-                            "front_image",
-                            "rear_image",
-                            "device_count",
-                        ]
-                    },
-                },
-            ],
-            "include_others": True,
+        extra_kwargs = {
+            "device_family": {
+                "required": False,
+            },
         }
 
 
-class ConsolePortTemplateSerializer(NautobotModelSerializer):
+class ConsolePortTemplateSerializer(ModularDeviceComponentTemplateSerializerMixin, NautobotModelSerializer):
     type = ChoiceField(choices=ConsolePortTypeChoices, allow_blank=True, required=False)
 
     class Meta:
         model = ConsolePortTemplate
         fields = "__all__"
+        validators = []
 
 
-class ConsoleServerPortTemplateSerializer(NautobotModelSerializer):
+class ConsoleServerPortTemplateSerializer(ModularDeviceComponentTemplateSerializerMixin, NautobotModelSerializer):
     type = ChoiceField(choices=ConsolePortTypeChoices, allow_blank=True, required=False)
 
     class Meta:
         model = ConsoleServerPortTemplate
         fields = "__all__"
+        validators = []
 
 
 #
@@ -441,7 +432,7 @@ class InterfaceRedundancyGroupAssociationSerializer(ValidatedModelSerializer):
         fields = "__all__"
 
 
-class InterfaceRedundancyGroupSerializer(NautobotModelSerializer, TaggedModelSerializerMixin):
+class InterfaceRedundancyGroupSerializer(TaggedModelSerializerMixin, NautobotModelSerializer):
     """InterfaceRedundancyGroup Serializer."""
 
     protocol = ChoiceField(choices=InterfaceRedundancyGroupProtocolChoices)
@@ -456,45 +447,55 @@ class InterfaceRedundancyGroupSerializer(NautobotModelSerializer, TaggedModelSer
         }
 
 
-class PowerPortTemplateSerializer(NautobotModelSerializer):
+class PowerPortTemplateSerializer(ModularDeviceComponentTemplateSerializerMixin, NautobotModelSerializer):
     type = ChoiceField(choices=PowerPortTypeChoices, allow_blank=True, required=False)
 
     class Meta:
         model = PowerPortTemplate
         fields = "__all__"
+        validators = []
 
 
-class PowerOutletTemplateSerializer(NautobotModelSerializer):
+class PowerOutletTemplateSerializer(ModularDeviceComponentTemplateSerializerMixin, NautobotModelSerializer):
     type = ChoiceField(choices=PowerOutletTypeChoices, allow_blank=True, required=False)
     feed_leg = ChoiceField(choices=PowerOutletFeedLegChoices, allow_blank=True, required=False)
 
     class Meta:
         model = PowerOutletTemplate
         fields = "__all__"
+        validators = []
 
 
-class InterfaceTemplateSerializer(NautobotModelSerializer):
+class InterfaceTemplateSerializer(ModularDeviceComponentTemplateSerializerMixin, NautobotModelSerializer):
     type = ChoiceField(choices=InterfaceTypeChoices)
 
     class Meta:
         model = InterfaceTemplate
         fields = "__all__"
+        validators = []
 
 
-class RearPortTemplateSerializer(NautobotModelSerializer):
+class RearPortTemplateSerializer(ModularDeviceComponentTemplateSerializerMixin, NautobotModelSerializer):
     type = ChoiceField(choices=PortTypeChoices)
 
     class Meta:
         model = RearPortTemplate
         fields = "__all__"
+        validators = []
 
 
-class FrontPortTemplateSerializer(NautobotModelSerializer):
+class FrontPortTemplateSerializer(ModularDeviceComponentTemplateSerializerMixin, NautobotModelSerializer):
     type = ChoiceField(choices=PortTypeChoices)
 
     class Meta:
         model = FrontPortTemplate
         fields = "__all__"
+        validators = [
+            UniqueTogetherValidator(
+                queryset=FrontPortTemplate.objects.all(),
+                fields=("rear_port_template", "rear_port_position"),
+            ),
+        ]
 
 
 class DeviceBayTemplateSerializer(NautobotModelSerializer):
@@ -516,34 +517,21 @@ class PlatformSerializer(NautobotModelSerializer):
     class Meta:
         model = Platform
         fields = "__all__"
-        list_display_fields = [
-            "name",
-            "manufacturer",
-            "napalm_driver",
-            "napalm_args",
-            "network_driver",
-            "network_driver_mappings",
-            "device_count",
-            "virtual_machine_count",
-            "napalm_driver",
-            "description",
-        ]
 
 
-class DeviceBaySerializer(NautobotModelSerializer, TaggedModelSerializerMixin):
+class DeviceBaySerializer(TaggedModelSerializerMixin, NautobotModelSerializer):
     class Meta:
         model = DeviceBay
         fields = "__all__"
 
 
-class DeviceSerializer(NautobotModelSerializer, TaggedModelSerializerMixin):
+class DeviceSerializer(TaggedModelSerializerMixin, NautobotModelSerializer):
     face = ChoiceField(choices=DeviceFaceChoices, allow_blank=True, required=False)
     config_context = serializers.SerializerMethodField()
 
     class Meta:
         model = Device
         fields = "__all__"
-        list_display_fields = ["name", "status", "tenant", "location", "rack", "role", "device_type", "primary_ip"]
         validators = []
         extra_kwargs = {
             "parent_bay": {"required": False, "allow_null": True},
@@ -551,48 +539,6 @@ class DeviceSerializer(NautobotModelSerializer, TaggedModelSerializerMixin):
             "vc_priority": {"label": "Virtual chassis priority"},
         }
 
-        detail_view_config = {
-            "layout": [
-                {
-                    "Device": {
-                        "fields": [
-                            "name",
-                            "location",
-                            "rack",
-                            "face",
-                            "position",
-                            "tenant",
-                            "device_type",
-                            "serial",
-                            "asset_tag",
-                            "cluster",
-                            "parent_bay",
-                        ]
-                    },
-                    "Device Management": {
-                        "fields": [
-                            "role",
-                            "platform",
-                            "primary_ip4",
-                            "primary_ip6",
-                            "secrets_group",
-                            "device_redundancy_group",
-                            "device_redundancy_group_priority",
-                        ]
-                    },
-                },
-            ],
-            "include_others": False,  # TODO: config_context, local_config_context_data, local_config_context_schema
-        }
-
-    def get_additional_detail_view_tabs(self):
-        """Add "Virtual Chassis" as a separate detail tab."""
-        tabs = super().get_additional_detail_view_tabs()
-        tabs["Virtual Chassis"] = [
-            {"Virtual Chassis": {"fields": ["virtual_chassis", "vc_position", "vc_priority"]}},
-        ]
-        return tabs
-
     def get_field_names(self, declared_fields, info):
         """
         Add a couple of special fields to the serializer.
@@ -609,20 +555,55 @@ class DeviceSerializer(NautobotModelSerializer, TaggedModelSerializerMixin):
     def get_config_context(self, obj):
         return obj.get_config_context()
 
-    def validate(self, data):
+    def validate(self, attrs):
         # Validate uniqueness of (rack, position, face) since we omitted the automatically-created validator from Meta.
-        if data.get("rack") and data.get("position") and data.get("face"):
+        if attrs.get("rack") and attrs.get("position") and attrs.get("face"):
             validator = UniqueTogetherValidator(
                 queryset=Device.objects.all(),
                 fields=("rack", "position", "face"),
                 message=f"The position and face is already occupied on this rack. {UniqueTogetherValidator.message}",
             )
-            validator(data, self)
+            validator(attrs, self)
+
+        # Validate parent bay
+        if parent_bay := attrs.get("parent_bay", None):
+            if parent_bay.installed_device and parent_bay.installed_device != self.instance:
+                raise ValidationError(
+                    {
+                        "installed_device": f"Cannot install device; parent bay is already taken ({parent_bay.installed_device})"
+                    }
+                )
+
+            if self.instance:
+                parent_bay.installed_device = self.instance
+                parent_bay.full_clean()
 
         # Enforce model validation
-        super().validate(data)
+        super().validate(attrs)
 
-        return data
+        return attrs
+
+    def create(self, validated_data):
+        instance = super().create(validated_data)
+        self.update_parent_bay(validated_data, instance)
+        return instance
+
+    def update(self, instance, validated_data):
+        instance = super().update(instance, validated_data)
+        self.update_parent_bay(validated_data, instance)
+        return instance
+
+    def update_parent_bay(self, validated_data, instance):
+        update_parent_bay = "parent_bay" in validated_data.keys()
+        parent_bay = validated_data.get("parent_bay")
+        if update_parent_bay:
+            if parent_bay:
+                parent_bay.installed_device = instance
+                parent_bay.save()
+            elif hasattr(instance, "parent_bay"):
+                parent_bay = instance.parent_bay
+                parent_bay.installed_device = None
+                parent_bay.validated_save()
 
 
 class DeviceNAPALMSerializer(serializers.Serializer):
@@ -630,10 +611,11 @@ class DeviceNAPALMSerializer(serializers.Serializer):
 
 
 class ConsoleServerPortSerializer(
-    NautobotModelSerializer,
+    ModularDeviceComponentSerializerMixin,
     TaggedModelSerializerMixin,
     CableTerminationModelSerializerMixin,
     PathEndpointModelSerializerMixin,
+    NautobotModelSerializer,
 ):
     type = ChoiceField(choices=ConsolePortTypeChoices, allow_blank=True, required=False)
 
@@ -641,13 +623,15 @@ class ConsoleServerPortSerializer(
         model = ConsoleServerPort
         fields = "__all__"
         extra_kwargs = {"cable": {"read_only": True}}
+        validators = []
 
 
 class ConsolePortSerializer(
-    NautobotModelSerializer,
+    ModularDeviceComponentSerializerMixin,
     TaggedModelSerializerMixin,
     CableTerminationModelSerializerMixin,
     PathEndpointModelSerializerMixin,
+    NautobotModelSerializer,
 ):
     type = ChoiceField(choices=ConsolePortTypeChoices, allow_blank=True, required=False)
 
@@ -655,13 +639,15 @@ class ConsolePortSerializer(
         model = ConsolePort
         fields = "__all__"
         extra_kwargs = {"cable": {"read_only": True}}
+        validators = []
 
 
 class PowerOutletSerializer(
-    NautobotModelSerializer,
+    ModularDeviceComponentSerializerMixin,
     TaggedModelSerializerMixin,
     CableTerminationModelSerializerMixin,
     PathEndpointModelSerializerMixin,
+    NautobotModelSerializer,
 ):
     type = ChoiceField(choices=PowerOutletTypeChoices, allow_blank=True, required=False)
     feed_leg = ChoiceField(choices=PowerOutletFeedLegChoices, allow_blank=True, required=False)
@@ -670,13 +656,15 @@ class PowerOutletSerializer(
         model = PowerOutlet
         fields = "__all__"
         extra_kwargs = {"cable": {"read_only": True}}
+        validators = []
 
 
 class PowerPortSerializer(
-    NautobotModelSerializer,
+    ModularDeviceComponentSerializerMixin,
     TaggedModelSerializerMixin,
     CableTerminationModelSerializerMixin,
     PathEndpointModelSerializerMixin,
+    NautobotModelSerializer,
 ):
     type = ChoiceField(choices=PowerPortTypeChoices, allow_blank=True, required=False)
 
@@ -684,82 +672,103 @@ class PowerPortSerializer(
         model = PowerPort
         fields = "__all__"
         extra_kwargs = {"cable": {"read_only": True}}
+        validators = []
 
 
-class InterfaceCommonSerializer(NautobotModelSerializer, TaggedModelSerializerMixin):
-    def validate(self, data):
+class InterfaceCommonSerializer(TaggedModelSerializerMixin, NautobotModelSerializer):
+    def validate(self, attrs):
         # Validate many-to-many VLAN assignments
-        mode = data.get("mode", getattr(self.instance, "mode", None))
+        mode = attrs.get("mode", getattr(self.instance, "mode", None))
 
         if mode != InterfaceModeChoices.MODE_TAGGED:
-            if data.get("tagged_vlans"):
+            if attrs.get("tagged_vlans"):
                 raise serializers.ValidationError(
                     {
                         "tagged_vlans": f"Mode must be set to {InterfaceModeChoices.MODE_TAGGED} when specifying tagged_vlans"
                     }
                 )
 
-            if data.get("tagged_vlans") != [] and self.instance and self.instance.tagged_vlans.exists():
+            if attrs.get("tagged_vlans") != [] and self.instance and self.instance.tagged_vlans.exists():
                 raise serializers.ValidationError({"tagged_vlans": f"Clear tagged_vlans to set mode to {mode}"})
 
-        return super().validate(data)
+        return super().validate(attrs)
 
 
 class InterfaceSerializer(
-    InterfaceCommonSerializer,
+    ModularDeviceComponentSerializerMixin,
     CableTerminationModelSerializerMixin,
     PathEndpointModelSerializerMixin,
+    InterfaceCommonSerializer,
 ):
     type = ChoiceField(choices=InterfaceTypeChoices)
     mode = ChoiceField(choices=InterfaceModeChoices, allow_blank=True, required=False)
     mac_address = serializers.CharField(allow_blank=True, allow_null=True, required=False)
-    ip_address_count = serializers.IntegerField(read_only=True)
+    ip_address_count = serializers.IntegerField(read_only=True, source="_ip_address_count")
 
     class Meta:
         model = Interface
         fields = "__all__"
-        list_display_fields = ["device", "name", "status", "label", "enabled", "type", "description"]
         extra_kwargs = {"cable": {"read_only": True}}
+        validators = []
 
     def validate(self, data):
         # Validate many-to-many VLAN assignments
         device = self.instance.device if self.instance else data.get("device")
-        # TODO: after Location model replaced Site, which was not a hierarchical model, should we allow users to assign a VLAN belongs to
-        # the parent Location or the child location of `device.location`?
+        location = None
+        if device:
+            location = device.location
+        if location:
+            location_ids = location.ancestors(include_self=True).values_list("id", flat=True)
+        else:
+            location_ids = []
         for vlan in data.get("tagged_vlans", []):
-            if vlan.location not in [device.location, None]:
+            if vlan.locations.exists() and not vlan.locations.filter(pk__in=location_ids).exists():
                 raise serializers.ValidationError(
                     {
-                        "tagged_vlans": f"VLAN {vlan} must belong to the same location as the interface's parent device, or "
-                        f"it must be global."
+                        "tagged_vlans": f"VLAN {vlan} must have the same location as the interface's parent device, "
+                        f"or is in one of the parents of the interface's parent device's location, or it must be global."
                     }
                 )
 
         return super().validate(data)
 
 
-class RearPortSerializer(NautobotModelSerializer, TaggedModelSerializerMixin, CableTerminationModelSerializerMixin):
+class RearPortSerializer(
+    ModularDeviceComponentSerializerMixin,
+    TaggedModelSerializerMixin,
+    CableTerminationModelSerializerMixin,
+    NautobotModelSerializer,
+):
     type = ChoiceField(choices=PortTypeChoices)
 
     class Meta:
         model = RearPort
         fields = "__all__"
         extra_kwargs = {"cable": {"read_only": True}}
+        validators = []
 
 
-class FrontPortSerializer(NautobotModelSerializer, TaggedModelSerializerMixin, CableTerminationModelSerializerMixin):
+class FrontPortSerializer(
+    ModularDeviceComponentSerializerMixin,
+    TaggedModelSerializerMixin,
+    CableTerminationModelSerializerMixin,
+    NautobotModelSerializer,
+):
     type = ChoiceField(choices=PortTypeChoices)
 
     class Meta:
         model = FrontPort
         fields = "__all__"
         extra_kwargs = {"cable": {"read_only": True}}
+        validators = [
+            UniqueTogetherValidator(
+                queryset=FrontPort.objects.all(),
+                fields=("rear_port", "rear_port_position"),
+            ),
+        ]
 
 
-class DeviceRedundancyGroupSerializer(
-    NautobotModelSerializer,
-    TaggedModelSerializerMixin,
-):
+class DeviceRedundancyGroupSerializer(TaggedModelSerializerMixin, NautobotModelSerializer):
     failover_strategy = ChoiceField(
         choices=DeviceRedundancyGroupFailoverStrategyChoices,
         allow_blank=True,
@@ -769,7 +778,6 @@ class DeviceRedundancyGroupSerializer(
     class Meta:
         model = DeviceRedundancyGroup
         fields = "__all__"
-        list_display_fields = ["name", "status", "failover_strategy", "device_count"]
 
 
 #
@@ -777,25 +785,25 @@ class DeviceRedundancyGroupSerializer(
 #
 
 
-class InventoryItemSerializer(NautobotModelSerializer, TaggedModelSerializerMixin, TreeModelSerializerMixin):
+class InventoryItemSerializer(TaggedModelSerializerMixin, TreeModelSerializerMixin, NautobotModelSerializer):
     class Meta:
         model = InventoryItem
         fields = "__all__"
         # https://www.django-rest-framework.org/api-guide/validators/#optional-fields
         validators = []
 
-    def validate(self, data):
+    def validate(self, attrs):
         # Validate uniqueness of (device, parent, name) since we omitted the automatically created validator from Meta.
-        if data.get("device") and data.get("parent") and data.get("name"):
+        if attrs.get("device") and attrs.get("parent") and attrs.get("name"):
             validator = UniqueTogetherValidator(
                 queryset=InventoryItem.objects.all(),
                 fields=("device", "parent", "name"),
             )
-            validator(data, self)
+            validator(attrs, self)
 
-        super().validate(data)
+        super().validate(attrs)
 
-        return data
+        return attrs
 
 
 #
@@ -803,10 +811,7 @@ class InventoryItemSerializer(NautobotModelSerializer, TaggedModelSerializerMixi
 #
 
 
-class CableSerializer(
-    NautobotModelSerializer,
-    TaggedModelSerializerMixin,
-):
+class CableSerializer(TaggedModelSerializerMixin, NautobotModelSerializer):
     # TODO: termination_a_type/termination_b_type are a bit redundant with the full termination_a/termination_b dicts
     termination_a_type = ContentTypeField(queryset=ContentType.objects.filter(CABLE_TERMINATION_MODELS))
     termination_b_type = ContentTypeField(queryset=ContentType.objects.filter(CABLE_TERMINATION_MODELS))
@@ -821,13 +826,6 @@ class CableSerializer(
         extra_kwargs = {
             "color": {"help_text": "RGB color in hexadecimal (e.g. 00ff00)"},
         }
-        list_display_fields = [
-            "label",
-            "termination_a",
-            "termination_b",
-            "status",
-            "type",
-        ]
 
     def _get_termination(self, obj, side):
         """
@@ -962,13 +960,12 @@ class InterfaceConnectionSerializer(ValidatedModelSerializer):
 #
 
 
-class VirtualChassisSerializer(NautobotModelSerializer, TaggedModelSerializerMixin):
+class VirtualChassisSerializer(TaggedModelSerializerMixin, NautobotModelSerializer):
     member_count = serializers.IntegerField(read_only=True)
 
     class Meta:
         model = VirtualChassis
         fields = "__all__"
-        list_display_fields = ["name", "domain", "master", "member_count"]
 
 
 #
@@ -976,7 +973,9 @@ class VirtualChassisSerializer(NautobotModelSerializer, TaggedModelSerializerMix
 #
 
 
-class PowerPanelSerializer(NautobotModelSerializer, TaggedModelSerializerMixin):
+class PowerPanelSerializer(TaggedModelSerializerMixin, NautobotModelSerializer):
+    panel_type = ChoiceField(choices=PowerPanelTypeChoices, allow_blank=True, required=False)
+    power_path = ChoiceField(choices=PowerPathChoices, allow_blank=True, required=False)
     power_feed_count = serializers.IntegerField(read_only=True)
 
     class Meta:
@@ -985,15 +984,165 @@ class PowerPanelSerializer(NautobotModelSerializer, TaggedModelSerializerMixin):
 
 
 class PowerFeedSerializer(
-    NautobotModelSerializer,
     TaggedModelSerializerMixin,
     CableTerminationModelSerializerMixin,
     PathEndpointModelSerializerMixin,
+    NautobotModelSerializer,
 ):
     type = ChoiceField(choices=PowerFeedTypeChoices, default=PowerFeedTypeChoices.TYPE_PRIMARY)
+    power_path = ChoiceField(choices=PowerPathChoices, allow_blank=True, required=False)
     supply = ChoiceField(choices=PowerFeedSupplyChoices, default=PowerFeedSupplyChoices.SUPPLY_AC)
     phase = ChoiceField(choices=PowerFeedPhaseChoices, default=PowerFeedPhaseChoices.PHASE_SINGLE)
+    breaker_pole_count = ChoiceField(
+        choices=PowerFeedBreakerPoleChoices, allow_blank=True, allow_null=True, required=False
+    )
 
     class Meta:
         model = PowerFeed
         fields = "__all__"
+
+
+#
+# Software image files
+#
+
+
+class SoftwareImageFileSerializer(TaggedModelSerializerMixin, NautobotModelSerializer):
+    class Meta:
+        model = SoftwareImageFile
+        fields = "__all__"
+
+
+class SoftwareVersionSerializer(TaggedModelSerializerMixin, NautobotModelSerializer):
+    class Meta:
+        model = SoftwareVersion
+        fields = "__all__"
+
+
+class DeviceTypeToSoftwareImageFileSerializer(ValidatedModelSerializer):
+    class Meta:
+        model = DeviceTypeToSoftwareImageFile
+        fields = "__all__"
+
+
+class ControllerSerializer(TaggedModelSerializerMixin, NautobotModelSerializer):
+    capabilities = serializers.ListField(
+        child=ChoiceField(choices=ControllerCapabilitiesChoices, required=False), allow_empty=True, required=False
+    )
+
+    class Meta:
+        model = Controller
+        fields = "__all__"
+
+
+class ControllerManagedDeviceGroupSerializer(TaggedModelSerializerMixin, NautobotModelSerializer):
+    capabilities = serializers.ListField(
+        child=ChoiceField(choices=ControllerCapabilitiesChoices, required=False), allow_empty=True, required=False
+    )
+
+    class Meta:
+        model = ControllerManagedDeviceGroup
+        fields = "__all__"
+
+
+#
+# Modules
+#
+
+
+class ModuleBaySerializer(TaggedModelSerializerMixin, NautobotModelSerializer):
+    class Meta:
+        model = ModuleBay
+        fields = "__all__"
+        validators = []
+
+    def validate(self, attrs):
+        """Validate device and module field constraints for module bay."""
+        if attrs.get("parent_device") and attrs.get("parent_module"):
+            raise serializers.ValidationError("Only one of parent_device or parent_module must be set")
+        if attrs.get("parent_device"):
+            validator = UniqueTogetherValidator(
+                queryset=self.Meta.model.objects.all(), fields=("parent_device", "name")
+            )
+            validator(attrs, self)
+        if attrs.get("parent_module"):
+            validator = UniqueTogetherValidator(
+                queryset=self.Meta.model.objects.all(), fields=("parent_module", "name")
+            )
+            validator(attrs, self)
+        return super().validate(attrs)
+
+
+class ModuleBayTemplateSerializer(NautobotModelSerializer):
+    class Meta:
+        model = ModuleBayTemplate
+        fields = "__all__"
+        validators = []
+
+    def validate(self, attrs):
+        """Validate device_type and module_type field constraints for module bay template."""
+        if attrs.get("device_type") and attrs.get("module_type"):
+            raise serializers.ValidationError("Only one of device_type or module_type must be set")
+        if attrs.get("device_type"):
+            validator = UniqueTogetherValidator(queryset=self.Meta.model.objects.all(), fields=("device_type", "name"))
+            validator(attrs, self)
+        if attrs.get("module_type"):
+            validator = UniqueTogetherValidator(queryset=self.Meta.model.objects.all(), fields=("module_type", "name"))
+            validator(attrs, self)
+        return super().validate(attrs)
+
+
+class ModuleSerializer(TaggedModelSerializerMixin, NautobotModelSerializer):
+    class Meta:
+        model = Module
+        fields = "__all__"
+
+        validators = []
+
+    def validate(self, attrs):
+        """Validate asset_Tag, serial, parent_module_bay and location field constraints for module."""
+        if attrs.get("parent_module_bay") and attrs.get("location"):
+            raise serializers.ValidationError("Only one of parent_module_bay or location must be set")
+        if attrs.get("serial"):
+            validator = UniqueTogetherValidator(queryset=Module.objects.all(), fields=("module_type", "serial"))
+            validator(attrs, self)
+        if attrs.get("asset_tag"):
+            validator = UniqueValidator(queryset=Module.objects.all())
+            validator(attrs["asset_tag"], self.fields["asset_tag"])
+        return super().validate(attrs)
+
+
+class ModuleTypeSerializer(TaggedModelSerializerMixin, NautobotModelSerializer):
+    class Meta:
+        model = ModuleType
+        fields = "__all__"
+
+
+class VirtualDeviceContextSerializer(TaggedModelSerializerMixin, NautobotModelSerializer):
+    class Meta:
+        model = VirtualDeviceContext
+        fields = "__all__"
+
+    def validate(self, attrs):
+        """Validate device cannot be changed for VirtualDeviceContext."""
+        if attrs.get("device") and self.instance and self.instance.device != attrs.get("device"):
+            raise serializers.ValidationError("Changing the device of a VirtualDeviceContext is not allowed.")
+        return super().validate(attrs)
+
+
+class InterfaceVDCAssignmentSerializer(ValidatedModelSerializer):
+    class Meta:
+        model = InterfaceVDCAssignment
+        fields = "__all__"
+
+
+class ModuleFamilySerializer(NautobotModelSerializer):
+    """API serializer for ModuleFamily objects."""
+
+    url = serializers.HyperlinkedIdentityField(view_name="dcim-api:modulefamily-detail")
+    module_type_count = serializers.IntegerField(read_only=True)
+    module_bay_count = serializers.IntegerField(read_only=True)
+
+    class Meta:
+        model = ModuleFamily
+        fields = "__all__"
