@@ -1,6 +1,5 @@
 from nautobot.core.choices import ChoiceSet
 
-
 #
 # Locations
 #
@@ -22,6 +21,20 @@ class LocationStatusChoices(ChoiceSet):
     )
 
 
+class LocationDataToContactActionChoices(ChoiceSet):
+    ASSOCIATE_EXISTING_CONTACT = "associate existing contact"
+    ASSOCIATE_EXISTING_TEAM = "associate existing team"
+    CREATE_AND_ASSIGN_NEW_CONTACT = "create and assign new contact"
+    CREATE_AND_ASSIGN_NEW_TEAM = "create and assign new team"
+
+    CHOICES = (
+        (ASSOCIATE_EXISTING_CONTACT, "Associate to existing contact"),
+        (ASSOCIATE_EXISTING_TEAM, "Associate to existing team"),
+        (CREATE_AND_ASSIGN_NEW_CONTACT, "Create and assign new contact"),
+        (CREATE_AND_ASSIGN_NEW_TEAM, "Create and assign new team"),
+    )
+
+
 #
 # Racks
 #
@@ -223,6 +236,10 @@ class PowerPortTypeChoices(ChoiceSet):
     TYPE_IEC_3PNE4H = "iec-60309-3p-n-e-4h"
     TYPE_IEC_3PNE6H = "iec-60309-3p-n-e-6h"
     TYPE_IEC_3PNE9H = "iec-60309-3p-n-e-9h"
+    # IEC 60906-1
+    TYPE_IEC_60906_1 = "iec-60906-1"
+    TYPE_NBR_14136_10A = "nbr-14136-10a"
+    TYPE_NBR_14136_20A = "nbr-14136-20a"
     # NEMA non-locking
     TYPE_NEMA_115P = "nema-1-15p"
     TYPE_NEMA_515P = "nema-5-15p"
@@ -340,6 +357,14 @@ class PowerPortTypeChoices(ChoiceSet):
                 (TYPE_IEC_3PNE9H, "3P+N+E 9H"),
             ),
         ),
+        (
+            "IEC 60906-1",
+            (
+                (TYPE_IEC_60906_1, "IEC 60906-1"),
+                (TYPE_NBR_14136_10A, "2P+T 10A (NBR 14136)"),
+                (TYPE_NBR_14136_20A, "2P+T 20A (NBR 14136)"),
+            ),
+        ),
         (
             "NEMA (Non-locking)",
             (
@@ -483,6 +508,10 @@ class PowerOutletTypeChoices(ChoiceSet):
     TYPE_IEC_3PNE4H = "iec-60309-3p-n-e-4h"
     TYPE_IEC_3PNE6H = "iec-60309-3p-n-e-6h"
     TYPE_IEC_3PNE9H = "iec-60309-3p-n-e-9h"
+    # IEC 60906-1
+    TYPE_IEC_60906_1 = "iec-60906-1"
+    TYPE_NBR_14136_10A = "nbr-14136-10a"
+    TYPE_NBR_14136_20A = "nbr-14136-20a"
     # NEMA non-locking
     TYPE_NEMA_115R = "nema-1-15r"
     TYPE_NEMA_515R = "nema-5-15r"
@@ -560,6 +589,7 @@ class PowerOutletTypeChoices(ChoiceSet):
     TYPE_NEUTRIK_POWERCON_TRUE1 = "neutrik-powercon-true1"
     TYPE_NEUTRIK_POWERCON_TRUE1_TOP = "neutrik-powercon-true1-top"
     TYPE_UBIQUITI_SMARTPOWER = "ubiquiti-smartpower"
+    TYPE_EATON_C39 = "eaton-c39"
     # Other
     TYPE_HARDWIRED = "hardwired"
     TYPE_OTHER = "other"
@@ -593,6 +623,14 @@ class PowerOutletTypeChoices(ChoiceSet):
                 (TYPE_IEC_3PNE9H, "3P+N+E 9H"),
             ),
         ),
+        (
+            "IEC 60906-1",
+            (
+                (TYPE_IEC_60906_1, "IEC 60906-1"),
+                (TYPE_NBR_14136_10A, "2P+T 10A (NBR 14136)"),
+                (TYPE_NBR_14136_20A, "2P+T 20A (NBR 14136)"),
+            ),
+        ),
         (
             "NEMA (Non-locking)",
             (
@@ -691,6 +729,7 @@ class PowerOutletTypeChoices(ChoiceSet):
                 (TYPE_NEUTRIK_POWERCON_TRUE1, "Neutrik powerCON TRUE1"),
                 (TYPE_NEUTRIK_POWERCON_TRUE1_TOP, "Neutrik powerCON TRUE1 TOP"),
                 (TYPE_UBIQUITI_SMARTPOWER, "Ubiquiti SmartPower"),
+                (TYPE_EATON_C39, "Eaton C39"),
             ),
         ),
         (
@@ -751,12 +790,25 @@ class InterfaceTypeChoices(ChoiceSet):
     TYPE_100GE_CFP4 = "100gbase-x-cfp4"
     TYPE_100GE_CPAK = "100gbase-x-cpak"
     TYPE_100GE_QSFP28 = "100gbase-x-qsfp28"
+    TYPE_100GE_CXP = "100gbase-x-cxp"
+    TYPE_100GE_DSFP = "100gbase-x-dsfp"
+    TYPE_100GE_SFP_DD = "100gbase-x-sfpdd"
+    TYPE_100GE_QSFP_DD = "100gbase-x-qsfpdd"
     TYPE_200GE_CFP2 = "200gbase-x-cfp2"
     TYPE_200GE_QSFP56 = "200gbase-x-qsfp56"
+    TYPE_200GE_QSFP_DD = "200gbase-x-qsfpdd"
+    TYPE_400GE_QSFP112 = "400gbase-x-qsfp112"
     TYPE_400GE_QSFP_DD = "400gbase-x-qsfpdd"
     TYPE_400GE_OSFP = "400gbase-x-osfp"
+    TYPE_400GE_CFP2 = "400gbase-x-cfp2"
+    TYPE_400GE_OSFP_RHS = "400gbase-x-osfp-rhs"
+    TYPE_400GE_CDFP = "400gbase-x-cdfp"
+    TYPE_400GE_CFP8 = "400gbase-x-cfp8"
     TYPE_800GE_QSFP_DD = "800gbase-x-qsfpdd"
     TYPE_800GE_OSFP = "800gbase-x-osfp"
+    TYPE_800GE_OSFP_XD = "800gbase-x-osfp-xd"
+    TYPE_1600GE_OSFP = "1600gbase-x-osfp"
+    TYPE_1600GE_OSFP_XD = "1600gbase-x-osfp-xd"
 
     # Ethernet Backplane
     TYPE_1GE_KX = "1000base-kx"
@@ -801,7 +853,10 @@ class InterfaceTypeChoices(ChoiceSet):
     TYPE_8GFC_SFP_PLUS = "8gfc-sfpp"
     TYPE_16GFC_SFP_PLUS = "16gfc-sfpp"
     TYPE_32GFC_SFP28 = "32gfc-sfp28"
+    TYPE_32GFC_SFP_PLUS = "32gfc-sfpp"
     TYPE_64GFC_QSFP_PLUS = "64gfc-qsfpp"
+    TYPE_64GFC_SFP_DD = "64gfc-sfpdd"
+    TYPE_64GFC_SFP_PLUS = "64gfc-sfpp"
     TYPE_128GFC_QSFP28 = "128gfc-sfp28"
 
     # InfiniBand
@@ -820,6 +875,22 @@ class InterfaceTypeChoices(ChoiceSet):
     TYPE_E1 = "e1"
     TYPE_T3 = "t3"
     TYPE_E3 = "e3"
+    TYPE_DA15 = "da15"
+    TYPE_DA26 = "da26"
+    TYPE_DA31 = "da31"
+    TYPE_DB25 = "db25"
+    TYPE_DB44 = "db44"
+    TYPE_DB60 = "db60"
+    TYPE_DC37 = "dc37"
+    TYPE_DC62 = "dc62"
+    TYPE_DC79 = "dc79"
+    TYPE_DD50 = "dd50"
+    TYPE_DD78 = "dd78"
+    TYPE_DD100 = "dd100"
+    TYPE_DE9 = "de9"
+    TYPE_DE15 = "de15"
+    TYPE_DE19 = "de19"
+    TYPE_DF104 = "df104"
 
     # ATM/DSL
     TYPE_XDSL = "xdsl"
@@ -893,14 +964,27 @@ class InterfaceTypeChoices(ChoiceSet):
                 (TYPE_100GE_CFP, "CFP (100GE)"),
                 (TYPE_100GE_CFP2, "CFP2 (100GE)"),
                 (TYPE_200GE_CFP2, "CFP2 (200GE)"),
+                (TYPE_400GE_CFP2, "CFP2 (400GE)"),
                 (TYPE_100GE_CFP4, "CFP4 (100GE)"),
                 (TYPE_100GE_CPAK, "Cisco CPAK (100GE)"),
                 (TYPE_100GE_QSFP28, "QSFP28 (100GE)"),
+                (TYPE_100GE_CXP, "CXP (100GE)"),
+                (TYPE_100GE_QSFP_DD, "QSFP-DD (100GE)"),
+                (TYPE_100GE_DSFP, "DSFP (100GE)"),
+                (TYPE_100GE_SFP_DD, "SFP-DD (100GE)"),
                 (TYPE_200GE_QSFP56, "QSFP56 (200GE)"),
+                (TYPE_200GE_QSFP_DD, "QSFP-DD (200GE)"),
+                (TYPE_400GE_QSFP112, "QSFP112 (400GE)"),
                 (TYPE_400GE_QSFP_DD, "QSFP-DD (400GE)"),
                 (TYPE_400GE_OSFP, "OSFP (400GE)"),
+                (TYPE_400GE_OSFP_RHS, "OSFP-RHS (400GE)"),
+                (TYPE_400GE_CDFP, "CDFP (400GE)"),
+                (TYPE_400GE_CFP8, "CPF8 (400GE)"),
                 (TYPE_800GE_QSFP_DD, "QSFP-DD (800GE)"),
                 (TYPE_800GE_OSFP, "OSFP (800GE)"),
+                (TYPE_800GE_OSFP_XD, "OSFP-XD (800GE)"),
+                (TYPE_1600GE_OSFP, "OSFP (1600GE)"),
+                (TYPE_1600GE_OSFP_XD, "OSFP-XD (1600GE)"),
             ),
         ),
         (
@@ -960,7 +1044,10 @@ class InterfaceTypeChoices(ChoiceSet):
                 (TYPE_8GFC_SFP_PLUS, "SFP+ (8GFC)"),
                 (TYPE_16GFC_SFP_PLUS, "SFP+ (16GFC)"),
                 (TYPE_32GFC_SFP28, "SFP28 (32GFC)"),
+                (TYPE_32GFC_SFP_PLUS, "SFP+ (32GFC)"),
                 (TYPE_64GFC_QSFP_PLUS, "QSFP+ (64GFC)"),
+                (TYPE_64GFC_SFP_DD, "SFP-DD (64GFC)"),
+                (TYPE_64GFC_SFP_PLUS, "SFP+ (64GFC)"),
                 (TYPE_128GFC_QSFP28, "QSFP28 (128GFC)"),
             ),
         ),
@@ -985,6 +1072,22 @@ class InterfaceTypeChoices(ChoiceSet):
                 (TYPE_E1, "E1 (2.048 Mbps)"),
                 (TYPE_T3, "T3 (45 Mbps)"),
                 (TYPE_E3, "E3 (34 Mbps)"),
+                (TYPE_DA15, "DA15"),
+                (TYPE_DA26, "DA26"),
+                (TYPE_DA31, "DA31"),
+                (TYPE_DB25, "DB25"),
+                (TYPE_DB44, "DB44"),
+                (TYPE_DB60, "DB60"),
+                (TYPE_DC37, "DC37"),
+                (TYPE_DC62, "DC62"),
+                (TYPE_DC79, "DC79"),
+                (TYPE_DD50, "DD50"),
+                (TYPE_DD78, "DD78"),
+                (TYPE_DD100, "DD100"),
+                (TYPE_DE9, "DE9"),
+                (TYPE_DE15, "DE15"),
+                (TYPE_DE19, "DE19"),
+                (TYPE_DF104, "DF104"),
             ),
         ),
         ("ATM", ((TYPE_XDSL, "xDSL"),)),
@@ -1091,6 +1194,10 @@ class PortTypeChoices(ChoiceSet):
     TYPE_LSH_PC = "lsh-pc"
     TYPE_LSH_UPC = "lsh-upc"
     TYPE_LSH_APC = "lsh-apc"
+    TYPE_LX5 = "lx5"
+    TYPE_LX5_PC = "lx5-pc"
+    TYPE_LX5_UPC = "lx5-upc"
+    TYPE_LX5_APC = "lx5-apc"
     TYPE_SPLICE = "splice"
     TYPE_CS = "cs"
     TYPE_SN = "sn"
@@ -1137,6 +1244,10 @@ class PortTypeChoices(ChoiceSet):
                 (TYPE_LSH_PC, "LSH/PC"),
                 (TYPE_LSH_UPC, "LSH/UPC"),
                 (TYPE_LSH_APC, "LSH/APC"),
+                (TYPE_LX5, "LX.5"),
+                (TYPE_LX5_PC, "LX.5/PC"),
+                (TYPE_LX5_UPC, "LX.5/UPC"),
+                (TYPE_LX5_APC, "LX.5/APC"),
                 (TYPE_MPO, "MPO"),
                 (TYPE_MTRJ, "MTRJ"),
                 (TYPE_SC, "SC"),
@@ -1181,6 +1292,7 @@ class CableTypeChoices(ChoiceSet):
     TYPE_MMF_OM2 = "mmf-om2"
     TYPE_MMF_OM3 = "mmf-om3"
     TYPE_MMF_OM4 = "mmf-om4"
+    TYPE_MMF_OM5 = "mmf-om5"
     TYPE_SMF = "smf"
     TYPE_SMF_OS1 = "smf-os1"
     TYPE_SMF_OS2 = "smf-os2"
@@ -1214,6 +1326,7 @@ class CableTypeChoices(ChoiceSet):
                 (TYPE_MMF_OM2, "Multimode Fiber (OM2)"),
                 (TYPE_MMF_OM3, "Multimode Fiber (OM3)"),
                 (TYPE_MMF_OM4, "Multimode Fiber (OM4)"),
+                (TYPE_MMF_OM5, "Multimode Fiber (OM5)"),
                 (TYPE_SMF, "Singlemode Fiber"),
                 (TYPE_SMF_OS1, "Singlemode Fiber (OS1)"),
                 (TYPE_SMF_OS2, "Singlemode Fiber (OS2)"),
@@ -1258,6 +1371,37 @@ class CableLengthUnitChoices(ChoiceSet):
     )
 
 
+#
+# Power Panels
+#
+
+
+class PowerPanelTypeChoices(ChoiceSet):
+    TYPE_UTILITY = "utility"
+    TYPE_GENERATOR = "generator"
+    TYPE_SWITCHGEAR = "switchgear"
+    TYPE_MDP = "mdp"
+    TYPE_UPS = "ups"
+    TYPE_TRANSFER_SWITCH = "transfer-switch"
+    TYPE_PDU = "pdu"
+    TYPE_PANELBOARD = "panelboard"
+    TYPE_MLC = "mlc"
+    TYPE_RPP = "rpp"
+
+    CHOICES = (
+        (TYPE_UTILITY, "Utility"),
+        (TYPE_GENERATOR, "Generator"),
+        (TYPE_SWITCHGEAR, "Switchgear"),
+        (TYPE_MDP, "Main Distribution Panel (MDP)"),
+        (TYPE_UPS, "Uninterruptible Power Supply (UPS)"),
+        (TYPE_TRANSFER_SWITCH, "Transfer Switch (TS)"),
+        (TYPE_PDU, "Power Distribution Unit (PDU)"),
+        (TYPE_PANELBOARD, "Panelboard"),
+        (TYPE_MLC, "Mini Load Center (MLC)"),
+        (TYPE_RPP, "Remote Power Panel (RPP)"),
+    )
+
+
 #
 # PowerFeeds
 #
@@ -1292,6 +1436,16 @@ class PowerFeedTypeChoices(ChoiceSet):
     }
 
 
+class PowerPathChoices(ChoiceSet):
+    PATH_A = "a"
+    PATH_B = "b"
+
+    CHOICES = (
+        (PATH_A, "Path A"),
+        (PATH_B, "Path B"),
+    )
+
+
 class PowerFeedSupplyChoices(ChoiceSet):
     SUPPLY_AC = "ac"
     SUPPLY_DC = "dc"
@@ -1312,6 +1466,18 @@ class PowerFeedPhaseChoices(ChoiceSet):
     )
 
 
+class PowerFeedBreakerPoleChoices(ChoiceSet):
+    POLE_1 = 1
+    POLE_2 = 2
+    POLE_3 = 3
+
+    CHOICES = [
+        (POLE_1, "1-Pole"),
+        (POLE_2, "2-Pole"),
+        (POLE_3, "3-Pole"),
+    ]
+
+
 #
 # Device Redundancy Groups
 #
@@ -1378,3 +1544,109 @@ class InterfaceRedundancyGroupProtocolChoices(ChoiceSet):
         (GLBP, "GLBP"),
         (CARP, "CARP"),
     )
+
+
+#
+# Software Image Files
+#
+
+
+class SoftwareImageFileHashingAlgorithmChoices(ChoiceSet):
+    MD5 = "md5"
+    SHA1 = "sha1"
+    SHA224 = "sha224"
+    SHA384 = "sha384"
+    SHA256 = "sha256"
+    SHA512 = "sha512"
+    SHA3 = "sha3"
+    BLAKE2 = "blake2"
+    BLAKE3 = "blake3"
+
+    CHOICES = (
+        (MD5, "MD5"),
+        (SHA1, "SHA1"),
+        (SHA224, "SHA224"),
+        (SHA384, "SHA384"),
+        (SHA256, "SHA256"),
+        (SHA512, "SHA512"),
+        (SHA3, "SHA3"),
+        (BLAKE2, "BLAKE2"),
+        (BLAKE3, "BLAKE3"),
+    )
+
+
+class SoftwareImageFileStatusChoices(ChoiceSet):
+    STATUS_ACTIVE = "active"
+    STATUS_EXTENDED_SUPPORT = "extended-support"
+    STATUS_END_OF_LIFE = "end-of-life"
+
+    CHOICES = (
+        (STATUS_ACTIVE, "Active"),
+        (STATUS_EXTENDED_SUPPORT, "Extended Support"),
+        (STATUS_END_OF_LIFE, "End-of-Life"),
+    )
+
+
+class SoftwareVersionStatusChoices(ChoiceSet):
+    STATUS_ACTIVE = "active"
+    STATUS_EXTENDED_SUPPORT = "extended-support"
+    STATUS_END_OF_LIFE = "end-of-life"
+
+    CHOICES = (
+        (STATUS_ACTIVE, "Active"),
+        (STATUS_EXTENDED_SUPPORT, "Extended Support"),
+        (STATUS_END_OF_LIFE, "End-of-Life"),
+    )
+
+
+#
+# Modules
+#
+
+
+class ModuleStatusChoices(ChoiceSet):
+    STATUS_OFFLINE = "offline"
+    STATUS_ACTIVE = "active"
+    STATUS_PLANNED = "planned"
+    STATUS_STAGED = "staged"
+    STATUS_FAILED = "failed"
+    STATUS_INVENTORY = "inventory"
+    STATUS_DECOMMISSIONING = "decommissioning"
+
+    CHOICES = (
+        (STATUS_OFFLINE, "Offline"),
+        (STATUS_ACTIVE, "Active"),
+        (STATUS_PLANNED, "Planned"),
+        (STATUS_STAGED, "Staged"),
+        (STATUS_FAILED, "Failed"),
+        (STATUS_INVENTORY, "Inventory"),
+        (STATUS_DECOMMISSIONING, "Decommissioning"),
+    )
+
+
+#
+# Virtual Device Context
+#
+
+
+class VirtualDeviceContextStatusChoices(ChoiceSet):
+    STATUS_OFFLINE = "offline"
+    STATUS_ACTIVE = "active"
+    STATUS_PLANNED = "planned"
+
+    CHOICES = (
+        (STATUS_OFFLINE, "Offline"),
+        (STATUS_ACTIVE, "Active"),
+        (STATUS_PLANNED, "Planned"),
+    )
+
+
+#
+# Controllers
+#
+
+
+class ControllerCapabilitiesChoices(ChoiceSet):
+    WIRELESS = "wireless"
+
+    CHOICES = ((WIRELESS, "Wireless"),)
