@@ -2,22 +2,26 @@ import logging
 
 from django.contrib.contenttypes.models import ContentType
 from django.core.exceptions import ValidationError
-from django.db.models.signals import m2m_changed, post_save, pre_delete
 from django.db import transaction
+from django.db.models.signals import m2m_changed, post_save, pre_delete
 from django.dispatch import receiver
 
 from nautobot.core.signals import disable_for_loaddata
+
 from .models import (
     Cable,
     CablePath,
+    ControllerManagedDeviceGroup,
     Device,
     DeviceRedundancyGroup,
+    Interface,
+    InterfaceVDCAssignment,
+    LocationType,
     PathEndpoint,
     PowerPanel,
     Rack,
     RackGroup,
     VirtualChassis,
-    Interface,
 )
 from .utils import validate_interface_tagged_vlans
 
@@ -285,3 +289,95 @@ def prevent_adding_tagged_vlans_with_incorrect_mode_or_site(sender, instance, ac
         return
 
     validate_interface_tagged_vlans(instance, kwargs["model"], kwargs["pk_set"])
+
+
+#
+# VirtualDeviceContext Interfaces
+#
+
+
+@receiver(m2m_changed, sender=InterfaceVDCAssignment)
+@disable_for_loaddata
+def validate_vdcs_interface_relationships(sender, instance, action, **kwargs):
+    if action != "pre_add":
+        return
+
+    pk_set = kwargs.get("pk_set", [])
+    if isinstance(instance, Interface):
+        invalid_vdcs = kwargs["model"].objects.filter(pk__in=pk_set).exclude(device=instance.device)
+        if invalid_vdcs.count():
+            raise ValidationError(
+                {
+                    "virtual_device_contexts": (
+                        f"Virtual Device Context with names {list(invalid_vdcs.values_list('name', flat=True))} must all belong to the "
+                        f"same device as the interface's device."
+                    )
+                }
+            )
+    else:
+        vc_interfaces_ids = instance.device.vc_interfaces.values_list("pk", flat=True)
+        invalid_interfaces = (
+            kwargs["model"]
+            .objects.filter(pk__in=pk_set)
+            .exclude(device=instance.device)
+            .exclude(id__in=vc_interfaces_ids)
+        )
+
+        if invalid_interfaces.count():
+            raise ValidationError(
+                {
+                    "interfaces": (
+                        f"Interfaces with names {list(invalid_interfaces.values_list('name', flat=True))} must all belong to the "
+                        f"same device as the Virtual Device Context's device."
+                    )
+                }
+            )
+
+
+#
+# ControllerManagedDeviceGroup
+#
+
+
+@receiver(post_save, sender=ControllerManagedDeviceGroup)
+def handle_controller_managed_device_group_controller_change(instance, raw=False, **_):
+    """Update descendants when the top level `ControllerManagedDeviceGroup.controller` changes."""
+    logger = logging.getLogger(__name__ + ".ControllerManagedDeviceGroup")
+
+    if raw:
+        logger.debug("Skipping controller update for imported controller device group %s", instance)
+        return
+
+    if instance.parent or instance._original_controller == instance.controller:
+        return
+
+    with transaction.atomic():
+        for group in instance.descendants(include_self=False):
+            group.controller = instance.controller
+            group.save()
+            logger.debug("Updated controller from parent %s for child %s", instance, group)
+
+
+@receiver(m2m_changed, sender=LocationType.content_types.through)
+def content_type_changed(instance, action, **kwargs):
+    """
+    Prevents removal of a ContentType from LocationType if it's in use by any models
+    associated with the locations.
+    """
+
+    if action != "pre_remove":
+        return
+
+    removed_content_types = ContentType.objects.filter(pk__in=kwargs.get("pk_set", []))
+
+    for content_type in removed_content_types:
+        model_class = content_type.model_class()
+
+        if model_class.objects.filter(location__location_type=instance).exists():
+            raise ValidationError(
+                {
+                    "content_types": (
+                        f"Cannot remove the content type {content_type} as currently at least one {model_class._meta.verbose_name} is associated to a location of this location type. "
+                    )
+                }
+            )
