@@ -2,11 +2,13 @@ from .git import (
     enqueue_git_repository_diff_origin_and_local,
     enqueue_pull_git_repository_and_refresh_data,
     ensure_git_repository,
+    get_repo_access_url,
     git_repository_dry_run,
+    refresh_job_code_from_repository,
 )
 from .registry import (
-    get_datasource_contents,
     get_datasource_content_choices,
+    get_datasource_contents,
     refresh_datasource_content,
 )
 
@@ -16,6 +18,8 @@ __all__ = (
     "ensure_git_repository",
     "get_datasource_content_choices",
     "get_datasource_contents",
+    "get_repo_access_url",
     "git_repository_dry_run",
     "refresh_datasource_content",
+    "refresh_job_code_from_repository",
 )
