@@ -1,50 +1,63 @@
 from .change_logging import ChangeLoggedModel, ObjectChange
-from .roles import Role, RoleField
-from .statuses import Status, StatusField, StatusModel
+from .contacts import Contact, ContactAssociation, Team
 from .customfields import ComputedField, CustomField, CustomFieldChoice, CustomFieldModel
 from .datasources import GitRepository
-from .groups import DynamicGroup, DynamicGroupMembership
+from .groups import DynamicGroup, DynamicGroupMembership, StaticGroupAssociation
 from .jobs import (
     Job,
     JobButton,
     JobHook,
     JobLogEntry,
+    JobQueue,
+    JobQueueAssignment,
     JobResult,
     ScheduledJob,
     ScheduledJobs,
 )
+from .metadata import MetadataChoice, MetadataType, ObjectMetadata
+from .mixins import ContactMixin, DynamicGroupMixin, DynamicGroupsModelMixin, SavedViewMixin
 from .models import (
     ConfigContext,
     ConfigContextModel,
     ConfigContextSchema,
     CustomLink,
     ExportTemplate,
+    ExternalIntegration,
     FileAttachment,
     FileProxy,
     GraphQLQuery,
     HealthCheckTestModel,
     ImageAttachment,
     Note,
+    SavedView,
+    UserSavedViewAssociation,
     Webhook,
 )
 from .relationships import Relationship, RelationshipAssociation, RelationshipModel
+from .roles import Role, RoleField
 from .secrets import Secret, SecretsGroup, SecretsGroupAssociation
+from .statuses import Status, StatusField, StatusModel
 from .tags import Tag, TaggedItem
 
-
 __all__ = (
     "ChangeLoggedModel",
     "ComputedField",
     "ConfigContext",
     "ConfigContextModel",
     "ConfigContextSchema",
+    "Contact",
+    "ContactAssociation",
+    "ContactMixin",
     "CustomField",
     "CustomFieldChoice",
     "CustomFieldModel",
     "CustomLink",
     "DynamicGroup",
     "DynamicGroupMembership",
+    "DynamicGroupMixin",  # deprecated
+    "DynamicGroupsModelMixin",
     "ExportTemplate",
+    "ExternalIntegration",
     "FileAttachment",
     "FileProxy",
     "GitRepository",
@@ -55,23 +68,33 @@ __all__ = (
     "JobButton",
     "JobHook",
     "JobLogEntry",
+    "JobQueue",
+    "JobQueueAssignment",
     "JobResult",
+    "MetadataChoice",
+    "MetadataType",
     "Note",
     "ObjectChange",
+    "ObjectMetadata",
     "Relationship",
-    "RelationshipModel",
     "RelationshipAssociation",
+    "RelationshipModel",
     "Role",
     "RoleField",
+    "SavedView",
+    "SavedViewMixin",
     "ScheduledJob",
     "ScheduledJobs",
     "Secret",
     "SecretsGroup",
     "SecretsGroupAssociation",
+    "StaticGroupAssociation",
     "Status",
     "StatusField",
     "StatusModel",
     "Tag",
     "TaggedItem",
+    "Team",
+    "UserSavedViewAssociation",
     "Webhook",
 )
