@@ -1,13 +1,16 @@
-import json
 from collections import OrderedDict
+import json
 
 from db_file_storage.model_utils import delete_file, delete_file_if_needed
 from db_file_storage.storage import DatabaseFileStorage
 from django.conf import settings
+from django.contrib.auth import get_user_model
 from django.contrib.contenttypes.fields import GenericForeignKey
 from django.contrib.contenttypes.models import ContentType
-from django.core.serializers.json import DjangoJSONEncoder
 from django.core.exceptions import ValidationError
+from django.core.files.storage import get_storage_class
+from django.core.serializers.json import DjangoJSONEncoder
+from django.core.validators import MinValueValidator
 from django.db import models
 from django.http import HttpResponse
 from graphene_django.settings import graphene_settings
@@ -18,9 +21,10 @@ from jsonschema.exceptions import SchemaError, ValidationError as JSONSchemaVali
 from jsonschema.validators import Draft7Validator
 from rest_framework.utils.encoders import JSONEncoder
 
+from nautobot.core.constants import CHARFIELD_MAX_LENGTH
 from nautobot.core.models import BaseManager, BaseModel
-from nautobot.core.models.fields import ForeignKeyWithAutoRelatedName
-from nautobot.core.models.generics import OrganizationalModel
+from nautobot.core.models.fields import ForeignKeyWithAutoRelatedName, LaxURLField
+from nautobot.core.models.generics import OrganizationalModel, PrimaryModel
 from nautobot.core.utils.data import deepmerge, render_jinja2
 from nautobot.extras.choices import (
     ButtonClassChoices,
@@ -28,14 +32,15 @@ from nautobot.extras.choices import (
 )
 from nautobot.extras.constants import HTTP_CONTENT_TYPE_JSON
 from nautobot.extras.models import ChangeLoggedModel
-from nautobot.extras.models.mixins import NotesMixin
+from nautobot.extras.models.mixins import ContactMixin, DynamicGroupsModelMixin, NotesMixin, SavedViewMixin
 from nautobot.extras.models.relationships import RelationshipModel
 from nautobot.extras.querysets import ConfigContextQuerySet, NotesQuerySet
 from nautobot.extras.utils import extras_features, FeatureQuery, image_upload
 
 # Avoid breaking backward compatibility on anything that might expect these to still be defined here:
-from .jobs import JOB_LOGS, Job, JobLogEntry, JobResult, ScheduledJob, ScheduledJobs  # noqa: F401
+from .jobs import Job, JOB_LOGS, JobLogEntry, JobResult, ScheduledJob, ScheduledJobs  # noqa: F401  # unused-import
 
+User = get_user_model()
 #
 # Config contexts
 #
@@ -65,14 +70,22 @@ def limit_dynamic_group_choices():
 
 
 @extras_features("graphql")
-class ConfigContext(BaseModel, ChangeLoggedModel, ConfigContextSchemaValidationMixin, NotesMixin):
+class ConfigContext(
+    ChangeLoggedModel,
+    ConfigContextSchemaValidationMixin,
+    ContactMixin,
+    # DynamicGroupsModelMixin,  # TODO: conflicts with "dynamic_groups" M2M field on this model
+    NotesMixin,
+    SavedViewMixin,
+    BaseModel,
+):
     """
     A ConfigContext represents a set of arbitrary data available to any Device or VirtualMachine matching its assigned
     qualifiers (location, tenant, etc.). For example, the data stored in a ConfigContext assigned to location A and tenant B
     will be available to a Device in location A assigned to tenant B. Data is stored in JSON format.
     """
 
-    name = models.CharField(max_length=100, unique=True)
+    name = models.CharField(max_length=CHARFIELD_MAX_LENGTH, unique=True)
 
     # A ConfigContext *may* be owned by another model, such as a GitRepository, or it may be un-owned
     owner_content_type = models.ForeignKey(
@@ -91,7 +104,7 @@ class ConfigContext(BaseModel, ChangeLoggedModel, ConfigContextSchemaValidationM
     )
 
     weight = models.PositiveSmallIntegerField(default=1000)
-    description = models.CharField(max_length=200, blank=True)
+    description = models.CharField(max_length=CHARFIELD_MAX_LENGTH, blank=True)
     is_active = models.BooleanField(
         default=True,
     )
@@ -190,35 +203,14 @@ class ConfigContextModel(models.Model, ConfigContextSchemaValidationMixin):
         """
         Return the rendered configuration context for a device or VM.
         """
-
         if not hasattr(self, "config_context_data"):
             # Annotation not available, so fall back to manually querying for the config context
             config_context_data = ConfigContext.objects.get_for_object(self).values_list("data", flat=True)
         else:
-            config_context_data = self.config_context_data or []
-            # Device and VirtualMachine's Location has its own ConfigContext and its parent Locations' ConfigContext, if any, should
-            # also be applied. However, since moving from mptt to django-tree-queries https://github.com/nautobot/nautobot/issues/510,
-            # we lost the ability to query the ancestors for a particular tree node for subquery https://github.com/matthiask/django-tree-queries/issues/54.
-            # So instead of constructing the location related query in ConfigContextModelQueryset._get_config_context_filters(), which is complicated across databases
-            # We append the missing parent location query here as a patch.
-            location_config_context_queryset = ConfigContext.objects.none()
-            if self._meta.model_name == "device":
-                location_config_context_queryset = ConfigContext.objects.filter(
-                    locations__in=self.location.ancestors(include_self=True)
-                ).distinct()
-            else:
-                if self.cluster and self.cluster.location:
-                    location_config_context_queryset = ConfigContext.objects.filter(
-                        locations__in=self.cluster.location.ancestors(include_self=True)
-                    ).distinct()
-
-            # Annotation has keys "weight" and "name" (used for ordering) and "data" (the actual config context data)
-            for cc in location_config_context_queryset:
-                config_context_data.append({"data": cc.data, "name": cc.name, "weight": cc.weight})
+            config_context_data = self.config_context_data or []  # pylint: disable=no-member
             config_context_data = [
                 c["data"] for c in sorted(config_context_data, key=lambda k: (k["weight"], k["name"]))
             ]
-
         # Compile all config data, overwriting lower-weight values with higher-weight values where a collision occurs
         data = OrderedDict()
         for context in config_context_data:
@@ -257,8 +249,8 @@ class ConfigContextSchema(OrganizationalModel):
     This model stores jsonschema documents where are used to optionally validate config context data payloads.
     """
 
-    name = models.CharField(max_length=200, unique=True)
-    description = models.CharField(max_length=200, blank=True)
+    name = models.CharField(max_length=CHARFIELD_MAX_LENGTH, unique=True)
+    description = models.CharField(max_length=CHARFIELD_MAX_LENGTH, blank=True)
     data_schema = models.JSONField(
         help_text="A JSON Schema document which is used to validate a config context object."
     )
@@ -315,10 +307,18 @@ class ConfigContextSchema(OrganizationalModel):
 
 
 @extras_features("graphql")
-class CustomLink(BaseModel, ChangeLoggedModel, NotesMixin):
+class CustomLink(
+    ChangeLoggedModel,
+    ContactMixin,
+    DynamicGroupsModelMixin,
+    NotesMixin,
+    SavedViewMixin,
+    BaseModel,
+):
     """
-    A custom link to an external representation of a Nautobot object. The link text and URL fields accept Jinja2 template
-    code to be rendered with an object as context.
+    A custom link to an external representation of a Nautobot object.
+
+    The link text and URL fields accept Jinja2 template code to be rendered with an object as context.
     """
 
     content_type = models.ForeignKey(
@@ -327,7 +327,7 @@ class CustomLink(BaseModel, ChangeLoggedModel, NotesMixin):
         limit_choices_to=FeatureQuery("custom_links"),
         related_name="custom_links",
     )
-    name = models.CharField(max_length=100, unique=True)
+    name = models.CharField(max_length=CHARFIELD_MAX_LENGTH, unique=True)
     text = models.CharField(
         max_length=500,
         help_text="Jinja2 template code for link text. "
@@ -342,7 +342,7 @@ class CustomLink(BaseModel, ChangeLoggedModel, NotesMixin):
     )
     weight = models.PositiveSmallIntegerField(default=100)
     group_name = models.CharField(
-        max_length=50,
+        max_length=CHARFIELD_MAX_LENGTH,
         blank=True,
         help_text="Links with the same group will appear as a dropdown menu",
     )
@@ -369,7 +369,15 @@ class CustomLink(BaseModel, ChangeLoggedModel, NotesMixin):
 @extras_features(
     "graphql",
 )
-class ExportTemplate(BaseModel, ChangeLoggedModel, RelationshipModel, NotesMixin):
+class ExportTemplate(
+    ChangeLoggedModel,
+    ContactMixin,
+    DynamicGroupsModelMixin,
+    RelationshipModel,
+    NotesMixin,
+    SavedViewMixin,
+    BaseModel,
+):
     # An ExportTemplate *may* be owned by another model, such as a GitRepository, or it may be un-owned
     owner_content_type = models.ForeignKey(
         to=ContentType,
@@ -391,19 +399,19 @@ class ExportTemplate(BaseModel, ChangeLoggedModel, RelationshipModel, NotesMixin
         limit_choices_to=FeatureQuery("export_templates"),
         related_name="export_templates",
     )
-    name = models.CharField(max_length=100)
-    description = models.CharField(max_length=200, blank=True)
+    name = models.CharField(max_length=CHARFIELD_MAX_LENGTH)
+    description = models.CharField(max_length=CHARFIELD_MAX_LENGTH, blank=True)
     template_code = models.TextField(
         help_text="The list of objects being exported is passed as a context variable named <code>queryset</code>."
     )
     mime_type = models.CharField(
-        max_length=50,
+        max_length=CHARFIELD_MAX_LENGTH,
         blank=True,
         verbose_name="MIME type",
         help_text="Defaults to <code>text/plain</code>",
     )
     file_extension = models.CharField(
-        max_length=15,
+        max_length=CHARFIELD_MAX_LENGTH,
         blank=True,
         help_text="Extension to append to the rendered filename",
     )
@@ -449,6 +457,109 @@ class ExportTemplate(BaseModel, ChangeLoggedModel, RelationshipModel, NotesMixin
         if self.file_extension.startswith("."):
             self.file_extension = self.file_extension[1:]
 
+    clean.alters_data = True
+
+
+#
+# External integrations
+#
+
+
+class ExternalIntegration(PrimaryModel):
+    """Model for tracking integrations with external applications."""
+
+    name = models.CharField(max_length=CHARFIELD_MAX_LENGTH, unique=True)
+    remote_url = LaxURLField(
+        max_length=500,
+        verbose_name="Remote URL",
+    )
+    secrets_group = models.ForeignKey(
+        null=True,
+        blank=True,
+        to="extras.SecretsGroup",
+        on_delete=models.PROTECT,
+        help_text="Credentials used for authenticating with the remote system",
+    )
+    verify_ssl = models.BooleanField(
+        default=True,
+        verbose_name="Verify SSL",
+        help_text="Verify SSL certificates when connecting to the remote system",
+    )
+    timeout = models.IntegerField(
+        default=30,
+        validators=[MinValueValidator(0)],
+        help_text="Number of seconds to wait for a response",
+    )
+    extra_config = models.JSONField(
+        blank=True,
+        null=True,
+        help_text="Optional user-defined JSON data for this integration",
+    )
+    http_method = models.CharField(
+        max_length=10,
+        choices=WebhookHttpMethodChoices,
+        verbose_name="HTTP method",
+        blank=True,
+    )
+    headers = models.JSONField(
+        encoder=DjangoJSONEncoder,
+        blank=True,
+        null=True,
+        help_text="Headers for the HTTP request",
+    )
+    ca_file_path = models.CharField(
+        max_length=CHARFIELD_MAX_LENGTH,
+        blank=True,
+        verbose_name="CA file path",
+    )
+
+    def __str__(self):
+        return f"{self.name} ({self.remote_url})"
+
+    class Meta:
+        ordering = ["name"]
+
+    def render_headers(self, context):
+        """
+        Render headers and return a dict of Header: Value pairs.
+
+        Raises:
+            TemplateAssertionError: Raised when an invalid template helper function exists in headers.
+            TemplateSyntaxError: Raised when an invalid template variable exists in headers.
+            json.decoder.JSONDecodeError: Raised when invalid JSON data exists in context.
+        """
+        if not self.headers:
+            return {}
+
+        data = json.loads(render_jinja2(json.dumps(self.headers, ensure_ascii=False), context))
+        return data
+
+    def render_extra_config(self, context):
+        """
+        Render extra_config and return a dict of Key: Value pairs.
+
+        Raises:
+            TemplateAssertionError: Raised when an invalid template helper function exists in extra_config.
+            TemplateSyntaxError: Raised when an invalid template variable exists in extra_config.
+            json.decoder.JSONDecodeError: Raised when invalid JSON data exists in context.
+        """
+        if not self.extra_config:
+            return {}
+
+        data = json.loads(render_jinja2(json.dumps(self.extra_config, ensure_ascii=False), context))
+        return data
+
+    def render_remote_url(self, context):
+        """
+        Render remote_url in jinja2 templates.
+
+        Raises:
+            TemplateAssertionError: Raised when an invalid template helper function exists in remote_url.
+            TemplateSyntaxError: Raised when an invalid template variable exists in remote_url.
+        """
+        data = render_jinja2(self.remote_url, context)
+        return data
+
 
 #
 # File attachments
@@ -463,8 +574,10 @@ class FileAttachment(BaseModel):
     """
 
     bytes = models.BinaryField()
-    filename = models.CharField(max_length=255)
-    mimetype = models.CharField(max_length=255)
+    filename = models.CharField(max_length=CHARFIELD_MAX_LENGTH)
+    mimetype = models.CharField(max_length=CHARFIELD_MAX_LENGTH)
+
+    is_metadata_associable_model = False
 
     natural_key_field_names = ["pk"]
 
@@ -476,29 +589,55 @@ class FileAttachment(BaseModel):
 
 
 def database_storage():
-    """Returns storage backend used by `FileProxy.file` to store files in the database."""
+    """
+    Returns an instance of DatabaseFileStorage() unconditionally.
+
+    This is kept around to support legacy migrations; it shouldn't generally be used outside that.
+    Use `_job_storage()` instead.
+    """
     return DatabaseFileStorage()
 
 
+def _job_storage():
+    return get_storage_class(settings.JOB_FILE_IO_STORAGE)()
+
+
+def _upload_to(instance, filename):
+    """
+    Returns the upload path for attaching the given filename to the given FileProxy instance.
+
+    Because django-db-file-storage has specific requirements for this path to configure the FileAttachment model,
+    this needs to inspect which storage backend is in use in order to make the right determination.
+    """
+    if get_storage_class(settings.JOB_FILE_IO_STORAGE) == DatabaseFileStorage:
+        # must be a string of the form
+        # "<app_label>.<ModelName>/<data field name>/<filename field name>/<mimetype field name>/filename"
+        return f"extras.FileAttachment/bytes/filename/mimetype/{filename}"
+    else:
+        return f"files/{instance.pk}-{filename}"
+
+
 class FileProxy(BaseModel):
-    """An object to store a file in the database.
+    """A database object to reference and index a file, such as a Job input file or a Job output file.
 
-    The `file` field can be used like a file handle. The file contents are stored and retrieved from
-    `FileAttachment` objects.
+    The `file` field can be used like a file handle.
 
-    The associated `FileAttachment` is removed when `delete()` is called. For this reason, one
-    should never use bulk delete operations on `FileProxy` objects, unless `FileAttachment` objects
-    are also bulk-deleted, because a model's `delete()` method is not called during bulk operations.
+    The file contents are stored and retrieved from `FileAttachment` objects,
+    if `settings.JOB_FILE_IO_STORAGE` is set to use DatabaseFileStorage,
+    otherwise they're written to whatever other file storage backend is in use.
+
+    When using DatabaseFileStorage, the associated `FileAttachment` is removed when `delete()` is called.
+    For this reason, one should never use bulk delete operations on `FileProxy` objects,
+    unless `FileAttachment` objects are also bulk-deleted,
+    because a model's `delete()` method is not called during bulk operations.
     In most cases, it is better to iterate over a queryset of `FileProxy` objects and call
     `delete()` on each one individually.
     """
 
-    name = models.CharField(max_length=255)
-    file = models.FileField(
-        upload_to="extras.FileAttachment/bytes/filename/mimetype",
-        storage=database_storage,  # Use only this backend
-    )
+    name = models.CharField(max_length=CHARFIELD_MAX_LENGTH)
+    file = models.FileField(upload_to=_upload_to, storage=_job_storage)
     uploaded_at = models.DateTimeField(auto_now_add=True)
+    job_result = models.ForeignKey(to=JobResult, null=True, blank=True, on_delete=models.CASCADE, related_name="files")
 
     def __str__(self):
         return self.name
@@ -514,12 +653,19 @@ class FileProxy(BaseModel):
     natural_key_field_names = ["name", "uploaded_at"]
 
     def save(self, *args, **kwargs):
-        delete_file_if_needed(self, "file")
+        if get_storage_class(settings.JOB_FILE_IO_STORAGE) == DatabaseFileStorage:
+            delete_file_if_needed(self, "file")
+        else:
+            # TODO check whether there's an existing file with a different filename and delete it if so?
+            pass
         super().save(*args, **kwargs)
 
     def delete(self, *args, **kwargs):
+        if get_storage_class(settings.JOB_FILE_IO_STORAGE) != DatabaseFileStorage:
+            self.file.delete()
         super().delete(*args, **kwargs)
-        delete_file(self, "file")
+        if get_storage_class(settings.JOB_FILE_IO_STORAGE) == DatabaseFileStorage:
+            delete_file(self, "file")
 
 
 #
@@ -528,8 +674,30 @@ class FileProxy(BaseModel):
 
 
 @extras_features("graphql")
-class GraphQLQuery(BaseModel, ChangeLoggedModel, NotesMixin):
-    name = models.CharField(max_length=100, unique=True)
+class GraphQLQuery(
+    ChangeLoggedModel,
+    ContactMixin,
+    DynamicGroupsModelMixin,
+    NotesMixin,
+    SavedViewMixin,
+    BaseModel,
+):
+    # A Graphql Query *may* be owned by another model, such as a GitRepository, or it may be un-owned
+    owner_content_type = models.ForeignKey(
+        to=ContentType,
+        on_delete=models.CASCADE,
+        limit_choices_to=FeatureQuery("graphql_query_owners"),
+        default=None,
+        null=True,
+        blank=True,
+        related_name="graphql_queries",
+    )
+    owner_object_id = models.UUIDField(default=None, null=True, blank=True)
+    owner = GenericForeignKey(
+        ct_field="owner_content_type",
+        fk_field="owner_object_id",
+    )
+    name = models.CharField(max_length=CHARFIELD_MAX_LENGTH, unique=True)
     query = models.TextField()
     variables = models.JSONField(encoder=DjangoJSONEncoder, default=dict, blank=True)
 
@@ -580,7 +748,8 @@ class GraphQLQuery(BaseModel, ChangeLoggedModel, NotesMixin):
 
 
 class HealthCheckTestModel(BaseModel):
-    title = models.CharField(max_length=128)
+    title = models.CharField(max_length=CHARFIELD_MAX_LENGTH)
+    is_metadata_associable_model = False
 
 
 #
@@ -599,7 +768,7 @@ class ImageAttachment(BaseModel):
     image = models.ImageField(upload_to=image_upload, height_field="image_height", width_field="image_width")
     image_height = models.PositiveSmallIntegerField()
     image_width = models.PositiveSmallIntegerField()
-    name = models.CharField(max_length=50, blank=True, db_index=True)
+    name = models.CharField(max_length=CHARFIELD_MAX_LENGTH, blank=True, db_index=True)
     created = models.DateTimeField(auto_now_add=True)
 
     natural_key_field_names = ["pk"]
@@ -652,7 +821,7 @@ class ImageAttachment(BaseModel):
 
 
 @extras_features("graphql", "webhooks")
-class Note(BaseModel, ChangeLoggedModel):
+class Note(ChangeLoggedModel, BaseModel):
     """
     Notes allow anyone with proper permissions to add a note to an object.
     """
@@ -672,12 +841,14 @@ class Note(BaseModel, ChangeLoggedModel):
     note = models.TextField()
     objects = BaseManager.from_queryset(NotesQuerySet)()
 
+    is_metadata_associable_model = False
+
     class Meta:
         ordering = ["created"]
         unique_together = [["assigned_object_type", "assigned_object_id", "user_name", "created"]]
 
     def __str__(self):
-        return f"{self.assigned_object} - {self.created.isoformat()}"
+        return f"{self.assigned_object} - {self.created.isoformat() if self.created else None}"  # pylint: disable=no-member
 
     def save(self, *args, **kwargs):
         # Record the user's name as static strings
@@ -686,12 +857,85 @@ class Note(BaseModel, ChangeLoggedModel):
 
 
 #
-# Webhooks
+# SavedView
 #
 
 
+@extras_features(
+    "custom_validators",
+)
+class SavedView(BaseModel, ChangeLoggedModel):
+    owner = models.ForeignKey(
+        to=User, blank=False, null=False, on_delete=models.CASCADE, help_text="The user that created this view"
+    )
+    name = models.CharField(max_length=CHARFIELD_MAX_LENGTH, blank=False, null=False, help_text="The name of this view")
+    view = models.CharField(
+        max_length=CHARFIELD_MAX_LENGTH,
+        blank=False,
+        null=False,
+        help_text="The name of the list view that the saved view is derived from, e.g. dcim:device_list",
+    )
+    config = models.JSONField(
+        encoder=DjangoJSONEncoder, blank=True, default=dict, help_text="Saved Configuration on this view"
+    )
+    is_global_default = models.BooleanField(default=False)
+    is_shared = models.BooleanField(default=True)
+
+    documentation_static_path = "docs/user-guide/platform-functionality/savedview.html"
+
+    class Meta:
+        ordering = ["owner", "view", "name"]
+        unique_together = [["owner", "name", "view"]]
+        verbose_name = "saved view"
+        verbose_name_plural = "saved views"
+
+    def __str__(self):
+        return f"{self.owner.username} - {self.view} - {self.name}"
+
+    def save(self, *args, **kwargs):
+        # If this SavedView is set to a global default, all other saved views related to this view name should not be the global default.
+        if self.is_global_default:
+            SavedView.objects.filter(view=self.view, is_global_default=True).exclude(pk=self.pk).update(
+                is_global_default=False
+            )
+            # If the view is set to Global default, is_shared should be true regardless
+            self.is_shared = True
+
+        super().save(*args, **kwargs)
+
+
 @extras_features("graphql")
-class Webhook(BaseModel, ChangeLoggedModel, NotesMixin):
+class UserSavedViewAssociation(BaseModel):
+    saved_view = models.ForeignKey("extras.SavedView", on_delete=models.CASCADE, related_name="user_assignments")
+    user = models.ForeignKey("users.User", on_delete=models.CASCADE, related_name="saved_view_assignments")
+    view_name = models.CharField(max_length=CHARFIELD_MAX_LENGTH)
+    is_metadata_associable_model = False
+
+    class Meta:
+        unique_together = [["user", "view_name"]]
+
+    def __str__(self):
+        return f"{self.user}: {self.saved_view} - {self.view_name}"
+
+    def clean(self):
+        super().clean()
+        self.view_name = self.saved_view.view
+
+    clean.alters_data = True
+
+
+#
+# Webhooks
+#
+@extras_features("graphql")
+class Webhook(
+    ChangeLoggedModel,
+    ContactMixin,
+    DynamicGroupsModelMixin,
+    NotesMixin,
+    SavedViewMixin,
+    BaseModel,
+):
     """
     A Webhook defines a request that will be sent to a remote application when an object is created, updated, and/or
     delete in Nautobot. The request will contain a representation of the object, which the remote application can act on.
@@ -705,7 +949,7 @@ class Webhook(BaseModel, ChangeLoggedModel, NotesMixin):
         limit_choices_to=FeatureQuery("webhooks"),
         help_text="The object(s) to which this Webhook applies.",
     )
-    name = models.CharField(max_length=150, unique=True)
+    name = models.CharField(max_length=CHARFIELD_MAX_LENGTH, unique=True)
     type_create = models.BooleanField(default=False, help_text="Call this webhook when a matching object is created.")
     type_update = models.BooleanField(default=False, help_text="Call this webhook when a matching object is updated.")
     type_delete = models.BooleanField(default=False, help_text="Call this webhook when a matching object is deleted.")
@@ -722,7 +966,7 @@ class Webhook(BaseModel, ChangeLoggedModel, NotesMixin):
         verbose_name="HTTP method",
     )
     http_content_type = models.CharField(
-        max_length=100,
+        max_length=CHARFIELD_MAX_LENGTH,
         default=HTTP_CONTENT_TYPE_JSON,
         verbose_name="HTTP content type",
         help_text="The complete list of official content types is available "
@@ -732,7 +976,7 @@ class Webhook(BaseModel, ChangeLoggedModel, NotesMixin):
         blank=True,
         help_text="User-supplied HTTP headers to be sent with the request in addition to the HTTP content type. "
         "Headers should be defined in the format <code>Name: Value</code>. Jinja2 template processing is "
-        "support with the same context as the request body (below).",
+        "supported with the same context as the request body (below).",
     )
     body_template = models.TextField(
         blank=True,
@@ -741,7 +985,7 @@ class Webhook(BaseModel, ChangeLoggedModel, NotesMixin):
         "<code>timestamp</code>, <code>username</code>, <code>request_id</code>, and <code>data</code>.",
     )
     secret = models.CharField(
-        max_length=255,
+        max_length=CHARFIELD_MAX_LENGTH,
         blank=True,
         help_text="When provided, the request will include a 'X-Hook-Signature' "
         "header containing a HMAC hex digest of the payload body using "
