@@ -3,7 +3,7 @@ from django.db import models
 
 from nautobot.core.models.fields import ForeignKeyLimitedByContentTypes
 from nautobot.core.models.name_color_content_types import NameColorContentTypesModel
-from nautobot.extras.utils import RoleModelsQuery, extras_features
+from nautobot.extras.utils import extras_features, RoleModelsQuery
 
 
 @extras_features(
