@@ -1,5 +1,7 @@
 from abc import ABC, abstractmethod
 
+from jinja2.sandbox import unsafe
+
 from nautobot.extras.registry import registry
 
 from .exceptions import SecretError, SecretParametersError, SecretProviderError, SecretValueNotFoundError
@@ -31,6 +33,7 @@ class SecretsProvider(ABC):
 
     @classmethod
     @abstractmethod
+    @unsafe
     def get_value_for_secret(cls, secret, obj=None, **kwargs):
         """Retrieve the stored value described by the given Secret record.
 
@@ -41,6 +44,17 @@ class SecretsProvider(ABC):
             obj (object): Django model instance or similar providing additional context for retrieving the secret.
         """
 
+    get_value_for_secret.__func__.do_not_call_in_templates = True
+
+    def __init_subclass__(cls, **kwargs):
+        # Automatically apply protection against Django and Jinja2 template execution to child classes.
+        if not getattr(cls.get_value_for_secret, "do_not_call_in_templates", False):  # Django
+            cls.get_value_for_secret.__func__.do_not_call_in_templates = True
+        if not getattr(cls.get_value_for_secret, "unsafe_callable", False):  # Jinja @unsafe decorator
+            cls.get_value_for_secret.__func__.unsafe_callable = True
+
+        super().__init_subclass__(**kwargs)
+
 
 def register_secrets_provider(provider):
     """
@@ -51,16 +65,16 @@ def register_secrets_provider(provider):
     if provider.slug in registry["secrets_providers"]:
         raise KeyError(
             f'Cannot register {provider} as slug "{provider.slug}" is already registered '
-            f'by {registry["secrets_providers"][provider.slug]}'
+            f"by {registry['secrets_providers'][provider.slug]}"
         )
     registry["secrets_providers"][provider.slug] = provider
 
 
 __all__ = (
-    "register_secrets_provider",
     "SecretError",
     "SecretParametersError",
     "SecretProviderError",
-    "SecretsProvider",
     "SecretValueNotFoundError",
+    "SecretsProvider",
+    "register_secrets_provider",
 )
