@@ -1,6 +1,5 @@
 from nautobot.core.celery import register_jobs
-from nautobot.extras.jobs import Job, FileVar, get_task_logger
-
+from nautobot.extras.jobs import FileVar, get_task_logger, Job
 
 logger = get_task_logger(__name__)
 
@@ -14,7 +13,7 @@ class TestFileUploadPass(Job):
         description="File to upload",
     )
 
-    def run(self, file):
+    def run(self, file):  # pylint:disable=arguments-differ
         contents = str(file.read())
         logger.warning("File contents: %s", contents)
         logger.info("Job didn't crash!")
