@@ -1,8 +1,7 @@
 from nautobot.core.celery import register_jobs
-from nautobot.extras.jobs import Job, ObjectVar, MultiObjectVar, get_task_logger
+from nautobot.extras.jobs import get_task_logger, Job, MultiObjectVar, ObjectVar
 from nautobot.extras.models import Role
 
-
 logger = get_task_logger(__name__)
 name = "Object Vars"
 
@@ -14,7 +13,7 @@ class TestObjectVars(Job):
     role = ObjectVar(model=Role)
     roles = MultiObjectVar(model=Role)
 
-    def run(self, role, roles):
+    def run(self, role, roles):  # pylint: disable=arguments-differ
         logger.info("Role: %s", role)
         logger.warning("Roles: %s", roles)
 
