@@ -1,45 +1,45 @@
-from unittest import mock, skipIf, skip
+from unittest import mock, skip
 
+from django.apps import apps
 from django.conf import settings
 from django.contrib.contenttypes.models import ContentType
 from django.core.exceptions import ValidationError
 from django.template import engines
 from django.test import override_settings
 from django.urls import NoReverseMatch, reverse
-
+import django_tables2 as tables
 import netaddr
 
 from nautobot.circuits.models import Circuit, CircuitType, Provider
-from nautobot.core.celery import app
-from nautobot.core.testing import APIViewTestCases, TestCase, ViewTestCases, extract_page_body
-from nautobot.dcim.models import Device, DeviceType, Manufacturer, Location, LocationType
+from nautobot.core.testing import APIViewTestCases, disable_warnings, extract_page_body, TestCase, ViewTestCases
+from nautobot.core.utils.lookup import get_table_for_model
+from nautobot.dcim.models import Device, DeviceType, Location, LocationType, Manufacturer
 from nautobot.dcim.tests.test_views import create_test_device
-from nautobot.tenancy.models import Tenant, TenantGroup
-from nautobot.tenancy.filters import TenantFilterSet
-from nautobot.tenancy.forms import TenantFilterForm
+from nautobot.extras import plugins
 from nautobot.extras.choices import CustomFieldTypeChoices, RelationshipTypeChoices
+from nautobot.extras.context_managers import web_request_context
 from nautobot.extras.jobs import get_job
 from nautobot.extras.models import CustomField, Relationship, RelationshipAssociation, Role, Secret, Status
 from nautobot.extras.plugins.exceptions import PluginImproperlyConfigured
 from nautobot.extras.plugins.utils import load_plugin
-from nautobot.extras.plugins.validators import wrap_model_clean_methods
-from nautobot.extras.registry import registry, DatasourceContent
-from nautobot.ipam.models import Prefix, IPAddress, Namespace
+from nautobot.extras.plugins.validators import CustomValidator, wrap_model_clean_methods
+from nautobot.extras.plugins.views import extract_app_data
+from nautobot.extras.registry import DatasourceContent, registry
+from nautobot.ipam.models import IPAddress, Namespace, Prefix
+from nautobot.tenancy.filters import TenantFilterSet
+from nautobot.tenancy.forms import TenantFilterForm
+from nautobot.tenancy.models import Tenant, TenantGroup
 from nautobot.users.models import ObjectPermission
 
-from example_plugin import config as example_config
-from example_plugin.datasources import refresh_git_text_files
-from example_plugin.models import ExampleModel
+from example_app import config as example_config
+from example_app.datasources import refresh_git_text_files
+from example_app.models import ExampleModel
 
 
-@skipIf(
-    "example_plugin" not in settings.PLUGINS,
-    "example_plugin not in settings.PLUGINS",
-)
-class PluginTest(TestCase):
+class AppTest(TestCase):
     def test_config(self):
         self.assertIn(
-            "example_plugin.ExamplePluginConfig",
+            "example_app.ExampleAppConfig",
             settings.INSTALLED_APPS,
         )
 
@@ -55,30 +55,30 @@ class PluginTest(TestCase):
 
     def test_admin(self):
         # Test admin view URL resolution
-        url = reverse("admin:example_plugin_examplemodel_add")
-        self.assertEqual(url, "/admin/example_plugin/examplemodel/add/")
+        url = reverse("admin:example_app_examplemodel_add")
+        self.assertEqual(url, "/admin/example_app/examplemodel/add/")
 
     def test_banner_registration(self):
         """
-        Check that plugin Banner is registered.
+        Check that App Banner is registered.
         """
-        from example_plugin.banner import banner
+        from example_app.banner import banner
 
         self.assertIn(banner, registry["plugin_banners"])
 
     def test_template_extensions_registration(self):
         """
-        Check that plugin TemplateExtensions are registered.
+        Check that App TemplateExtensions are registered.
         """
-        from example_plugin.template_content import LocationContent
+        from example_app.template_content import LocationContent
 
         self.assertIn(LocationContent, registry["plugin_template_extensions"]["dcim.location"])
 
     def test_custom_validators_registration(self):
         """
-        Check that plugin custom validators are registered correctly.
+        Check that App CustomValidators are registered correctly.
         """
-        from example_plugin.custom_validators import LocationCustomValidator, RelationshipAssociationCustomValidator
+        from example_app.custom_validators import LocationCustomValidator, RelationshipAssociationCustomValidator
 
         self.assertIn(LocationCustomValidator, registry["plugin_custom_validators"]["dcim.location"])
         self.assertIn(
@@ -88,9 +88,9 @@ class PluginTest(TestCase):
 
     def test_jinja_filter_registration(self):
         """
-        Check that plugin custom jinja filters are registered correctly.
+        Check that App custom Jinja filters are registered correctly.
         """
-        from example_plugin.jinja_filters import leet_speak
+        from example_app.jinja_filters import leet_speak
 
         rendering_engine = engines["jinja"]
 
@@ -98,61 +98,60 @@ class PluginTest(TestCase):
 
     def test_graphql_types_registration(self):
         """
-        Check that plugin GraphQL Types are registered.
+        Check that App GraphQL Types are registered.
         """
-        from example_plugin.graphql.types import AnotherExampleModelType
+        from example_app.graphql.types import AnotherExampleModelType
 
         self.assertIn(AnotherExampleModelType, registry["plugin_graphql_types"])
 
     def test_extras_features_graphql(self):
         """
-        Check that plugin GraphQL Types are registered.
+        Check that App GraphQL Types are registered.
         """
         registered_models = registry.get("model_features", {}).get("graphql", {})
 
-        self.assertIn("example_plugin", registered_models.keys())
-        self.assertIn("examplemodel", registered_models["example_plugin"])
+        self.assertIn("example_app", registered_models.keys())
+        self.assertIn("examplemodel", registered_models["example_app"])
 
     def test_jobs_registration(self):
         """
-        Check that plugin jobs are registered correctly and discoverable.
+        Check that App Jobs are registered correctly and discoverable.
         """
-        from example_plugin.jobs import ExampleJob
+        from example_app.jobs import ExampleJob
 
-        self.assertIn(ExampleJob, registry.get("plugin_jobs", []))
-        self.assertEqual(ExampleJob, get_job("example_plugin.jobs.ExampleJob"))
-        self.assertIn("example_plugin.jobs.ExampleJob", app.tasks)
+        self.assertIn(ExampleJob.class_path, registry.get("jobs", {}))
+        self.assertEqual(ExampleJob, get_job("example_app.jobs.ExampleJob"))
 
     def test_git_datasource_contents_registration(self):
         """
-        Check that plugin DatasourceContents are registered.
+        Check that App DatasourceContents are registered.
         """
         registered_datasources = registry.get("datasource_contents", {}).get("extras.gitrepository", [])
 
-        plugin_datasource = DatasourceContent(
+        app_datasource = DatasourceContent(
             name="text files",
-            content_identifier="example_plugin.textfile",
+            content_identifier="example_app.textfile",
             icon="mdi-note-text",
             weight=1000,
             callback=refresh_git_text_files,
         )
 
         for datasource in registered_datasources:
-            if datasource.name == plugin_datasource.name:
-                self.assertEqual(datasource.content_identifier, plugin_datasource.content_identifier)
-                self.assertEqual(datasource.icon, plugin_datasource.icon)
-                self.assertEqual(datasource.weight, plugin_datasource.weight)
-                self.assertEqual(datasource.callback, plugin_datasource.callback)
+            if datasource.name == app_datasource.name:
+                self.assertEqual(datasource.content_identifier, app_datasource.content_identifier)
+                self.assertEqual(datasource.icon, app_datasource.icon)
+                self.assertEqual(datasource.weight, app_datasource.weight)
+                self.assertEqual(datasource.callback, app_datasource.callback)
                 break
         else:
-            self.fail(f"Datasource {plugin_datasource.name} not found in registered_datasources!")
+            self.fail(f"Datasource {app_datasource.name} not found in registered_datasources!")
 
     def test_middleware(self):
         """
-        Check that plugin middleware is registered.
+        Check that App middleware is registered.
         """
         self.assertIn(
-            "example_plugin.middleware.ExampleMiddleware",
+            "example_app.middleware.ExampleMiddleware",
             settings.MIDDLEWARE,
         )
 
@@ -230,13 +229,13 @@ class PluginTest(TestCase):
 
     def test_installed_apps(self):
         """
-        Validate that plugin installed apps and dependencies are are registered.
+        Validate that App installed Django apps and dependencies are are registered.
         """
         self.assertIn(
-            "example_plugin.ExamplePluginConfig",
+            "example_app.ExampleAppConfig",
             settings.INSTALLED_APPS,
         )
-        self.assertIn("nautobot.extras.tests.example_plugin_dependency", settings.INSTALLED_APPS)
+        self.assertIn("nautobot.extras.tests.example_app_dependency", settings.INSTALLED_APPS)
 
         # Establish example config to have invalid installed_apps (tuple)
         class ExampleConfigWithInstalledApps(example_config):
@@ -248,25 +247,25 @@ class PluginTest(TestCase):
 
     def test_registry_nav_menu_dict(self):
         """
-        Validate that example plugin is adding new items to `registry["nav_menu"]`.
+        Validate that Example App is adding new items to `registry["nav_menu"]`.
         """
         self.assertIn("Example Group 1", registry["nav_menu"]["tabs"]["Example Menu"]["groups"])
 
     def test_nautobot_database_ready_signal(self):
         """
-        Validate that the plugin's registered callback for the `nautobot_database_ready` signal got called,
+        Validate that the App's registered callback for the `nautobot_database_ready` signal got called,
         creating a custom field definition in the database.
         """
-        cf = CustomField.objects.get(key="example_plugin_auto_custom_field")
+        cf = CustomField.objects.get(key="example_app_auto_custom_field")
         self.assertEqual(cf.type, CustomFieldTypeChoices.TYPE_TEXT)
-        self.assertEqual(cf.label, "Example Plugin Automatically Added Custom Field")
+        self.assertEqual(cf.label, "Example App Automatically Added Custom Field")
         self.assertEqual(list(cf.content_types.all()), [ContentType.objects.get_for_model(Location)])
 
     def test_secrets_provider(self):
         """
-        Validate that a plugin can provide a custom Secret provider and it will be used.
+        Validate that an App can provide a custom Secret provider and it will be used.
         """
-        # The "constant-value" provider is implemented by the plugin
+        # The "constant-value" provider is implemented by the Example App
         secret = Secret.objects.create(
             name="Constant Secret",
             provider="constant-value",
@@ -276,11 +275,7 @@ class PluginTest(TestCase):
         self.assertEqual(secret.get_value(obj=secret), secret.parameters["constant"])
 
 
-@skipIf(
-    "example_plugin" not in settings.PLUGINS,
-    "example_plugin not in settings.PLUGINS",
-)
-class PluginGenericViewTest(ViewTestCases.PrimaryObjectViewTestCase):
+class AppGenericViewTest(ViewTestCases.PrimaryObjectViewTestCase):
     model = ExampleModel
 
     @classmethod
@@ -308,7 +303,7 @@ class PluginGenericViewTest(ViewTestCases.PrimaryObjectViewTestCase):
         }
 
 
-class PluginListViewTest(TestCase):
+class AppListViewTest(TestCase):
     def test_list_plugins_anonymous(self):
         # Make the request as an unauthenticated user
         self.client.logout()
@@ -316,50 +311,89 @@ class PluginListViewTest(TestCase):
         # Redirects to the login page
         self.assertHttpStatus(response, 302)
 
-    def test_list_plugins_authenticated_superuser(self):
-        self.user.is_superuser = True
-        self.user.save()
-
+    def test_list_plugins_authenticated(self):
         response = self.client.get(reverse("plugins:plugins_list"))
         self.assertHttpStatus(response, 200)
 
         response_body = extract_page_body(response.content.decode(response.charset)).lower()
         self.assertIn("example app", response_body, msg=response_body)
 
-    def test_list_plugins_authenticated_not_admin(self):
-        response = self.client.get(reverse("plugins:plugins_list"))
-        # Access Denied
-        self.assertHttpStatus(response, 403)
+    def test_extract_app_data(self):
+        app_config = apps.get_app_config("example_app")
+        # Without a corresponding entry in marketplace_data
+        self.assertEqual(
+            extract_app_data(app_config, {"apps": []}),
+            {
+                "name": "Example Nautobot App",
+                "package": "example_app",
+                "app_label": "example_app",
+                "author": "Nautobot development team",
+                "author_email": "nautobot@example.com",
+                "headline": "For testing purposes only",
+                "description": "For testing purposes only",
+                "version": "1.0.0",
+                "home_url": "plugins:example_app:home",
+                "config_url": "plugins:example_app:config",
+                "docs_url": "plugins:example_app:docs",
+            },
+        )
+
+        # With a corresponding entry in marketplace_data
+        mock_marketplace_data = {
+            "apps": [
+                {
+                    "name": "Nautobot Example App",
+                    "use_cases": ["Example", "Development"],
+                    "requires": [],
+                    "docs": "https://example.com/docs/example_app/",
+                    "headline": "Demonstrate and test various parts of Nautobot App APIs and functionality.",
+                    "description": "An example of the kinds of things a Nautobot App can do, including ...",
+                    "package_name": "example_app",
+                    "author": "Network to Code (NTC)",
+                    "availability": "Open Source",
+                    "icon": "...",
+                },
+            ],
+        }
+        self.assertEqual(
+            extract_app_data(app_config, mock_marketplace_data),
+            {
+                "name": "Nautobot Example App",
+                "package": "example_app",
+                "app_label": "example_app",
+                "author": "Network to Code (NTC)",
+                "author_email": "nautobot@example.com",
+                "availability": "Open Source",
+                "headline": "Demonstrate and test various parts of Nautobot App APIs and functionality.",
+                "description": "An example of the kinds of things a Nautobot App can do, including ...",
+                "version": "1.0.0",
+                "home_url": "plugins:example_app:home",
+                "config_url": "plugins:example_app:config",
+                "docs_url": "plugins:example_app:docs",
+            },
+        )
 
 
-@skipIf(
-    "example_plugin" not in settings.PLUGINS,
-    "example_plugin not in settings.PLUGINS",
-)
 class PluginDetailViewTest(TestCase):
     def test_view_detail_anonymous(self):
         # Make the request as an unauthenticated user
         self.client.logout()
-        response = self.client.get(reverse("plugins:plugin_detail", kwargs={"plugin": "example_plugin"}))
+        response = self.client.get(reverse("plugins:plugin_detail", kwargs={"plugin": "example_app"}))
         # Redirects to the login page
         self.assertHttpStatus(response, 302)
 
     def test_view_detail_authenticated(self):
-        response = self.client.get(reverse("plugins:plugin_detail", kwargs={"plugin": "example_plugin"}))
+        response = self.client.get(reverse("plugins:plugin_detail", kwargs={"plugin": "example_app"}))
         self.assertHttpStatus(response, 200)
 
         response_body = extract_page_body(response.content.decode(response.charset))
-        # plugin verbose name
+        # App verbose name
         self.assertIn("Example Nautobot App", response_body, msg=response_body)
-        # plugin description
+        # App description
         self.assertIn("For testing purposes only", response_body, msg=response_body)
 
 
-@skipIf(
-    "example_plugin" not in settings.PLUGINS,
-    "example_plugin not in settings.PLUGINS",
-)
-class PluginAPITest(APIViewTestCases.APIViewTestCase):
+class AppAPITest(APIViewTestCases.APIViewTestCase):
     model = ExampleModel
     bulk_update_data = {
         "number": 2600,
@@ -390,12 +424,12 @@ class PluginAPITest(APIViewTestCases.APIViewTestCase):
     @override_settings(EXEMPT_VIEW_PERMISSIONS=["*"])
     def test_api_urls(self):
         # Test list URL resolution
-        list_url = reverse("plugins-api:example_plugin-api:examplemodel-list")
+        list_url = reverse("plugins-api:example_app-api:examplemodel-list")
         self.assertEqual(list_url, self._get_list_url())
 
         # Test detail URL resolution
         instance = ExampleModel.objects.first()
-        detail_url = reverse("plugins-api:example_plugin-api:examplemodel-detail", kwargs={"pk": instance.pk})
+        detail_url = reverse("plugins-api:example_app-api:examplemodel-detail", kwargs={"pk": instance.pk})
         self.assertEqual(detail_url, self._get_detail_url(instance))
 
     @override_settings(EXEMPT_VIEW_PERMISSIONS=[])
@@ -445,17 +479,24 @@ class PluginAPITest(APIViewTestCases.APIViewTestCase):
         pass
 
 
-@skipIf(
-    "example_plugin" not in settings.PLUGINS,
-    "example_plugin not in settings.PLUGINS",
-)
-class PluginCustomValidationTest(TestCase):
+class TestUserContextCustomValidator(CustomValidator):
+    model = "dcim.locationtype"
+
+    def clean(self):
+        """
+        Used to validate that the correct user context is available in the custom validator.
+        """
+        self.validation_error(f"TestUserContextCustomValidator: user is {self.context['user']}")
+
+
+class AppCustomValidationTest(TestCase):
     def setUp(self):
         # When creating a fresh test DB, wrapping model clean methods fails, which is normal.
         # This always occurs during the first run of migrations, however, During testing we
         # must manually call the method again to actually perform the action, now that the
         # ContentType table has been created.
         wrap_model_clean_methods()
+        super().setUp()
 
     def test_custom_validator_raises_exception(self):
         location_type = LocationType.objects.get(name="Campus")
@@ -484,6 +525,93 @@ class PluginCustomValidationTest(TestCase):
         with self.assertRaises(ValidationError):
             relationship_assoc.clean()
 
+    def test_custom_validator_non_web_request_uses_anonymous_user(self):
+        location_type = LocationType.objects.get(name="Campus")
+        before = registry["plugin_custom_validators"]["dcim.locationtype"]
+        try:
+            registry["plugin_custom_validators"]["dcim.locationtype"] = [TestUserContextCustomValidator]
+
+            with self.assertRaises(ValidationError) as context:
+                location_type.clean()
+            self.assertEqual(context.exception.message, "TestUserContextCustomValidator: user is AnonymousUser")
+        finally:
+            registry["plugin_custom_validators"]["dcim.locationtype"] = before
+
+    def test_custom_validator_web_request_uses_real_user(self):
+        location_type = LocationType.objects.get(name="Campus")
+        before = registry["plugin_custom_validators"]["dcim.locationtype"]
+        try:
+            registry["plugin_custom_validators"]["dcim.locationtype"] = [TestUserContextCustomValidator]
+
+            with self.assertRaises(ValidationError) as context:
+                with web_request_context(user=self.user):
+                    location_type.clean()
+            self.assertEqual(context.exception.message, f"TestUserContextCustomValidator: user is {self.user}")
+        finally:
+            registry["plugin_custom_validators"]["dcim.locationtype"] = before
+
+
+class ExampleModelCustomActionViewTest(TestCase):
+    """Test for custom action view `all_names` added to Example App"""
+
+    model = ExampleModel
+
+    @classmethod
+    def setUpTestData(cls):
+        ExampleModel.objects.create(name="Example 1", number=100)
+        ExampleModel.objects.create(name="Example 2", number=200)
+        ExampleModel.objects.create(name="Example 3", number=300)
+        cls.custom_view_url = reverse("plugins:example_app:examplemodel_all_names")
+
+    @override_settings(EXEMPT_VIEW_PERMISSIONS=["*"])
+    def test_custom_action_view_anonymous(self):
+        self.client.logout()
+        response = self.client.get(self.custom_view_url)
+        self.assertHttpStatus(response, 200)
+        # TODO: all this is doing is checking that a login link appears somewhere on the page (i.e. in the nav).
+        response_body = response.content.decode(response.charset)
+        self.assertIn("/login/?next=", response_body, msg=response_body)
+
+    def test_custom_action_view_without_permission(self):
+        with disable_warnings("django.request"):
+            response = self.client.get(self.custom_view_url)
+            self.assertHttpStatus(response, 403)
+            response_body = response.content.decode(response.charset)
+            self.assertNotIn("/login/", response_body, msg=response_body)
+
+    def test_custom_action_view_with_permission(self):
+        self.add_permissions(f"{self.model._meta.app_label}.view_{self.model._meta.model_name}")
+
+        response = self.client.get(self.custom_view_url)
+        self.assertHttpStatus(response, 200)
+
+        response_body = extract_page_body(response.content.decode(response.charset))
+
+        for example_model in self.model.objects.all():
+            self.assertIn(example_model.name, response_body, msg=response_body)
+
+    def test_custom_action_view_with_constrained_permission(self):
+        instance1 = self.model.objects.first()
+        # Add object-level permission
+        obj_perm = ObjectPermission(
+            name="Test permission",
+            constraints={"pk": instance1.pk},
+            actions=["view"],
+        )
+        obj_perm.save()
+        obj_perm.users.add(self.user)
+        obj_perm.object_types.add(ContentType.objects.get_for_model(self.model))
+
+        response = self.client.get(self.custom_view_url)
+        self.assertHttpStatus(response, 200)
+
+        response_body = extract_page_body(response.content.decode(response.charset))
+
+        self.assertIn(instance1.name, response_body, msg=response_body)
+
+        for example_model in self.model.objects.exclude(pk=instance1.pk):
+            self.assertNotIn(example_model.name, response_body, msg=response_body)
+
 
 class FilterExtensionTest(TestCase):
     """
@@ -578,45 +706,136 @@ class FilterExtensionTest(TestCase):
         """
         Test that adding a custom filter, filters correctly.
         """
-        params = {"example_plugin_description": ["tenant-1.nautobot.com"]}
+        params = {"example_app_description": ["tenant-1.nautobot.com"]}
         self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
 
     def test_added_lookup(self):
         """
-        Test that the dynamically created filters work on plugin created filters as well.
+        Test that the dynamically created filters work on App created filters as well.
         """
-        params = {"example_plugin_description__ic": ["tenant-1.nautobot"]}
+        params = {"example_app_description__ic": ["tenant-1.nautobot"]}
         self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
 
     def test_nested_lookup(self):
         """
         Test that filters work against nested filters.
         """
-        params = {"example_plugin_dtype": ["Model 1"]}
+        params = {"example_app_dtype": ["Model 1"]}
         self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
-        params = {"example_plugin_dtype": ["Model 2"]}
+        params = {"example_app_dtype": ["Model 2"]}
         self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
 
     def test_filter_method_param(self):
         """
         Test that a custom filter works if a valid callable `method` is provided.
         """
-        params = {"example_plugin_sdescrip": ["tenant-1"]}
+        params = {"example_app_sdescrip": ["tenant-1"]}
         self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
 
     def test_filter_form(self):
         """
-        Test that filter forms work when added via a plugin.
+        Test that filter forms work when added via an App.
         """
         form = TenantFilterForm(
             data={
-                "example_plugin_description": "tenant-1.nautobot.com",
-                "example_plugin_dtype": "model-1",
-                "example_plugin_sdescrip": "tenant-1",
+                "example_app_description": "tenant-1.nautobot.com",
+                "example_app_dtype": "model-1",
+                "example_app_sdescrip": "tenant-1",
             }
         )
         self.assertTrue(form.is_valid())
-        self.assertIn("example_plugin_description", form.fields.keys())
+        self.assertIn("example_app_description", form.fields.keys())
+
+
+class TableExtensionTest(TestCase):
+    """Tests for adding table extensions."""
+
+    def test_alter_queryset(self):
+        """Test the 'alter_queryset' method of the TableExtension class."""
+        extension = plugins.TableExtension()
+        queryset = object()
+        result = extension.alter_queryset(queryset)
+        self.assertEqual(result, queryset)
+
+    def test__add_columns_into_model_table(self):
+        """Test the '_add_columns_into_model_table' function."""
+        extension = plugins.TableExtension()
+        extension.model = "tenancy.tenant"
+        extension.table_columns = {
+            "tenant_group": tables.Column(verbose_name="Name conflicts with existing column"),
+            "tenant_success": tables.Column(verbose_name="This name should be registered"),
+        }
+        extension.add_to_default_columns = ["tenant_group", "tenant_success"]
+
+        with self.assertLogs("nautobot.extras.plugins", level="WARNING") as logger:
+            plugins._add_columns_into_model_table(extension, "tenant")
+            table = get_table_for_model(extension.model)
+            expected = [
+                "ERROR:nautobot.extras.plugins:tenant: There was a name conflict with existing "
+                "table column `tenant_group`, the custom column was ignored."
+            ]
+            with self.subTest("error is logged"):
+                self.assertEqual(logger.output, expected)
+
+        with self.subTest("column is added to default_columns"):
+            self.assertIn("tenant_success", table.base_columns)
+
+    def test__add_column_to_table_base_columns(self):
+        """Test the '_add_column_to_table_base_columns' function."""
+        table = get_table_for_model("tenancy.tenant")
+
+        with self.subTest("raises TypeError"):
+            column = object()
+            with self.assertRaises(TypeError) as context:
+                plugins._add_column_to_table_base_columns(table, "test_column", column, "my_app")
+                self.assertEqual(
+                    context.exception, "Custom column `test_column` is not an instance of django_tables2.Column."
+                )
+
+        with self.subTest("raises AttributeError"):
+            column = tables.Column()
+            with self.assertRaises(AttributeError) as context:
+                plugins._add_column_to_table_base_columns(table, "pk", column, "my_app")
+                self.assertEqual(
+                    context.exception, "There was a conflict with table column `pk`, the custom column was ignored."
+                )
+
+        with self.subTest("Adds column to base_columns"):
+            column = tables.Column()
+            plugins._add_column_to_table_base_columns(table, "unique_name", column, "my_app")
+            self.assertIn("unique_name", table.base_columns)
+
+    def test__modify_default_table_columns(self):
+        """Test the '_modify_default_table_columns' function."""
+        extension = plugins.TableExtension()
+        extension.model = "tenancy.tenant"
+        extension.add_to_default_columns = ["new_column"]
+        extension.remove_from_default_columns = ["description"]
+
+        table = get_table_for_model(extension.model)
+        table.base_columns["new_column"] = tables.Column()
+
+        plugins._modify_default_table_columns(extension, "my_app")
+
+        with self.subTest("column is added to default_columns"):
+            self.assertIn("new_column", table.Meta.default_columns)
+
+        with self.subTest("column is removed from default_columns"):
+            self.assertNotIn("description", table.Meta.default_columns)
+
+    def test__validate_is_subclass_of_table_extension(self):
+        """Test the '_validate_is_subclass_of_table_extension' function."""
+        with self.assertRaises(TypeError):
+            plugins._validate_is_subclass_of_table_extension(object)
+
+    def test__validate_table_column_name_is_prefixed_with_app_name(self):
+        """Test the '_validate_table_column_name_is_prefixed_with_app_name' function."""
+        with self.assertRaises(ValueError) as context:
+            plugins._validate_table_column_name_is_prefixed_with_app_name("not_prefixed", "prefix")
+            self.assertEqual(
+                context.exception,
+                "Attempted to create a custom table column `not_prefixed` that did not start with `prefix`",
+            )
 
 
 class LoadPluginTest(TestCase):
@@ -630,20 +849,20 @@ class LoadPluginTest(TestCase):
     def test_load_plugin(self):
         """Test `load_plugin`."""
 
-        plugin_name = "bad.plugin"  # Start with a bogus plugin name
+        app_name = "bad.plugin"  # Start with a bogus App name
 
         # FIXME(jathan): We're expecting a PluginNotFound to be raised, but
         # unittest doesn't appear to let that happen and we only see the
         # original ModuleNotFoundError, so this will have to do for now.
         with self.assertRaises(ModuleNotFoundError):
-            load_plugin(plugin_name, settings)
+            load_plugin(app_name, settings)
 
-        # Move to the example plugin. No errors should be raised (which is good).
-        plugin_name = "example_plugin"
-        load_plugin(plugin_name, settings)
+        # Move to the example app. No errors should be raised (which is good).
+        app_name = "example_app"
+        load_plugin(app_name, settings)
 
 
-class TestPluginCoreViewOverrides(TestCase):
+class TestAppCoreViewOverrides(TestCase):
     """
     Validate that overridden core views work as expected.
 
@@ -665,16 +884,50 @@ class TestPluginCoreViewOverrides(TestCase):
         self.user.save()
 
     def test_views_are_overridden(self):
-        response = self.client.get(reverse("plugins:example_plugin:view_to_be_overridden"))
-        self.assertEqual(b"Hello world! I'm an overridden view.", response.content)
+        response = self.client.get(reverse("plugins:example_app:view_to_be_overridden"))
+        self.assertEqual("Hello world! I'm an overridden view.", response.content.decode(response.charset))
 
         response = self.client.get(
-            f'{reverse("plugins:plugin_detail", kwargs={"plugin": "example_plugin_with_view_override"})}'
+            f"{reverse('plugins:plugin_detail', kwargs={'plugin': 'example_app_with_view_override'})}"
         )
         self.assertIn(
-            (
-                b"plugins:example_plugin:view_to_be_overridden <code>"
-                b"example_plugin_with_view_override.views.ViewOverride</code>"
-            ),
-            response.content,
+            "plugins:example_app:view_to_be_overridden <code>example_app_with_view_override.views.ViewOverride</code>",
+            extract_page_body(response.content.decode(response.charset)),
         )
+
+
+class PluginTemplateExtensionsTest(TestCase):
+    """
+    Test that registered TemplateExtensions inject content as expected
+    """
+
+    def setUp(self):
+        super().setUp()
+        self.location = Location.objects.first()
+        self.user.is_superuser = True
+        self.user.save()
+
+    def test_list_view_buttons(self):
+        response = self.client.get(reverse("dcim:location_list"))
+        response_body = extract_page_body(response.content.decode(response.charset))
+        self.assertIn("LOCATION CONTENT - BUTTONS LIST", response_body, msg=response_body)
+
+    def test_detail_view_buttons(self):
+        response = self.client.get(reverse("dcim:location", kwargs={"pk": self.location.pk}))
+        response_body = extract_page_body(response.content.decode(response.charset))
+        self.assertIn("APP INJECTED LOCATION CONTENT - BUTTONS", response_body, msg=response_body)
+
+    def test_detail_view_left_page(self):
+        response = self.client.get(reverse("dcim:location", kwargs={"pk": self.location.pk}))
+        response_body = extract_page_body(response.content.decode(response.charset))
+        self.assertIn("App Injected Content - Left", response_body, msg=response_body)
+
+    def test_detail_view_right_page(self):
+        response = self.client.get(reverse("dcim:location", kwargs={"pk": self.location.pk}))
+        response_body = extract_page_body(response.content.decode(response.charset))
+        self.assertIn("App Injected Content - Right", response_body, msg=response_body)
+
+    def test_detail_view_full_width_page(self):
+        response = self.client.get(reverse("dcim:location", kwargs={"pk": self.location.pk}))
+        response_body = extract_page_body(response.content.decode(response.charset))
+        self.assertIn("App Injected Content - Full Width", response_body, msg=response_body)
