@@ -1,9 +1,12 @@
-import django_filters
-import netaddr
+import contextlib
+import uuid
+
 from django.core.exceptions import ValidationError
 from django.db.models import Q
-from netaddr.core import AddrFormatError
+import django_filters
+import netaddr
 
+from nautobot.cloud.models import CloudNetwork
 from nautobot.core.filters import (
     MultiValueCharFilter,
     MultiValueNumberFilter,
@@ -13,30 +16,36 @@ from nautobot.core.filters import (
     NumericArrayFilter,
     RelatedMembershipBooleanFilter,
     SearchFilter,
+    TreeNodeMultipleChoiceFilter,
 )
 from nautobot.dcim.filters import LocatableModelFilterSetMixin
-from nautobot.dcim.models import Device, Interface
+from nautobot.dcim.models import Device, Interface, Location, VirtualDeviceContext
 from nautobot.extras.filters import NautobotFilterSet, RoleModelFilterSetMixin, StatusModelFilterSetMixin
-from nautobot.ipam import choices
-from nautobot.tenancy.filters import TenancyModelFilterSetMixin
+from nautobot.ipam import choices, formfields
+from nautobot.tenancy.filters.mixins import TenancyModelFilterSetMixin
 from nautobot.virtualization.models import VirtualMachine, VMInterface
+
 from .models import (
     IPAddress,
     IPAddressToInterface,
     Namespace,
     Prefix,
+    PrefixLocationAssignment,
     RIR,
     RouteTarget,
     Service,
     VLAN,
     VLANGroup,
+    VLANLocationAssignment,
     VRF,
+    VRFDeviceAssignment,
+    VRFPrefixAssignment,
 )
 
-
 __all__ = (
     "IPAddressFilterSet",
     "NamespaceFilterSet",
+    "PrefixFilter",
     "PrefixFilterSet",
     "RIRFilterSet",
     "RouteTargetFilterSet",
@@ -47,6 +56,39 @@ __all__ = (
 )
 
 
+class PrefixFilter(NaturalKeyOrPKMultipleChoiceFilter):
+    """
+    Filter that supports filtering a foreign key to Prefix by either its PK or by a literal `prefix` string.
+    """
+
+    field_class = formfields.PrefixFilterFormField
+
+    def __init__(self, *args, **kwargs):
+        kwargs.setdefault("to_field_name", "pk")
+        kwargs.setdefault("label", "Prefix (ID or prefix string)")
+        kwargs.setdefault("queryset", Prefix.objects.all())
+        super().__init__(*args, **kwargs)
+
+    def get_filter_predicate(self, v):
+        # Null value filtering
+        if v is None:
+            return {f"{self.field_name}__isnull": True}
+
+        # If value is a model instance, stringify it to a pk.
+        if isinstance(v, Prefix):
+            v = v.pk
+
+        # Try to cast the value to a UUID to distinguish between PKs and prefix strings
+        v = str(v)
+        try:
+            uuid.UUID(v)
+            return {self.field_name: v}
+        except (AttributeError, TypeError, ValueError):
+            # It's a prefix string
+            prefixes_queryset = Prefix.objects.net_equals(v)
+            return {f"{self.field_name}__in": prefixes_queryset.values_list("pk", flat=True)}
+
+
 class NamespaceFilterSet(NautobotFilterSet):
     q = SearchFilter(
         filter_predicates={
@@ -59,7 +101,7 @@ class NamespaceFilterSet(NautobotFilterSet):
         fields = "__all__"
 
 
-class VRFFilterSet(NautobotFilterSet, TenancyModelFilterSetMixin):
+class VRFFilterSet(NautobotFilterSet, StatusModelFilterSetMixin, TenancyModelFilterSetMixin):
     q = SearchFilter(
         filter_predicates={
             "name": "icontains",
@@ -83,23 +125,84 @@ class VRFFilterSet(NautobotFilterSet, TenancyModelFilterSetMixin):
         to_field_name="name",
         label="Device (ID or name)",
     )
-    prefix = NaturalKeyOrPKMultipleChoiceFilter(
-        field_name="prefixes",
-        queryset=Prefix.objects.all(),
-        to_field_name="pk",  # TODO(jathan): Make this work with `prefix` "somehow"
-        label="Prefix (ID or name)",
+    virtual_machines = NaturalKeyOrPKMultipleChoiceFilter(
+        queryset=VirtualMachine.objects.all(),
+        to_field_name="name",
+        label="Virtual Machine (ID or name)",
     )
+    prefix = PrefixFilter(field_name="prefixes")
     namespace = NaturalKeyOrPKMultipleChoiceFilter(
         queryset=Namespace.objects.all(),
         to_field_name="name",
         label="Namespace (name or ID)",
     )
+    virtual_device_contexts = NaturalKeyOrPKMultipleChoiceFilter(
+        queryset=VirtualDeviceContext.objects.all(),
+        to_field_name="name",
+        label="Virtual Device Context (ID or name)",
+    )
 
     class Meta:
         model = VRF
         fields = ["id", "name", "rd", "tags"]
 
 
+class VRFDeviceAssignmentFilterSet(NautobotFilterSet):
+    q = SearchFilter(
+        filter_predicates={
+            "name": "icontains",
+            "vrf__name": "icontains",
+            "device__name": "icontains",
+            "virtual_machine__name": "icontains",
+            "virtual_device_context__name": "icontains",
+            "rd": "icontains",
+        },
+    )
+    vrf = NaturalKeyOrPKMultipleChoiceFilter(
+        queryset=VRF.objects.all(),
+        to_field_name="name",
+        label="VRF (ID or name)",
+    )
+    device = NaturalKeyOrPKMultipleChoiceFilter(
+        queryset=Device.objects.all(),
+        to_field_name="name",
+        label="Device (ID or name)",
+    )
+    virtual_machine = NaturalKeyOrPKMultipleChoiceFilter(
+        queryset=VirtualMachine.objects.all(),
+        to_field_name="name",
+        label="Virtual Machine (ID or name)",
+    )
+    virtual_device_context = NaturalKeyOrPKMultipleChoiceFilter(
+        queryset=VirtualDeviceContext.objects.all(),
+        to_field_name="name",
+        label="Virtual Device Context (ID or name)",
+    )
+
+    class Meta:
+        model = VRFDeviceAssignment
+        fields = ["id", "name", "rd"]
+
+
+class VRFPrefixAssignmentFilterSet(NautobotFilterSet):
+    q = SearchFilter(
+        filter_predicates={
+            # "prefix__prefix": "iexact",  # TODO?
+            "vrf__name": "icontains",
+        },
+    )
+    prefix = PrefixFilter()
+    vrf = NaturalKeyOrPKMultipleChoiceFilter(
+        queryset=VRF.objects.all(),
+        to_field_name="name",
+        label="VRF (ID or name)",
+    )
+
+    class Meta:
+        model = VRFPrefixAssignment
+        fields = ["id", "vrf", "prefix"]
+
+
 class RouteTargetFilterSet(NautobotFilterSet, TenancyModelFilterSetMixin):
     q = SearchFilter(
         filter_predicates={
@@ -149,28 +252,24 @@ class IPAMFilterSetMixin(django_filters.FilterSet):
 class PrefixFilterSet(
     NautobotFilterSet,
     IPAMFilterSetMixin,
-    LocatableModelFilterSetMixin,
     TenancyModelFilterSetMixin,
     StatusModelFilterSetMixin,
     RoleModelFilterSetMixin,
 ):
-    # Prefix doesn't have an appropriate single natural-key field for a NaturalKeyOrPKMultipleChoiceFilter
-    parent = django_filters.ModelMultipleChoiceFilter(
-        queryset=Prefix.objects.all(),
-    )
-    prefix = django_filters.CharFilter(
+    parent = PrefixFilter()
+    prefix = MultiValueCharFilter(
         method="filter_prefix",
         label="Prefix",
     )
-    within = django_filters.CharFilter(
+    within = MultiValueCharFilter(
         method="search_within",
         label="Within prefix",
     )
-    within_include = django_filters.CharFilter(
+    within_include = MultiValueCharFilter(
         method="search_within_include",
         label="Within and including prefix",
     )
-    contains = django_filters.CharFilter(
+    contains = MultiValueCharFilter(
         method="search_contains",
         label="Prefixes which contain this prefix or IP",
     )
@@ -216,62 +315,69 @@ class PrefixFilterSet(
         label="Namespace (name or ID)",
     )
     ip_version = django_filters.NumberFilter()
+    location = TreeNodeMultipleChoiceFilter(
+        prefers_id=True,
+        queryset=Location.objects.all(),
+        to_field_name="name",
+        field_name="locations",
+        label='Location (name or ID) (deprecated, use "locations" filter instead)',
+    )
+    locations = TreeNodeMultipleChoiceFilter(
+        prefers_id=True,
+        queryset=Location.objects.all(),
+        to_field_name="name",
+        label="Locations (name or ID)",
+    )
+    cloud_networks = NaturalKeyOrPKMultipleChoiceFilter(
+        queryset=CloudNetwork.objects.all(),
+        to_field_name="name",
+        label="Cloud Network (name or ID)",
+    )
 
     class Meta:
         model = Prefix
         fields = ["date_allocated", "id", "prefix_length", "tags"]
 
+    def _strip_values(self, values):
+        return [value.strip() for value in values if value.strip()]
+
     def filter_prefix(self, queryset, name, value):
-        value = value.strip()
-        if not value:
-            return queryset
-        # filter for Prefix models equal to |value|
-        try:
-            return queryset.net_equals(netaddr.IPNetwork(value))
-        except (AddrFormatError, ValueError):
-            return queryset.none()
+        prefixes = self._strip_values(value)
+        with contextlib.suppress(netaddr.AddrFormatError, ValueError):
+            return queryset.net_equals(*prefixes)
+        return queryset.none()
 
     def search_within(self, queryset, name, value):
-        value = value.strip()
-        if not value:
-            return queryset
-        try:
-            return queryset.net_contained(netaddr.IPNetwork(value))
-        except (AddrFormatError, ValueError):
-            return queryset.none()
+        prefixes = self._strip_values(value)
+        with contextlib.suppress(netaddr.AddrFormatError, ValueError):
+            return queryset.net_contained(*prefixes)
+        return queryset.none()
 
     def search_within_include(self, queryset, name, value):
-        value = value.strip()
-        if not value:
-            return queryset
-        try:
-            return queryset.net_contained_or_equal(netaddr.IPNetwork(value))
-        except (AddrFormatError, ValueError):
-            return queryset.none()
+        prefixes = self._strip_values(value)
+        with contextlib.suppress(netaddr.AddrFormatError, ValueError):
+            return queryset.net_contained_or_equal(*prefixes)
+        return queryset.none()
 
     def search_contains(self, queryset, name, value):
-        value = value.strip()
-        if not value:
-            return queryset
-        try:
-            # Searching by prefix
-            if "/" in value:
-                return queryset.net_contains_or_equals(netaddr.IPNetwork(value).cidr)
-            # Searching by IP address
-            else:
-                # filter for Prefixes containing |value|
-                # netaddr.IPAddress objects have no netmask
-                # so prefix_length is not considered
-                query = netaddr.IPAddress(value)
-                return queryset.filter(
-                    network__lte=bytes(query),
-                    broadcast__gte=bytes(query),
-                )
-        except (AddrFormatError, ValueError):
-            return queryset.none()
+        prefixes_queryset = queryset.none()
+        values = self._strip_values(value)
+
+        if prefixes := [prefix for prefix in values if "/" in prefix]:
+            with contextlib.suppress(netaddr.AddrFormatError, ValueError):
+                prefixes_queryset |= queryset.net_contains_or_equals(*prefixes)
+
+        if prefixes_without_length := [prefix for prefix in values if "/" not in prefix]:
+            query = Q()
+            for _prefix in prefixes_without_length:
+                with contextlib.suppress(netaddr.AddrFormatError, ValueError):
+                    prefix = netaddr.IPAddress(_prefix)
+                    query |= Q(network__lte=bytes(prefix), broadcast__gte=bytes(prefix))
+            prefixes_queryset |= queryset.filter(query)
+        return prefixes_queryset
 
     def generate_query_filter_present_in_vrf(self, value):
-        if isinstance(value, str):
+        if isinstance(value, (str, uuid.UUID)):
             value = VRF.objects.get(pk=value)
 
         query = Q(vrfs=value) | Q(vrfs__export_targets__in=value.import_targets.all())
@@ -284,6 +390,28 @@ class PrefixFilterSet(
         return queryset.filter(params).distinct()
 
 
+class PrefixLocationAssignmentFilterSet(NautobotFilterSet):
+    q = SearchFilter(
+        filter_predicates={
+            "location__name": "icontains",
+        },
+    )
+    prefix = PrefixFilter()
+    location = TreeNodeMultipleChoiceFilter(
+        prefers_id=True,
+        queryset=Location.objects.all(),
+        to_field_name="name",
+        label="Locations (name or ID)",
+    )
+
+    def _strip_values(self, values):
+        return [value.strip() for value in values if value.strip()]
+
+    class Meta:
+        model = PrefixLocationAssignment
+        fields = ["id", "prefix", "location"]
+
+
 class IPAddressFilterSet(
     NautobotFilterSet,
     IPAMFilterSetMixin,
@@ -295,7 +423,7 @@ class IPAddressFilterSet(
         queryset=Prefix.objects.all(),
         label="Parent prefix",
     )
-    prefix = django_filters.CharFilter(
+    prefix = MultiValueCharFilter(
         method="search_by_prefix",
         label="Contained in prefix",
     )
@@ -363,11 +491,19 @@ class IPAddressFilterSet(
         method="_has_interface_assignments",
         label="Has Interface Assignments",
     )
+    nat_inside = django_filters.ModelMultipleChoiceFilter(
+        queryset=IPAddress.objects.all(),
+        label="NAT (Inside)",
+    )
+    has_nat_inside = RelatedMembershipBooleanFilter(
+        field_name="nat_inside",
+        label="Has NAT Inside",
+    )
     ip_version = django_filters.NumberFilter()
 
     class Meta:
         model = IPAddress
-        fields = ["id", "dns_name", "type", "tags", "mask_length"]
+        fields = ["id", "dns_name", "type", "tags", "mask_length", "nat_inside"]
 
     def generate_query__has_interface_assignments(self, value):
         """Helper method used by DynamicGroups and by _assigned_to_interface method."""
@@ -383,14 +519,8 @@ class IPAddressFilterSet(
         return queryset.filter(params)
 
     def search_by_prefix(self, queryset, name, value):
-        value = value.strip()
-        if not value:
-            return queryset
-        try:
-            query = netaddr.IPNetwork(value.strip()).cidr
-            return queryset.net_host_contained(query)
-        except (AddrFormatError, ValueError):
-            return queryset.none()
+        prefixes = [prefix.strip() for prefix in value if prefix.strip()]
+        return queryset.net_host_contained(*prefixes)
 
     def filter_address(self, queryset, name, value):
         try:
@@ -399,7 +529,7 @@ class IPAddressFilterSet(
             return queryset.none()
 
     def generate_query_filter_present_in_vrf(self, value):
-        if isinstance(value, str):
+        if isinstance(value, (str, uuid.UUID)):
             value = VRF.objects.get(pk=value)
 
         query = Q(parent__vrfs=value) | Q(parent__vrfs__export_targets__in=value.import_targets.all())
@@ -431,6 +561,12 @@ class IPAddressFilterSet(
 
 
 class IPAddressToInterfaceFilterSet(NautobotFilterSet):
+    q = SearchFilter(
+        filter_predicates={
+            "interface__name": "icontains",
+            "vm_interface__name": "icontains",
+        },
+    )
     interface = NaturalKeyOrPKMultipleChoiceFilter(
         queryset=Interface.objects.all(),
         label="Interface (name or ID)",
@@ -452,12 +588,11 @@ class IPAddressToInterfaceFilterSet(NautobotFilterSet):
 class VLANGroupFilterSet(NautobotFilterSet, LocatableModelFilterSetMixin, NameSearchFilterSet):
     class Meta:
         model = VLANGroup
-        fields = ["id", "name", "description"]
+        fields = ["id", "name", "description", "tags"]
 
 
 class VLANFilterSet(
     NautobotFilterSet,
-    LocatableModelFilterSetMixin,
     TenancyModelFilterSetMixin,
     StatusModelFilterSetMixin,
     RoleModelFilterSetMixin,
@@ -481,20 +616,58 @@ class VLANFilterSet(
         queryset=VLANGroup.objects.all(),
         label="VLAN Group (name or ID)",
     )
+    location = TreeNodeMultipleChoiceFilter(
+        prefers_id=True,
+        queryset=Location.objects.all(),
+        to_field_name="name",
+        field_name="locations",
+        label='Location (name or ID) (deprecated, use "locations" filter instead)',
+    )
+    locations = TreeNodeMultipleChoiceFilter(
+        prefers_id=True,
+        queryset=Location.objects.all(),
+        to_field_name="name",
+        label="Locations (name or ID)",
+    )
 
     class Meta:
         model = VLAN
         fields = ["id", "name", "tags", "vid"]
 
     def get_for_device(self, queryset, name, value):
-        # TODO: after Location model replaced Site, which was not a hierarchical model, should we consider to include
-        # VLANs that belong to the parent/child locations of the `device.location`?
         """Return all VLANs available to the specified Device(value)."""
         devices = Device.objects.select_related("location").filter(**{f"{name}__in": value})
         if not devices.exists():
             return queryset.none()
         location_ids = list(devices.values_list("location__id", flat=True))
-        return queryset.filter(Q(location__isnull=True) | Q(location__in=location_ids))
+        for location in Location.objects.filter(pk__in=location_ids):
+            location_ids.extend([ancestor.id for ancestor in location.ancestors()])
+        return queryset.filter(Q(locations__isnull=True) | Q(locations__in=location_ids))
+
+
+class VLANLocationAssignmentFilterSet(NautobotFilterSet):
+    q = SearchFilter(
+        filter_predicates={
+            "vlan__vid": "iexact",
+            "location__name": "icontains",
+        },
+    )
+    vlan = NaturalKeyOrPKMultipleChoiceFilter(
+        prefers_id=True,
+        to_field_name="vid",
+        queryset=VLAN.objects.all(),
+        label="VLAN (VID or ID)",
+    )
+    location = TreeNodeMultipleChoiceFilter(
+        prefers_id=True,
+        queryset=Location.objects.all(),
+        to_field_name="name",
+        label="Locations (name or ID)",
+    )
+
+    class Meta:
+        model = VLANLocationAssignment
+        fields = ["id", "vlan", "location"]
 
 
 class ServiceFilterSet(NautobotFilterSet):
