@@ -18,6 +18,8 @@ def migrate_null_statuses(apps, schema):
 
 class Migration(migrations.Migration):
     dependencies = [
+        ("contenttypes", "0002_remove_content_type_name"),
+        ("extras", "0001_initial_part_1"),
         ("ipam", "0032_ipam__namespaces_finish"),
     ]
 
