@@ -1,4 +1,3 @@
-from django.test import override_settings
 from django.urls import reverse
 from rest_framework import status
 
@@ -9,8 +8,8 @@ class TestPrefix(APITestCase):
     def setUp(self):
         super().setUp()
         self.api_url = reverse("graphql-api")
+        self.add_permissions("ipam.view_prefix")
 
-    @override_settings(EXEMPT_VIEW_PERMISSIONS=["*"])
     def test_prefix_ip_version(self):
         """Test ip_version is available for a Prefix via GraphQL."""
         get_prefixes_query = """
@@ -23,7 +22,7 @@ class TestPrefix(APITestCase):
         }
         """
         payload = {"query": get_prefixes_query}
-        response = self.client.post(self.api_url, payload, format="json")
+        response = self.client.post(self.api_url, payload, format="json", **self.header)
         self.assertEqual(response.status_code, status.HTTP_200_OK)
         prefixes = response.data["data"]["prefixes"]
         self.assertIsInstance(prefixes, list)
