@@ -1,18 +1,29 @@
 import datetime
 import random
 
-from netaddr import IPNetwork
 from django.contrib.contenttypes.models import ContentType
+from django.db.models import Count
 from django.test import override_settings
-from django.utils.html import strip_tags
-from django.utils.timezone import make_aware
 from django.urls import reverse
+from django.utils.html import strip_tags
+from django.utils.http import urlencode
+from django.utils.timezone import make_aware
+from netaddr import IPNetwork
 
 from nautobot.circuits.models import Circuit, Provider
-from nautobot.core.templatetags.helpers import queryset_to_pks
-from nautobot.core.testing import post_data, ModelViewTestCase, ViewTestCases
+from nautobot.core.templatetags.helpers import hyperlinked_object, queryset_to_pks
+from nautobot.core.testing import ModelViewTestCase, post_data, ViewTestCases
 from nautobot.core.testing.utils import extract_page_body
-from nautobot.dcim.models import Device, DeviceType, Interface, Location, LocationType, Manufacturer
+from nautobot.core.utils.lookup import get_route_for_model
+from nautobot.dcim.models import (
+    Device,
+    DeviceType,
+    Interface,
+    Location,
+    LocationType,
+    Manufacturer,
+    VirtualDeviceContext,
+)
 from nautobot.extras.choices import CustomFieldTypeChoices, RelationshipTypeChoices
 from nautobot.extras.models import (
     CustomField,
@@ -26,7 +37,6 @@ from nautobot.extras.models import (
 from nautobot.ipam.choices import IPAddressTypeChoices, PrefixTypeChoices, ServiceProtocolChoices
 from nautobot.ipam.models import (
     IPAddress,
-    IPAddressToInterface,
     Namespace,
     Prefix,
     RIR,
@@ -38,7 +48,7 @@ from nautobot.ipam.models import (
 )
 from nautobot.tenancy.models import Tenant
 from nautobot.users.models import ObjectPermission
-from nautobot.virtualization.models import Cluster, ClusterType, VirtualMachine, VMInterface
+from nautobot.virtualization.models import Cluster, ClusterType, VirtualMachine
 
 
 class NamespaceTestCase(
@@ -49,22 +59,26 @@ class NamespaceTestCase(
     ViewTestCases.EditObjectViewTestCase,
     ViewTestCases.DeleteObjectViewTestCase,
     ViewTestCases.ListObjectsViewTestCase,
-    ViewTestCases.BulkImportObjectsViewTestCase,
+    ViewTestCases.BulkEditObjectsViewTestCase,
+    ViewTestCases.BulkDeleteObjectsViewTestCase,
 ):
     model = Namespace
+    custom_action_required_permissions = {
+        "ipam:namespace_vrfs": ["ipam.view_namespace", "ipam.view_vrf"],
+        "ipam:namespace_prefixes": ["ipam.view_namespace", "ipam.view_prefix"],
+        "ipam:namespace_ip_addresses": ["ipam.view_namespace", "ipam.view_ipaddress"],
+    }
 
     @classmethod
     def setUpTestData(cls):
-        locations = Location.objects.all()[:4]
+        locations = Location.objects.get_for_model(Namespace)
 
         cls.form_data = {"name": "Namespace X", "location": locations[0].pk, "description": "A new Namespace"}
 
-        cls.csv_data = (
-            "name,location",
-            f"Namespace 4,{locations[1].pk}",
-            f"Namespace 5,{locations[2].pk}",
-            "Namespace 6,",
-        )
+        cls.bulk_edit_data = {
+            "description": "New description",
+            "location": locations[1].pk,
+        }
 
 
 class VRFTestCase(ViewTestCases.PrimaryObjectViewTestCase):
@@ -73,7 +87,10 @@ class VRFTestCase(ViewTestCases.PrimaryObjectViewTestCase):
     @classmethod
     def setUpTestData(cls):
         tenants = Tenant.objects.all()[:2]
-        namespace = Namespace.objects.create(name="ipam_test_views_vrf_test")
+        namespace = Namespace.objects.annotate(prefix_count=Count("prefixes")).filter(prefix_count__gt=2).first()
+        prefixes = Prefix.objects.filter(namespace=namespace)
+        vdcs = VirtualDeviceContext.objects.all()
+        vrf_statuses = Status.objects.get_for_model(VRF)
 
         cls.form_data = {
             "name": "VRF X",
@@ -81,19 +98,21 @@ class VRFTestCase(ViewTestCases.PrimaryObjectViewTestCase):
             "rd": "65000:999",
             "tenant": tenants[0].pk,
             "description": "A new VRF",
+            "prefixes": [prefixes[1].id],
             "tags": [t.pk for t in Tag.objects.get_for_model(VRF)],
+            "status": vrf_statuses.first().pk,
+            "virtual_device_contexts": [vdcs[0].id, vdcs[1].id],
         }
 
-        cls.csv_data = (
-            "name,rd,namespace",
-            f"VRF 4,abc123,{namespace.name}",
-            f"VRF 5,xyz246,{namespace.name}",
-            f"VRF 6,,{namespace.name}",
-        )
-
         cls.bulk_edit_data = {
+            "status": vrf_statuses.first().pk,
             "tenant": tenants[1].pk,
             "description": "New description",
+            "namespace": prefixes[0].namespace.id,
+            "add_prefixes": [prefixes[0].id],
+            "remove_prefixes": [prefixes[1].id],
+            "add_virtual_device_contexts": [vdcs[2].id, vdcs[3].id],
+            "remove_virtual_device_contexts": [vdcs[0].id],
         }
 
 
@@ -110,13 +129,6 @@ class RouteTargetTestCase(ViewTestCases.PrimaryObjectViewTestCase):
             "tags": [t.pk for t in Tag.objects.get_for_model(RouteTarget)],
         }
 
-        cls.csv_data = (
-            "name,tenant,description",
-            f'65000:1004,"{tenants[0].name}",Foo',
-            f'65000:1005,"{tenants[1].name}",Bar',
-            "65000:1006,,No tenant",
-        )
-
         cls.bulk_edit_data = {
             "tenant": tenants[1].pk,
             "description": "New description",
@@ -134,22 +146,34 @@ class RIRTestCase(ViewTestCases.OrganizationalObjectViewTestCase):
             "description": "A new RIR",
         }
 
-        cls.csv_data = (
-            "name,description",
-            "RIR 4,Fourth RIR",
-            "RIR 5,Fifth RIR",
-            "RIR 6,Sixth RIR",
-            "RIR 7,Seventh RIR",
-        )
-
     def setUp(self):
         super().setUp()
         # Ensure that we have at least one RIR with no prefixes that can be used for the "delete_object" tests.
         RIR.objects.create(name="RIR XYZ")
 
+    @override_settings(EXEMPT_VIEW_PERMISSIONS=[])
+    def test_list_objects_with_permission(self):
+        """Test rendering of LinkedCountColumn for related fields without display_field override."""
+        response = super().test_list_objects_with_permission()
+        response_body = extract_page_body(response.content.decode(response.charset))
+
+        prefix_list_url = reverse(get_route_for_model(Prefix, "list"))
+
+        for rir in self._get_queryset().all():
+            if str(rir.pk) in response_body:
+                count = rir.prefixes.count()
+                if count > 1:
+                    self.assertBodyContains(
+                        response,
+                        f'<a href="{prefix_list_url}?{urlencode({"rir": rir.name})}" class="badge">{count}</a>',
+                    )
+                elif count == 1:
+                    self.assertBodyContains(response, hyperlinked_object(rir.prefixes.first()))
+
 
 class PrefixTestCase(ViewTestCases.PrimaryObjectViewTestCase, ViewTestCases.ListObjectsViewTestCase):
     model = Prefix
+    filter_on_field = "prefix_length"
 
     @classmethod
     def setUpTestData(cls):
@@ -166,7 +190,7 @@ class PrefixTestCase(ViewTestCases.PrimaryObjectViewTestCase, ViewTestCases.List
         cls.form_data = {
             "prefix": IPNetwork("192.0.2.0/24"),
             "namespace": cls.namespace.pk,
-            "location": cls.locations[1].pk,
+            "locations": [cls.locations[1].pk],
             "vrf": vrfs[1].pk,
             "tenant": None,
             "vlan": None,
@@ -179,24 +203,38 @@ class PrefixTestCase(ViewTestCases.PrimaryObjectViewTestCase, ViewTestCases.List
             "tags": [t.pk for t in Tag.objects.get_for_model(Prefix)],
         }
 
-        cls.csv_data = (
-            "vrf,prefix,status,rir,namespace",
-            f"{vrfs[0].name},10.4.0.0/16,{cls.statuses[0].name},{rir.name},{cls.namespace.name}",
-            f"{vrfs[0].name},10.5.0.0/16,{cls.statuses[0].name},{rir.name},{cls.namespace.name}",
-            f"{vrfs[0].name},10.6.0.0/16,{cls.statuses[1].name},{rir.name},{cls.namespace.name}",
-        )
-
         cls.bulk_edit_data = {
-            "location": None,
-            "vrf": vrfs[1].pk,
             "tenant": None,
             "status": cls.statuses[1].pk,
             "role": cls.roles[1].pk,
             "rir": RIR.objects.last().pk,
             "date_allocated": make_aware(datetime.datetime(2020, 1, 1, 0, 0, 0, 0)),
             "description": "New description",
+            "add_locations": [cls.locations[0].pk],
+            "remove_locations": [cls.locations[1].pk],
+            "namespace": vrfs[0].namespace.pk,
+            "add_vrfs": [vrfs[0].pk],
+            "remove_vrfs": [vrfs[1].pk],
         }
 
+    @override_settings(EXEMPT_VIEW_PERMISSIONS=[])
+    def test_list_objects_with_permission(self):
+        """Test rendering of LinkedCountColumn for related fields with display_field override."""
+        response = super().test_list_objects_with_permission()
+        response_body = extract_page_body(response.content.decode(response.charset))
+
+        locations_list_url = reverse(get_route_for_model(Location, "list"))
+
+        for prefix in self._get_queryset().all():
+            if str(prefix.pk) in response_body:
+                count = prefix.locations.count()
+                if count > 1:
+                    self.assertBodyContains(
+                        response, f'<a href="{locations_list_url}?prefixes={prefix.pk}" class="badge">{count}</a>'
+                    )
+                elif count == 1:
+                    self.assertBodyContains(response, hyperlinked_object(prefix.locations.first(), "name"))
+
     @override_settings(EXEMPT_VIEW_PERMISSIONS=["*"])
     def test_empty_queryset(self):
         """
@@ -347,6 +385,43 @@ class PrefixTestCase(ViewTestCases.PrimaryObjectViewTestCase, ViewTestCases.List
                 strip_tags(content),
             )
 
+    @override_settings(EXEMPT_VIEW_PERMISSIONS=["*"])
+    def test_prefix_ipaddresses_table_list_includes_child_ips(self):
+        ip_status = Status.objects.get_for_model(IPAddress).first()
+        instance = Prefix.objects.create(
+            prefix="5.5.10.0/23",
+            namespace=self.namespace,
+            type=PrefixTypeChoices.TYPE_NETWORK,
+            status=self.statuses[1],
+        )
+        Prefix.objects.create(
+            prefix="5.5.10.0/30",
+            namespace=self.namespace,
+            type=PrefixTypeChoices.TYPE_POOL,
+            status=self.statuses[1],
+        )
+        IPAddress.objects.create(
+            address="5.5.10.1/23",
+            status=ip_status,
+            namespace=self.namespace,
+        )
+        IPAddress.objects.create(
+            address="5.5.10.4/23",
+            status=ip_status,
+            namespace=self.namespace,
+        )
+        url = reverse("ipam:prefix_ipaddresses", args=(instance.pk,))
+        response = self.client.get(url)
+        self.assertHttpStatus(response, 200)
+        content = extract_page_body(response.content.decode(response.charset))
+        # This validates that both parent prefix and child prefix IPAddresses are present in parent prefix IPAddresses list
+        self.assertIn("5.5.10.1/23", strip_tags(content))
+        self.assertIn("5.5.10.4/23", strip_tags(content))
+        ip_address_tab = (
+            f'<li role="presentation" class="active"><a href="{url}">IP Addresses <span class="badge">2</span></a></li>'
+        )
+        self.assertInHTML(ip_address_tab, content)
+
 
 class IPAddressTestCase(ViewTestCases.PrimaryObjectViewTestCase):
     model = IPAddress
@@ -357,7 +432,7 @@ class IPAddressTestCase(ViewTestCases.PrimaryObjectViewTestCase):
         cls.statuses = Status.objects.get_for_model(IPAddress)
         cls.prefix_status = Status.objects.get_for_model(Prefix).first()
         roles = Role.objects.get_for_model(IPAddress)
-        parent, _ = Prefix.objects.get_or_create(
+        Prefix.objects.get_or_create(
             prefix="192.0.2.0/24",
             defaults={"namespace": cls.namespace, "status": cls.prefix_status, "type": "network"},
         )
@@ -375,13 +450,6 @@ class IPAddressTestCase(ViewTestCases.PrimaryObjectViewTestCase):
             "tags": [t.pk for t in Tag.objects.get_for_model(IPAddress)],
         }
 
-        cls.csv_data = (
-            "address,status,parent",
-            f"192.0.2.4/24,{cls.statuses[0].name},{parent.composite_key}",
-            f"192.0.2.5/24,{cls.statuses[0].name},{parent.composite_key}",
-            f"192.0.2.6/24,{cls.statuses[0].name},{parent.composite_key}",
-        )
-
         cls.bulk_edit_data = {
             "tenant": None,
             "status": cls.statuses[1].pk,
@@ -428,8 +496,7 @@ class IPAddressTestCase(ViewTestCases.PrimaryObjectViewTestCase):
             "data": post_data(self.form_data),
         }
         response = self.client.post(**request)
-        self.assertEqual(200, response.status_code)
-        self.assertIn("Host address cannot be changed once created", str(response.content))
+        self.assertBodyContains(response, "Host address cannot be changed once created")
 
     @override_settings(EXEMPT_VIEW_PERMISSIONS=["*"])
     def test_move_ip_addresses_between_namespaces(self):
@@ -449,8 +516,7 @@ class IPAddressTestCase(ViewTestCases.PrimaryObjectViewTestCase):
             "data": post_data(form_data),
         }
         response = self.client.post(**request)
-        self.assertEqual(200, response.status_code)
-        self.assertIn("No suitable parent Prefix exists in this Namespace", str(response.content))
+        self.assertBodyContains(response, "No suitable parent Prefix exists in this Namespace")
         # Create an exact copy of the parent prefix but in a different namespace. See if the re-parenting is successful
         new_parent = Prefix.objects.create(
             prefix=instance.parent.prefix,
@@ -558,10 +624,11 @@ class IPAddressMergeTestCase(ModelViewTestCase):
         cls.devices = devices
 
         intf_status = Status.objects.get_for_model(Interface).first()
+        intf_role = Role.objects.get_for_model(Interface).first()
         cls.interfaces = (
-            Interface.objects.create(device=cls.devices[0], name="Interface 1", status=intf_status),
+            Interface.objects.create(device=cls.devices[0], name="Interface 1", status=intf_status, role=intf_role),
             Interface.objects.create(device=cls.devices[1], name="Interface 2", status=intf_status),
-            Interface.objects.create(device=cls.devices[2], name="Interface 3", status=intf_status),
+            Interface.objects.create(device=cls.devices[2], name="Interface 3", status=intf_status, role=intf_role),
         )
         cls.services = (
             Service.objects.create(
@@ -620,7 +687,7 @@ class IPAddressMergeTestCase(ModelViewTestCase):
                 ],
             },
         )
-        cls.dup_ip_1.tags.set(random.choices(Tag.objects.get_for_model(IPAddress), k=3))
+        cls.dup_ip_1.tags.set(random.choices(Tag.objects.get_for_model(IPAddress), k=3))  # noqa: S311  # suspicious-non-cryptographic-random-usage -- ok here in test code
         parent_2, _ = Prefix.objects.get_or_create(
             prefix="94.0.0.2/12",
             defaults={"namespace": cls.namespace_2, "status": prefix_status, "type": "network"},
@@ -644,7 +711,7 @@ class IPAddressMergeTestCase(ModelViewTestCase):
                 ],
             },
         )
-        cls.dup_ip_2.tags.set(random.choices(Tag.objects.get_for_model(IPAddress), k=2))
+        cls.dup_ip_2.tags.set(random.choices(Tag.objects.get_for_model(IPAddress), k=2))  # noqa: S311  # suspicious-non-cryptographic-random-usage -- ok here in test code
         parent_3, _ = Prefix.objects.get_or_create(
             prefix="94.0.0.2/15",
             defaults={"namespace": namespace_3, "status": prefix_status, "type": "network"},
@@ -668,7 +735,7 @@ class IPAddressMergeTestCase(ModelViewTestCase):
                 ],
             },
         )
-        cls.dup_ip_3.tags.set(random.choices(Tag.objects.get_for_model(IPAddress), k=3))
+        cls.dup_ip_3.tags.set(random.choices(Tag.objects.get_for_model(IPAddress), k=3))  # noqa: S311  # suspicious-non-cryptographic-random-usage -- ok here in test code
         cls.merge_data = {
             "pk": [cls.dup_ip_1.pk, cls.dup_ip_2.pk, cls.dup_ip_3.pk],
             "host": cls.dup_ip_1.host,
@@ -1007,133 +1074,36 @@ class IPAddressMergeTestCase(ModelViewTestCase):
                     self.assertEqual(set(associations), set(correct_associations))
 
 
-class IPAddressToInterfaceTestCase(ViewTestCases.BulkImportObjectsViewTestCase):
-    model = IPAddressToInterface
-
-    @classmethod
-    def setUpTestData(cls):
-        # Device/Interface
-        prefix_status = Status.objects.get_for_model(Prefix).first()
-        location = Location.objects.filter(parent__isnull=False, parent__parent__isnull=False).first()
-        manufacturer = Manufacturer.objects.first()
-        tenant = Tenant.objects.first()
-        devicetype = DeviceType.objects.create(manufacturer=manufacturer, model="Device Type 1")
-        devicerole = Role.objects.get_for_model(Device).first()
-        devicestatus = Status.objects.get_for_model(Device).first()
-        device = Device.objects.create(
-            name="Device 1",
-            location=location,
-            device_type=devicetype,
-            role=devicerole,
-            tenant=tenant,
-            status=devicestatus,
-        )
-        intf_status = Status.objects.get_for_model(Interface).first()
-        interface = Interface.objects.create(device=device, name="Interface 1", status=intf_status)
-
-        # Namespace, Prefix, IPAddress
-        namespace = Namespace.objects.create(name="ip2interface_namespace")
-        ip_status = Status.objects.get_for_model(IPAddress).first()
-        prefix_status = Status.objects.get_for_model(Prefix).first()
-        Prefix.objects.get_or_create(
-            prefix="192.0.2.0/24",
-            defaults={"namespace": namespace, "status": prefix_status, "type": "network"},
-        )
-        ip_addresses = [
-            IPAddress.objects.create(
-                address="192.0.2.1/32", status=ip_status, type=IPAddressTypeChoices.TYPE_DHCP, namespace=namespace
-            ),
-            IPAddress.objects.create(
-                address="192.0.2.2/32", status=ip_status, type=IPAddressTypeChoices.TYPE_DHCP, namespace=namespace
-            ),
-            IPAddress.objects.create(
-                address="192.0.2.3/32", status=ip_status, type=IPAddressTypeChoices.TYPE_DHCP, namespace=namespace
-            ),
-        ]
-        ip1, ip2, ip3 = ip_addresses
-
-        cls.csv_data = (
-            "ip_address__host,ip_address__parent__namespace__name,interface__name,interface__device__name,interface__device__tenant__name,interface__device__location__name,interface__device__location__parent__name,interface__device__location__parent__parent__name",
-            f'{ip1.host},{ip1.parent.namespace.name},{interface.name},{interface.device.name},"{interface.device.tenant.name}",{interface.device.location.name},{interface.device.location.parent.name},{interface.device.location.parent.parent.name}',
-            f'{ip2.host},{ip2.parent.namespace.name},{interface.name},{interface.device.name},"{interface.device.tenant.name}",{interface.device.location.name},{interface.device.location.parent.name},{interface.device.location.parent.parent.name}',
-            f'{ip3.host},{ip3.parent.namespace.name},{interface.name},{interface.device.name},"{interface.device.tenant.name}",{interface.device.location.name},{interface.device.location.parent.name},{interface.device.location.parent.parent.name}',
-        )
-
-    def test_vminterface_import(self):
-        """Explicitly tests that bulk import of VMInterface assignments also works."""
-        # Initialize any pre-existing assignments. There is no mercy in this dojo!!
-        self._get_queryset().all().delete()
-
-        # Namespace, Prefix, IPAddress
-        namespace = Namespace.objects.create(name="ip2vminterface_namespace")
-        ip_status = Status.objects.get_for_model(IPAddress).first()
-        prefix_status = Status.objects.get_for_model(Prefix).first()
-        Prefix.objects.get_or_create(
-            prefix="192.0.3.0/24",
-            defaults={"namespace": namespace, "status": prefix_status, "type": "network"},
-        )
-        ip_addresses = [
-            IPAddress.objects.create(
-                address="192.0.3.1/32", status=ip_status, type=IPAddressTypeChoices.TYPE_DHCP, namespace=namespace
-            ),
-            IPAddress.objects.create(
-                address="192.0.3.2/32", status=ip_status, type=IPAddressTypeChoices.TYPE_DHCP, namespace=namespace
-            ),
-            IPAddress.objects.create(
-                address="192.0.3.3/32", status=ip_status, type=IPAddressTypeChoices.TYPE_DHCP, namespace=namespace
-            ),
-        ]
-        ip1, ip2, ip3 = ip_addresses
-
-        # VirtualMachine/VMInterface
-        location = Location.objects.filter(parent__isnull=False, parent__parent__isnull=False).first()
-        cluster_type = ClusterType.objects.create(name="Cluster Type 2")
-        cluster = Cluster.objects.create(name="Cluster 1", cluster_type=cluster_type, location=location)
-        vm_status = Status.objects.get_for_model(VirtualMachine).first()
-        tenant = Tenant.objects.first()
-        virtual_machine = VirtualMachine.objects.create(cluster=cluster, name="VM 1", status=vm_status, tenant=tenant)
-        vm_intf_status = Status.objects.get_for_model(VMInterface).first()
-        vm_interface = VMInterface.objects.create(
-            virtual_machine=virtual_machine, name="VMInterface 1", status=vm_intf_status
-        )
-
-        csv_data = (
-            "ip_address__host,ip_address__parent__namespace__name,vm_interface__virtual_machine__cluster__name,vm_interface__virtual_machine__tenant__name,vm_interface__virtual_machine__name,vm_interface__name",
-            f'{ip1.host},{ip1.parent.namespace.name},{vm_interface.virtual_machine.cluster.name},"{vm_interface.virtual_machine.tenant.name}",{vm_interface.virtual_machine.name},{vm_interface.name}',
-            f'{ip2.host},{ip2.parent.namespace.name},{vm_interface.virtual_machine.cluster.name},"{vm_interface.virtual_machine.tenant.name}",{vm_interface.virtual_machine.name},{vm_interface.name}',
-            f'{ip3.host},{ip3.parent.namespace.name},{vm_interface.virtual_machine.cluster.name},"{vm_interface.virtual_machine.tenant.name}",{vm_interface.virtual_machine.name},{vm_interface.name}',
-        )
-
-        initial_count = self._get_queryset().count()
-        data = {
-            "csv_data": "\n".join(csv_data),
-        }
-
-        self.add_permissions("ipam.add_ipaddresstointerface")
-        self.assertHttpStatus(self.client.post(self._get_url("import"), data), 200)
-        self.assertEqual(self._get_queryset().count(), initial_count + len(csv_data) - 1)
-
-
-class VLANGroupTestCase(ViewTestCases.OrganizationalObjectViewTestCase):
+class VLANGroupTestCase(
+    ViewTestCases.OrganizationalObjectViewTestCase,
+    ViewTestCases.BulkEditObjectsViewTestCase,
+):
     model = VLANGroup
 
     @classmethod
     def setUpTestData(cls):
         location = Location.objects.filter(location_type=LocationType.objects.get(name="Campus")).first()
+        location_2 = Location.objects.filter(location_type=LocationType.objects.get(name="Building")).first()
 
         cls.form_data = {
             "name": "VLAN Group X",
             "location": location.pk,
             "description": "A new VLAN group",
+            "range": "1-4094",
+            "tags": [t.pk for t in Tag.objects.get_for_model(VLANGroup)],
         }
 
-        cls.csv_data = (
-            "name,description",
-            "VLAN Group 4,Fourth VLAN group",
-            "VLAN Group 5,Fifth VLAN group",
-            "VLAN Group 6,Sixth VLAN group",
-            "VLAN Group 7,Seventh VLAN group",
-        )
+        cls.bulk_edit_data = {
+            "location": location_2.pk,
+            "description": "Updated description for bulk edit",
+            "range": "1-4094",
+        }
+
+    def get_deletable_object(self):
+        return VLANGroup.objects.create(name="TEST DELETE ME")
+
+    def get_deletable_object_pks(self):
+        return [VLANGroup.objects.create(name="TEST DELETE ME").pk]
 
 
 class VLANTestCase(ViewTestCases.PrimaryObjectViewTestCase):
@@ -1142,80 +1112,51 @@ class VLANTestCase(ViewTestCases.PrimaryObjectViewTestCase):
     @classmethod
     def setUpTestData(cls):
         cls.locations = Location.objects.filter(location_type=LocationType.objects.get(name="Campus"))
-        location_1 = cls.locations.first()
 
-        vlangroups = (
+        cls.vlangroups = (
             VLANGroup.objects.create(name="VLAN Group 1", location=cls.locations.first()),
             VLANGroup.objects.create(name="VLAN Group 2", location=cls.locations.last()),
         )
 
         roles = Role.objects.get_for_model(VLAN)[:2]
 
-        statuses = Status.objects.get_for_model(VLAN)
-        status_1 = statuses[0]
-        status_2 = statuses[1]
-
-        VLAN.objects.create(
-            vlan_group=vlangroups[0],
-            vid=101,
-            name="VLAN101",
-            location=location_1,
-            role=roles[0],
-            status=status_1,
-            _custom_field_data={"custom_field": "Value"},
-        )
-        VLAN.objects.create(
-            vlan_group=vlangroups[0],
-            vid=102,
-            name="VLAN102",
-            location=location_1,
-            role=roles[0],
-            status=status_1,
-            _custom_field_data={"custom_field": "Value"},
-        )
-        VLAN.objects.create(
-            vlan_group=vlangroups[0],
-            vid=103,
-            name="VLAN103",
-            location=location_1,
-            role=roles[0],
-            status=status_1,
-            _custom_field_data={"custom_field": "Value"},
-        )
-
-        custom_field = CustomField.objects.create(
-            type=CustomFieldTypeChoices.TYPE_TEXT, label="Custom Field", default=""
-        )
-        custom_field.content_types.set([ContentType.objects.get_for_model(VLAN)])
+        status = Status.objects.get_for_model(VLAN).first()
 
         cls.form_data = {
-            "location": cls.locations.last().pk,
-            "vlan_group": vlangroups[1].pk,
+            "vlan_group": cls.vlangroups[0].pk,
             "vid": 999,
             "name": "VLAN999 with an unwieldy long name since we increased the limit to more than 64 characters",
             "tenant": None,
-            "status": status_2.pk,
+            "status": status.pk,
             "role": roles[1].pk,
+            "locations": list(cls.locations.values_list("pk", flat=True)[:1]),
             "description": "A new VLAN",
             "tags": [t.pk for t in Tag.objects.get_for_model(VLAN)],
         }
 
-        cls.csv_data = (
-            "vid,name,status,vlan_group",
-            f"104,VLAN104,{status_1.name},{vlangroups[0].composite_key}",
-            f"105,VLAN105,{status_1.name},{vlangroups[0].composite_key}",
-            f"106,VLAN106,{status_1.name},{vlangroups[0].composite_key}",
-        )
-
         cls.bulk_edit_data = {
-            "location": cls.locations.first().pk,
-            "vlan_group": vlangroups[0].pk,
+            "vlan_group": cls.vlangroups[0].pk,
             "tenant": Tenant.objects.first().pk,
-            "status": status_2.pk,
+            "status": status.pk,
             "role": roles[0].pk,
             "description": "New description",
         }
 
+    @override_settings(EXEMPT_VIEW_PERMISSIONS=["*"])
+    def test_vlan_group_not_belong_to_vlan_locations(self):
+        """Test that a VLAN cannot be assigned to a VLAN Group that is not in the same location as the VLAN."""
+        vlan_group = self.vlangroups[0]
+        form_data = self.form_data.copy()
+        form_data["vlan_group"] = vlan_group.pk
+        form_data["locations"] = [self.locations.last().pk]
+        self.add_permissions("ipam.add_vlan")
+        request = {
+            "path": self._get_url("add"),
+            "data": post_data(form_data),
+        }
+        response = self.client.post(**request)
+        self.assertBodyContains(response, f"vlan_group: VLAN Group {vlan_group} is not in locations")
+
 
 class ServiceTestCase(ViewTestCases.PrimaryObjectViewTestCase):
     model = Service
@@ -1268,13 +1209,6 @@ class ServiceTestCase(ViewTestCases.PrimaryObjectViewTestCase):
             "tags": [t.pk for t in Tag.objects.get_for_model(Service)],
         }
 
-        cls.csv_data = (
-            "device,name,protocol,ports,description",
-            f"{cls.device.composite_key},Service 4,tcp,1,First service",
-            f"{cls.device.composite_key},Service 5,tcp,2,Second service",
-            f'{cls.device.composite_key},Service 6,udp,"3,4,5",Third service',
-        )
-
         cls.bulk_edit_data = {
             "protocol": ServiceProtocolChoices.PROTOCOL_UDP,
             "ports": "106,107",
@@ -1299,9 +1233,7 @@ class ServiceTestCase(ViewTestCases.PrimaryObjectViewTestCase):
             "data": post_data(self.form_data),
         }
         response = self.client.post(**request)
-        self.assertHttpStatus(response, 200)
-        response_body = extract_page_body(response.content.decode(response.charset))
-        self.assertIn("Service with this Name and Device already exists.", response_body)
+        self.assertBodyContains(response, "Service with this Name and Device already exists.")
 
     @override_settings(EXEMPT_VIEW_PERMISSIONS=["*"])
     def test_service_cannot_be_assigned_to_both_device_and_vm(self):
@@ -1321,9 +1253,7 @@ class ServiceTestCase(ViewTestCases.PrimaryObjectViewTestCase):
             "data": post_data(self.form_data),
         }
         response = self.client.post(**request)
-        self.assertHttpStatus(response, 200)
-        response_body = extract_page_body(response.content.decode(response.charset))
-        self.assertIn("A service cannot be associated with both a device and a virtual machine.", response_body)
+        self.assertBodyContains(response, "A service cannot be associated with both a device and a virtual machine.")
 
     @override_settings(EXEMPT_VIEW_PERMISSIONS=["*"])
     def test_service_cannot_be_assigned_to_neither_device_nor_vm(self):
@@ -1343,6 +1273,26 @@ class ServiceTestCase(ViewTestCases.PrimaryObjectViewTestCase):
             "data": post_data(self.form_data),
         }
         response = self.client.post(**request)
+        self.assertBodyContains(response, "A service must be associated with either a device or a virtual machine.")
+
+    @override_settings(EXEMPT_VIEW_PERMISSIONS=["*"])
+    def test_port_bulk_edit_invalid(self):
+        self.add_permissions("ipam.change_service")
+        url = self._get_url("bulk_edit")
+        pk_list = list(self._get_queryset().values_list("pk", flat=True)[:3])
+
+        data = {
+            "pk": pk_list,
+            "protocol": ServiceProtocolChoices.PROTOCOL_UDP,
+            "ports": "[106,107]",  # String representation of the list
+            "description": "New description",
+            "_apply": True,
+        }
+
+        response = self.client.post(url, data)
+        response_content = response.content.decode(response.charset)
         self.assertHttpStatus(response, 200)
-        response_body = extract_page_body(response.content.decode(response.charset))
-        self.assertIn("A service must be associated with either a device or a virtual machine.", response_body)
+        self.assertInHTML(
+            ' <strong class="panel-title">Ports</strong>: <ul class="errorlist"><li>invalid literal for int() with base 10: &#x27;[106&#x27;</li></ul>',
+            response_content,
+        )
