@@ -2,4 +2,6 @@ from nautobot.core.apps import NautobotConfig
 
 
 class TenancyConfig(NautobotConfig):
+    default = True
     name = "nautobot.tenancy"
+    searchable_models = ["tenant"]
