@@ -1,13 +1,14 @@
 import factory
 
+from nautobot.core.constants import CHARFIELD_MAX_LENGTH
 from nautobot.core.factory import (
     NautobotBoolIterator,
     OrganizationalModelFactory,
     PrimaryModelFactory,
-    UniqueFaker,
     random_instance,
+    UniqueFaker,
 )
-from nautobot.tenancy.models import TenantGroup, Tenant
+from nautobot.tenancy.models import Tenant, TenantGroup
 
 
 class TenantGroupFactory(OrganizationalModelFactory):
@@ -21,7 +22,7 @@ class TenantGroupFactory(OrganizationalModelFactory):
     name = UniqueFaker("company")
 
     has_description = NautobotBoolIterator()
-    description = factory.Maybe("has_description", factory.Faker("text", max_nb_chars=200), "")
+    description = factory.Maybe("has_description", factory.Faker("text", max_nb_chars=CHARFIELD_MAX_LENGTH), "")
 
     has_parent = NautobotBoolIterator()
     parent = factory.Maybe("has_parent", random_instance(TenantGroup), None)
@@ -42,7 +43,7 @@ class TenantFactory(PrimaryModelFactory):
     comments = factory.Maybe("has_comments", factory.Faker("paragraph"), "")
 
     has_description = NautobotBoolIterator()
-    description = factory.Maybe("has_description", factory.Faker("text", max_nb_chars=200), "")
+    description = factory.Maybe("has_description", factory.Faker("text", max_nb_chars=CHARFIELD_MAX_LENGTH), "")
 
-    has_tenant_group = NautobotBoolIterator()
+    has_tenant_group = NautobotBoolIterator(chance_of_getting_true=90)
     tenant_group = factory.Maybe("has_tenant_group", random_instance(TenantGroup), None)
