@@ -4,78 +4,53 @@ from django.db.models import Prefetch
 from django.shortcuts import get_object_or_404, redirect, render
 from django.urls import reverse
 from django.utils.functional import cached_property
-from django_tables2 import RequestConfig
 
-from nautobot.core.models.querysets import count_related
+from nautobot.core.choices import ButtonActionColorChoices
+from nautobot.core.ui import object_detail
+from nautobot.core.ui.choices import SectionChoices
 from nautobot.core.utils.requests import normalize_querydict
 from nautobot.core.views import generic
-from nautobot.core.views.paginator import EnhancedPaginator, get_paginate_count
+from nautobot.core.views.viewsets import NautobotUIViewSet
 from nautobot.dcim.models import Device
 from nautobot.dcim.tables import DeviceTable
 from nautobot.extras.views import ObjectConfigContextView
 from nautobot.ipam.models import IPAddress, Service
 from nautobot.ipam.tables import InterfaceIPAddressTable, InterfaceVLANTable, VRFDeviceAssignmentTable
+from nautobot.virtualization.api import serializers
+
 from . import filters, forms, tables
 from .models import Cluster, ClusterGroup, ClusterType, VirtualMachine, VMInterface
 
-
 #
 # Cluster types
 #
 
 
-class ClusterTypeListView(generic.ObjectListView):
-    queryset = ClusterType.objects.annotate(cluster_count=count_related(Cluster, "cluster_type"))
-    filterset = filters.ClusterTypeFilterSet
-    table = tables.ClusterTypeTable
-
-
-class ClusterTypeView(generic.ObjectView):
+class ClusterTypeUIViewSet(NautobotUIViewSet):
+    bulk_update_form_class = forms.ClusterTypeBulkEditForm
+    filterset_class = filters.ClusterTypeFilterSet
+    filterset_form_class = forms.ClusterTypeFilterForm
+    form_class = forms.ClusterTypeForm
+    serializer_class = serializers.ClusterTypeSerializer
+    table_class = tables.ClusterTypeTable
     queryset = ClusterType.objects.all()
 
-    def get_extra_context(self, request, instance):
-        # Clusters
-        clusters = (
-            Cluster.objects.restrict(request.user, "view")
-            .filter(cluster_type=instance)
-            .select_related("cluster_group", "location", "tenant")
-        ).annotate(
-            device_count=count_related(Device, "cluster"),
-            vm_count=count_related(VirtualMachine, "cluster"),
+    object_detail_content = object_detail.ObjectDetailContent(
+        panels=(
+            object_detail.ObjectFieldsPanel(
+                weight=100,
+                section=SectionChoices.LEFT_HALF,
+                fields="__all__",
+            ),
+            object_detail.ObjectsTablePanel(
+                weight=100,
+                section=SectionChoices.RIGHT_HALF,
+                table_class=tables.ClusterTable,
+                table_filter="cluster_type",
+                exclude_columns=["cluster_type"],
+            ),
         )
-
-        cluster_table = tables.ClusterTable(clusters)
-        cluster_table.columns.hide("cluster_type")
-
-        paginate = {
-            "paginator_class": EnhancedPaginator,
-            "per_page": get_paginate_count(request),
-        }
-        RequestConfig(request, paginate).configure(cluster_table)
-
-        return {
-            "cluster_table": cluster_table,
-        }
-
-
-class ClusterTypeEditView(generic.ObjectEditView):
-    queryset = ClusterType.objects.all()
-    model_form = forms.ClusterTypeForm
-
-
-class ClusterTypeDeleteView(generic.ObjectDeleteView):
-    queryset = ClusterType.objects.all()
-
-
-class ClusterTypeBulkImportView(generic.BulkImportView):
-    queryset = ClusterType.objects.all()
-    table = tables.ClusterTypeTable
-
-
-class ClusterTypeBulkDeleteView(generic.BulkDeleteView):
-    queryset = ClusterType.objects.annotate(cluster_count=count_related(Cluster, "cluster_type"))
-    table = tables.ClusterTypeTable
-    filterset = filters.ClusterTypeFilterSet
+    )
 
 
 #
@@ -83,58 +58,31 @@ class ClusterTypeBulkDeleteView(generic.BulkDeleteView):
 #
 
 
-class ClusterGroupListView(generic.ObjectListView):
-    queryset = ClusterGroup.objects.annotate(cluster_count=count_related(Cluster, "cluster_group"))
-    filterset = filters.ClusterGroupFilterSet
-    table = tables.ClusterGroupTable
-
-
-class ClusterGroupView(generic.ObjectView):
+class ClusterGroupUIViewSet(NautobotUIViewSet):
+    bulk_update_form_class = forms.ClusterGroupBulkEditForm
+    filterset_class = filters.ClusterGroupFilterSet
+    filterset_form_class = forms.ClusterGroupFilterForm
+    form_class = forms.ClusterGroupForm
+    serializer_class = serializers.ClusterGroupSerializer
+    table_class = tables.ClusterGroupTable
     queryset = ClusterGroup.objects.all()
 
-    def get_extra_context(self, request, instance):
-        # Clusters
-        clusters = (
-            Cluster.objects.restrict(request.user, "view")
-            .filter(cluster_group=instance)
-            .select_related("cluster_type", "location", "tenant")
-        ).annotate(
-            device_count=count_related(Device, "cluster"),
-            vm_count=count_related(VirtualMachine, "cluster"),
+    object_detail_content = object_detail.ObjectDetailContent(
+        panels=(
+            object_detail.ObjectFieldsPanel(
+                weight=100,
+                section=SectionChoices.LEFT_HALF,
+                fields="__all__",
+            ),
+            object_detail.ObjectsTablePanel(
+                weight=100,
+                section=SectionChoices.RIGHT_HALF,
+                table_class=tables.ClusterTable,
+                table_filter="cluster_group",
+                exclude_columns=["cluster_group"],
+            ),
         )
-
-        cluster_table = tables.ClusterTable(clusters)
-        cluster_table.columns.hide("cluster_group")
-
-        paginate = {
-            "paginator_class": EnhancedPaginator,
-            "per_page": get_paginate_count(request),
-        }
-        RequestConfig(request, paginate).configure(cluster_table)
-
-        return {
-            "cluster_table": cluster_table,
-        }
-
-
-class ClusterGroupEditView(generic.ObjectEditView):
-    queryset = ClusterGroup.objects.all()
-    model_form = forms.ClusterGroupForm
-
-
-class ClusterGroupDeleteView(generic.ObjectDeleteView):
-    queryset = ClusterGroup.objects.all()
-
-
-class ClusterGroupBulkImportView(generic.BulkImportView):
-    queryset = ClusterGroup.objects.all()
-    table = tables.ClusterGroupTable
-
-
-class ClusterGroupBulkDeleteView(generic.BulkDeleteView):
-    queryset = ClusterGroup.objects.annotate(cluster_count=count_related(Cluster, "cluster_group"))
-    table = tables.ClusterGroupTable
-    filterset = filters.ClusterGroupFilterSet
+    )
 
 
 #
@@ -142,61 +90,60 @@ class ClusterGroupBulkDeleteView(generic.BulkDeleteView):
 #
 
 
-class ClusterListView(generic.ObjectListView):
-    permission_required = "virtualization.view_cluster"
-    queryset = Cluster.objects.select_related("cluster_type", "cluster_group").annotate(
-        device_count=count_related(Device, "cluster"),
-        vm_count=count_related(VirtualMachine, "cluster"),
-    )
-    table = tables.ClusterTable
-    filterset = filters.ClusterFilterSet
-    filterset_form = forms.ClusterFilterForm
-
-
-class ClusterView(generic.ObjectView):
+class ClusterUIViewSet(NautobotUIViewSet):
+    bulk_update_form_class = forms.ClusterBulkEditForm
+    filterset_class = filters.ClusterFilterSet
+    filterset_form_class = forms.ClusterFilterForm
+    form_class = forms.ClusterForm
+    serializer_class = serializers.ClusterSerializer
+    table_class = tables.ClusterTable
     queryset = Cluster.objects.all()
 
-    def get_extra_context(self, request, instance):
-        devices = (
-            Device.objects.restrict(request.user, "view")
-            .filter(cluster=instance)
-            .select_related("location", "rack", "tenant", "device_type__manufacturer")
+    object_detail_content = object_detail.ObjectDetailContent(
+        panels=(
+            object_detail.ObjectFieldsPanel(
+                weight=100,
+                section=SectionChoices.LEFT_HALF,
+                fields="__all__",
+            ),
+            object_detail.ObjectsTablePanel(
+                weight=100,
+                section=SectionChoices.RIGHT_HALF,
+                table_class=DeviceTable,
+                table_filter="cluster",
+                table_title="Host Devices",
+                enable_bulk_actions=True,
+                add_button_route=None,
+                form_id="device_form",
+                footer_buttons=[
+                    object_detail.FormButton(
+                        form_id="device_form",
+                        link_name="virtualization:cluster_remove_devices",
+                        label="Remove Devices",
+                        weight=100,
+                        color=ButtonActionColorChoices.DELETE,
+                        icon="mdi-trash-can-outline",
+                        size="xs",
+                    ),
+                    object_detail.Button(
+                        link_name="virtualization:cluster_add_devices",
+                        label="Add Devices",
+                        weight=200,
+                        color=ButtonActionColorChoices.ADD,
+                        icon="mdi-plus",
+                        size="xs",
+                    ),
+                ],
+            ),
+            object_detail.ObjectsTablePanel(
+                weight=100,
+                section=SectionChoices.FULL_WIDTH,
+                table_class=tables.VirtualMachineTable,
+                table_filter="cluster",
+                add_button_route=None,
+            ),
         )
-        device_table = DeviceTable(list(devices), orderable=False)
-        if request.user.has_perm("virtualization.change_cluster"):
-            device_table.columns.show("pk")
-
-        return {
-            "device_table": device_table,
-        }
-
-
-class ClusterEditView(generic.ObjectEditView):
-    template_name = "virtualization/cluster_edit.html"
-    queryset = Cluster.objects.all()
-    model_form = forms.ClusterForm
-
-
-class ClusterDeleteView(generic.ObjectDeleteView):
-    queryset = Cluster.objects.all()
-
-
-class ClusterBulkImportView(generic.BulkImportView):
-    queryset = Cluster.objects.all()
-    table = tables.ClusterTable
-
-
-class ClusterBulkEditView(generic.BulkEditView):
-    queryset = Cluster.objects.select_related("cluster_type", "cluster_group", "location")
-    filterset = filters.ClusterFilterSet
-    table = tables.ClusterTable
-    form = forms.ClusterBulkEditForm
-
-
-class ClusterBulkDeleteView(generic.BulkDeleteView):
-    queryset = Cluster.objects.select_related("cluster_type", "cluster_group", "location")
-    filterset = filters.ClusterFilterSet
-    table = tables.ClusterTable
+    )
 
 
 class ClusterAddDevicesView(generic.ObjectEditView):
@@ -295,47 +242,58 @@ class ClusterRemoveDevicesView(generic.ObjectEditView):
 #
 
 
-class VirtualMachineListView(generic.ObjectListView):
-    queryset = VirtualMachine.objects.select_related("cluster", "tenant", "platform", "primary_ip4", "primary_ip6")
-    filterset = filters.VirtualMachineFilterSet
-    filterset_form = forms.VirtualMachineFilterForm
-    table = tables.VirtualMachineDetailTable
-    template_name = "virtualization/virtualmachine_list.html"
-
-
-class VirtualMachineView(generic.ObjectView):
+class VirtualMachineUIViewSet(NautobotUIViewSet):
+    bulk_update_form_class = forms.VirtualMachineBulkEditForm
+    filterset_class = filters.VirtualMachineFilterSet
+    filterset_form_class = forms.VirtualMachineFilterForm
+    form_class = forms.VirtualMachineForm
+    serializer_class = serializers.VirtualMachineSerializer
+    table_class = tables.VirtualMachineDetailTable
     queryset = VirtualMachine.objects.select_related("tenant__tenant_group")
 
     def get_extra_context(self, request, instance):
-        # Interfaces
-        vminterfaces = (
-            VMInterface.objects.restrict(request.user, "view")
-            .filter(virtual_machine=instance)
-            .prefetch_related(Prefetch("ip_addresses", queryset=IPAddress.objects.restrict(request.user)))
-        )
-        vminterface_table = tables.VirtualMachineVMInterfaceTable(vminterfaces, user=request.user, orderable=False)
-        if request.user.has_perm("virtualization.change_vminterface") or request.user.has_perm(
-            "virtualization.delete_vminterface"
-        ):
-            vminterface_table.columns.show("pk")
+        context = super().get_extra_context(request, instance)
 
-        # Services
-        services = (
-            Service.objects.restrict(request.user, "view")
-            .filter(virtual_machine=instance)
-            .prefetch_related(Prefetch("ip_addresses", queryset=IPAddress.objects.restrict(request.user)))
-        )
+        if self.action == "retrieve":
+            # Interfaces
+            vminterfaces = (
+                VMInterface.objects.restrict(request.user, "view")
+                .filter(virtual_machine=instance)
+                .prefetch_related(Prefetch("ip_addresses", queryset=IPAddress.objects.restrict(request.user)))
+            )
+            vminterface_table = tables.VirtualMachineVMInterfaceTable(vminterfaces, user=request.user, orderable=False)
+            if request.user.has_perm("virtualization.change_vminterface") or request.user.has_perm(
+                "virtualization.delete_vminterface"
+            ):
+                vminterface_table.columns.show("pk")
 
-        # VRF assignments
-        vrf_assignments = instance.vrf_assignments.restrict(request.user, "view")
-        vrf_table = VRFDeviceAssignmentTable(vrf_assignments)
-        vrf_table.exclude = ("device", "virtual_machine")
+            # Services
+            services = (
+                Service.objects.restrict(request.user, "view")
+                .filter(virtual_machine=instance)
+                .prefetch_related(Prefetch("ip_addresses", queryset=IPAddress.objects.restrict(request.user)))
+            )
 
-        return {
-            "vminterface_table": vminterface_table,
-            "services": services,
-            "vrf_table": vrf_table,
-        }
+            # VRF assignments
+            vrf_assignments = instance.vrf_assignments.restrict(request.user, "view")
+            vrf_table = VRFDeviceAssignmentTable(vrf_assignments)
+
+            # Software images
+            if instance.software_version is not None:
+                software_version_images = instance.software_version.software_image_files.restrict(request.user, "view")
+            else:
+                software_version_images = []
+
+            context.update(
+                {
+                    "vminterface_table": vminterface_table,
+                    "services": services,
+                    "software_version_images": software_version_images,
+                    "vrf_table": vrf_table,
+                }
+            )
+
+        return context
 
 
 class VirtualMachineConfigContextView(ObjectConfigContextView):
@@ -349,34 +307,6 @@ class VirtualMachineConfigContextView(ObjectConfigContextView):
         return VirtualMachine.objects.annotate_config_context_data()
 
 
-class VirtualMachineEditView(generic.ObjectEditView):
-    queryset = VirtualMachine.objects.all()
-    model_form = forms.VirtualMachineForm
-    template_name = "virtualization/virtualmachine_edit.html"
-
-
-class VirtualMachineDeleteView(generic.ObjectDeleteView):
-    queryset = VirtualMachine.objects.all()
-
-
-class VirtualMachineBulkImportView(generic.BulkImportView):
-    queryset = VirtualMachine.objects.all()
-    table = tables.VirtualMachineTable
-
-
-class VirtualMachineBulkEditView(generic.BulkEditView):
-    queryset = VirtualMachine.objects.select_related("cluster", "role", "status", "tenant")
-    filterset = filters.VirtualMachineFilterSet
-    table = tables.VirtualMachineTable
-    form = forms.VirtualMachineBulkEditForm
-
-
-class VirtualMachineBulkDeleteView(generic.BulkDeleteView):
-    queryset = VirtualMachine.objects.select_related("cluster", "role", "status", "tenant")
-    filterset = filters.VirtualMachineFilterSet
-    table = tables.VirtualMachineTable
-
-
 #
 # VM interfaces
 #
@@ -412,9 +342,7 @@ class VMInterfaceView(generic.ObjectView):
             vlans.append(instance.untagged_vlan)
             vlans[0].tagged = False
 
-        for vlan in instance.tagged_vlans.restrict(request.user).select_related(
-            "location", "vlan_group", "tenant", "role"
-        ):
+        for vlan in instance.tagged_vlans.restrict(request.user).select_related("vlan_group", "tenant", "role"):
             vlan.tagged = True
             vlans.append(vlan)
         vlan_table = InterfaceVLANTable(interface=instance, data=vlans, orderable=False)
@@ -423,10 +351,10 @@ class VMInterfaceView(generic.ObjectView):
             "ipaddress_table": ipaddress_table,
             "child_interfaces_table": child_interfaces_tables,
             "vlan_table": vlan_table,
+            **super().get_extra_context(request, instance),
         }
 
 
-# 2.0 TODO: This will be collapsed into `InterfaceCreateView` and will go away.
 class VMInterfaceCreateView(generic.ComponentCreateView):
     queryset = VMInterface.objects.all()
     form = forms.VMInterfaceCreateForm
@@ -445,7 +373,7 @@ class VMInterfaceDeleteView(generic.ObjectDeleteView):
     template_name = "virtualization/virtual_machine_vminterface_delete.html"
 
 
-class VMInterfaceBulkImportView(generic.BulkImportView):
+class VMInterfaceBulkImportView(generic.BulkImportView):  # 3.0 TODO: remove, unused
     queryset = VMInterface.objects.all()
     table = tables.VMInterfaceTable
 
